(* Part of Fail-Safe C Compiler.
   Produced by Yutaka Oiwa.
   (c) 2005- AIST. *)

open Util
open Glist
open Linker_types

let system_path_ref = ref ""

let system_path () =
  if !system_path_ref = "" then
    failwith "panic: system_path not specified"
  else
    !system_path_ref

let re_separator = Str.regexp "\\(:\\|\t\t*\\)"
let re_line_separator = Str.regexp "\n"

type struct_declaration_info = 
    Sabstract
  | Sspecial
  | Sconcrete of (string * ltype) list

type module_info = {
    mi_structs : (struct_desc * string * struct_declaration_info) list;
    mi_required_typeinfo : ltype list;
    mi_required_values : (string * ltype) list;
    mi_provided_values : (string * ltype) list
  }

let parse_lines strs = 
  let struct_decls = Glist.empty () in
  let required_typeinfo = Glist.empty () in
  let required_value = Glist.empty () in
  let provided_value = Glist.empty () in
  List.iter
    (fun line ->
      match Str.split re_separator line with
	["R"; "1"] -> ()
      | "R" :: _ -> failwith "bad revision for type information record; possibly compiler version incompatibility"
      | ["T"; enc_type; base_name; "c"; str_decl] ->
	  Glist.put struct_decls 
	    (parse_enctype enc_type,
	     base_name,
	     Sconcrete (parse_structfields str_decl))
      | ["T"; enc_type; base_name; "a"] ->
	  Glist.put struct_decls (parse_enctype enc_type, base_name, Sabstract)
      | ["T"; enc_type; base_name; "s"] ->
	  Glist.put struct_decls (parse_enctype enc_type, base_name, Sspecial)
      | ["T"; enctype] ->
	  Glist.put required_typeinfo (parse_enctype enctype);
      | ["E"; value_name; enctype] ->
	  Glist.put required_value (value_name, parse_enctype enctype);
      | ["D"; value_name; enctype] ->
	  Glist.put provided_value (value_name, parse_enctype enctype)
      | _ ->
	  failwith ("unknown/invalid type information: \"" ^ line ^ "\"")
    ) strs;
  { mi_structs = 
    list_map
      (function (LTstruct t, b, d) -> t, b, d
	| _ -> failwith "parse error: non-struct struct declaration")
	(Glist.to_list struct_decls);
    mi_required_typeinfo = Glist.to_list required_typeinfo;
    mi_required_values = Glist.to_list required_value;
    mi_provided_values = Glist.to_list provided_value
  }

let read_string s = 
  let s = 
    (* strip final NUL *)
    if s.[String.length s - 1] = '\000' then 
      String.sub s 0 (String.length s - 1)
    else s
  in
  parse_lines (Str.split re_line_separator s)

let read_stream s =
  let r = Glist.empty () in
  begin
    try
      while true do
	let l = input_line s in
	let l = if l <> "" && Str.last_chars l 1 = "\000" then
	  String.sub l 0 (String.length l - 1) else l in
	let l = if l <> "" && Str.last_chars l 1 = "\n" then
	  String.sub l 0 (String.length l - 1) else l in
	if l <> "" && l <> "\n" then
	  Glist.put r l;
      done
    with
      End_of_file -> ()
  end;
  parse_lines (Glist.to_list r)
	
let parse_compile_unit file = 
  let command = system_path () ^ "/tools/read-elf-section" in
  let commandline = command ^ " -S __failsafeC_typeinfo " ^
    Filename.quote file in
  let in_proc = Unix.open_process_in commandline in
  let r = read_stream in_proc in
  if Unix.close_process_in in_proc <> Unix.WEXITED 0 then 
    failwith ("Reading elf section failed:")
  else
    r

let parse_unit_manifest file = 
  let i = open_in file in
  let r = read_stream i in
  close_in i;
  r

