(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2008 AIST.

   This file is written by Yutaka Oiwa in 2008. *)

(** Parses module type information embedded in compiled module objects *)

val system_path_ref : string ref
val system_path : unit -> string
(** Returns the path of Fail-Safe C compiler *)

(** Forms of struct declaration in the module. *)
type struct_declaration_info =
    Sabstract
  | Sspecial
  | Sconcrete of (string * Linker_types.ltype) list

type module_info = {
  mi_structs :
    (Linker_types.struct_desc * string * struct_declaration_info) list;
    (** declared structs *)
  mi_required_typeinfo : Linker_types.ltype list;
    (** typeinfo blocks required in the module code *)
  mi_required_values : (string * Linker_types.ltype) list;
    (** values required by the module code *)
  mi_provided_values : (string * Linker_types.ltype) list;
    (** values provided by the module code *)
}
(** the parsed information of the module object *)

val read_string : string -> module_info
(** reads the module information from a string. *)
val parse_compile_unit : string -> module_info
(** reads the module information from a compiled module. *)
val parse_unit_manifest : string -> module_info
(** reads the module information from a manifest text (used for standard library). *)
