(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2007 AIST.

   This file is written by Yutaka Oiwa in 2002-2006. *)

(** IL1: intermediate language with basic blocks *)

open Big_int
open Ctt_abstree
open Cttm_abstree
open Set_list

type temp_id = int

type identifier = string

and field = identifier * c_type

type il_switch_label = CASE of big_int | DEFAULT

type il_if_type = IFNOT | IFTRUE

type il_binop = 
    ILbinTimes | ILbinDiv
  | ILbinPlusVV | ILbinMinusVV 
  | ILbinPlusPV | ILbinMinusPP | ILbinMinusPV
  | ILbinModulo | ILbinLshift
  | ILbinRshift | ILbinLogAnd | ILbinLogOr | ILbinIntAnd | ILbinIntOr | ILbinIntXor
  | ILbinLessThan | ILbinLessEqual | ILbinGtrThan | ILbinGtrEqual
  | ILbinEqual | ILbinNotEqual

type il_lvalue = 
    ILlvPtr of temp_id
  | ILlvVar of identifier * c_type
  | ILlvTemp of temp_id

type il_expr =
  | ILexpCoerce of c_type * temp_id
  | ILexpConstant of c_constants
  | ILexpUndefined
  | ILexpBinop of il_binop * temp_id * temp_id
  | ILexpUnaryop of unaryop * temp_id
  | ILexpInvoke of il_lvalue * temp_id list
  | ILexpAddress of il_lvalue * field list
  | ILexpArgument of int
  | ILexpIdent of temp_id

type il_constant_exp_desc = 
    ILcexpCoerce of c_type * il_constant_exp
  | ILcexpConstant of c_constants
  | ILcexpBinop of il_binop * il_constant_exp * il_constant_exp
  | ILcexpUnaryop of unaryop * il_constant_exp
  | ILcexpAddress of il_lvalue * field list
  | ILcexpAddFieldOfs of il_constant_exp * field list

and il_constant_exp =
    { il_cexp_type : c_type; il_cexp_t : il_constant_exp_desc }

type il_initializer =
    ILinitConstantExp of c_type * il_constant_exp
  | ILinitPostFill
  | ILinitStruct of c_type * (identifier * il_initializer) list
  | ILinitArray of c_type * il_initializer list
  | ILinitAbstractCtt of Cttm_abstree.mexpr (* only for macro emit *)

type abort_reason = 
    ILabortNotReached
  | ILabortOthers of string

type il1_desc =
(* basically from statement *)
  | IL1stmtDeclAutoScalar of local_storage_class * c_type * identifier * temp_id option
  | IL1stmtDeclBulk of local_storage_class * c_type * identifier * il_initializer option
  | IL1stmtIf of il_if_type * temp_id * int
  | IL1stmtSwitch of temp_id * ( il_switch_label * int ) list
  | IL1stmtGoto of int
  | IL1stmtReturn of temp_id option
  | IL1stmtAbort of abort_reason
(* basically from expression *)
  | IL1stmtDefTemp of temp_id * c_type * il_expr
  | IL1stmtReadToTemp of temp_id * c_type * il_lvalue * field list
  | IL1stmtWrite of il_lvalue * field list * temp_id
  | IL1stmtSequence of il1 list
  | IL1stmtParallel of il1 list (* Data-flow parallel *)

and il1 = 
    { il1_depends : Int_set.t;
      il1_defines : Int_set.t;
      il1_t : il1_desc; }

type il1_basic_block =
    {
     predecessor : int list;
     successor : int list;
(*     immediate_dominator : int;*)
     code : il1 list
   }

type il1_function_body = il1_basic_block array

type il1_global_declaration = 
    IL1declFunction of global_storage_class * c_type 
	* identifier * identifier list * il1_function_body
  | IL1declVariable of 
      global_storage_class * c_type * identifier * il_initializer option

