(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-     AIST.

   This file is written by Yutaka Oiwa in 2006. *)

(** Type for terms attributed with a location in source files. *)

type location = string * int * int * bool
(** the type of locations in the source file: file-name, line, column, and whether it is came from trusted file. *)

type 'a t = private { locterm_v : 'a; locterm_loc : location; }
(** a term attributed with a location *)

val dummy_location : location
(** a dummy location not specifying any source location. *)

val locput : loc:location -> 'a -> 'a t
(** puts a location for a term. *)
val locput_dummy : 'a -> 'a t
(** puts the dummy location for a term. *)

val locval : 'a t -> 'a
(** gets a term inside a located-term. *)
val locget : 'a t -> location
(** gets a location from a located-term. *)

val loccopy : orig:'a t -> 'b -> 'b t
(** creates a new term which shares the same location as [orig]. *)

val locmap : (loc:location -> 'a -> 'b) -> 'a t -> 'b t
(** given a conversion function, creates a new converted term with the same location. *)
val locmap_list : (loc:location -> 'a -> 'b) -> 'a t list -> 'b t list
(** [locmap] combined with [List.map]. *)
val locmap_flatten : (loc:location -> 'a -> 'b list) -> 'a t list -> 'b t list
(** [locmap] combined with [Util.map_flatten]. *)
