(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2007 AIST.

   This file is written by Yutaka Oiwa in 2003-2005. *)

let read fname =
  C_parserhelper.reset fname;
  let chan = open_in fname in
  try
    let lexbuf = Lexing.from_channel chan in
    let t = C_parser.translation_unit C_lexer.token lexbuf in
    close_in chan;
    t
  with
    e -> close_in chan; raise e
	
let read_chan chan =
  C_parserhelper.reset "";
  let lexbuf = Lexing.from_channel chan in
  C_parser.translation_unit C_lexer.token lexbuf

let check_types = ref false
let filter_tree = ref false

let print_global_declarations = 
  List.iter (fun x -> print_string (Ctt_printer.strof_global_declaration x))

let process fname = 
  let cmdline = Printf.sprintf ("cpp %s %s") Fsc_config.cpp_options fname in
  prerr_endline ("invoking " ^ cmdline ^ " ...");
  let chan = Unix.open_process_in cmdline in
  try
    prerr_endline "parsing...";
    let t = read_chan chan in
    if !check_types then begin
      prerr_endline "typing...";
      let e2, t2 = C_typing.parse_global_declarations t in
      let e2, t2 = 
	if !filter_tree then begin
	  let t = Expunge_localstatic.reduce_program ~genv:e2 t2 in
	  let genv, t = Ctt_pad_structures.translate_program e2 t in
	  let genv, t = Ctt_reduce_mdarray.reduce_program ~genv t in
	  let genv, t = Ctt_reduce_unions.translate_program ~genv t in
	  Ctt_add_tentative_initializer.reduce_program ~genv t
	end
	else
	  e2, t2
      in
      prerr_endline "printing...";
      print_global_declarations t2
    end
    else begin
      prerr_endline "printing...";
      C_pp.pp_print_program Format.std_formatter t
    end;
    prerr_endline "done."
  with
    e -> close_in chan; raise e

open Arg

let _ =
  if not !Sys.interactive then begin
    Arg.parse
      [
       "-o", String ignore, "(ignored)";
       "-Xfsc_extension", Set C_lexer.use_failsafec_extension, "enable Fail-Safe C extension keywords";
       "-Xkeyword", String
	 (fun s -> C_lexer.extension_keywords := s :: !C_lexer.extension_keywords), 
         "add keyword to extension keyword list";
       "-t", Set check_types, "perform type checking";
       "-f", Set filter_tree, "apply filters on typed abstract tree";
     ]
      process "test reader:"
  end
