AC_DEFUN([FSC_PROG_AR],
[
  AC_CHECK_TOOL(AR, [ar])
  AC_SUBST(AR)
])

AC_DEFUN([FSC_PROG_OBJDUMP],
[
  AC_CHECK_TOOL(OBJDUMP, [objdump])
  AC_SUBST(OBJDUMP)
])

AC_DEFUN([FSC_WITH_GC],
[
  AC_ARG_WITH(
    [gc],
    [AS_HELP_STRING([--with-gc=PATH], [Use libgc in PATH])],
    [
      if test "x$withval" == xyes ; then
        :
      elif test "x$withval" != xno && test "x$withval" != x ; then
        if test -d $withval/include ; then
          CPPFLAGS="$CPPFLAGS -I$withval/include"
        else
          CPPFLAGS="$CPPFLAGS -I$withval"
        fi 
        if test -d $withval/lib ; then
          LDFLAGS="$LDFLAGS -L$withval/lib"
        else
          LDFLAGS="$LDFLAGS -L$withval"
        fi
      fi
    ]
  )

  AC_CHECK_HEADERS(
    [gc.h],
    [],
    [AC_CHECK_HEADERS(
      [gc/gc.h],
      [],
      AC_MSG_ERROR([gc.h was not found. use --with-gc])
      )])
  AC_CHECK_LIB(
    [gc],
    [GC_malloc],
    [],
    AC_MSG_ERROR([libgc was not found. use --with-gc])
    )
])

AC_DEFUN([FSC_CHECK_NDBM_H],
[
  AC_CHECK_HEADERS([ndbm.h], [ndbm_h_found='yes'])
  unset ac_cv_header_gdbm_ndbm_h
  AC_CHECK_HEADER([gdbm-ndbm.h], 
    [
      AC_DEFINE([HAVE_GDBM_DASH_NDBM_H], [1], [Define to 1 if you have the <gdbm-ndbm.h> header file.])
      ndbm_h_found='yes'
    ])
  unset ac_cv_header_gdbm_ndbm_h
  AC_CHECK_HEADER([gdbm/ndbm.h], 
    [
      AC_DEFINE([HAVE_GDBM_SLASH_NDBM_H], [1], [Define to 1 if you have the <gdbm/ndbm.h> header file.])
      ndbm_h_found='yes'
    ])
  AS_IF(
    [test x"$ndbm_h_found" != 'xyes'],
    AC_MSG_ERROR([ndbm header file was not found!])
  )
])

# FSC_CC_OPTION_IFELSE([option], [iftrue], [ifelse])
AC_DEFUN([FSC_CC_OPTION_IFELSE],
[fsc_save_CFLAGS="$CFLAGS"
CFLAGS="$1"
AC_MSG_CHECKING([whether $[CC] $1 works])
AC_COMPILE_IFELSE([AC_LANG_PROGRAM()],
  [AC_MSG_RESULT([yes])
   CFLAGS="$fsc_save_CFLAGS"
  $2],
  [AC_MSG_RESULT([no])
   CFLAGS="$fsc_save_CFLAGS"
  $3])
])

#
# FSC_CC_OPTION([var1,var2,...], [opt1,opt2 ...])
#
AC_DEFUN([FSC_CC_OPTION],
[m4_ifval(
  [$2],
  [FSC_CC_OPTION_IFELSE(m4_car($2),
     [m4_foreach([fsc_var], [$1],
       [fsc_var="${fsc_var} m4_car($2)"
        ])],
     [FSC_CC_OPTION($1, m4_cdr($2))])])])

AC_DEFUN([FSC_CHECK_LIBM],
[
  AC_CHECK_FUNC(
    [log],
    [],
    AC_CHECK_LIB(
      [m],
      [log],
      [],
      AC_MSG_ERROR([libm was not found.])
      )
    )
])

AC_DEFUN([FSC_PERLIFY],
[
  $1=`perl -MData::Dumper -e '
    if (scalar(@ARGV) > 0) {
      @S|@Data::Dumper::Indent = 0;
      @S|@a = Dumper(\@ARGV);
      @S|@a =~ m/^.*?\@<:@(.*)\@:>@.*@S|@/;
      print @S|@1, ", ";
    }
    ' -- @S|@$2`
  AC_SUBST($1)
])
