/* 
   Part of Fail-Safe C Runtime. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2006 AIST.
   (c) 2006	 Lepidum Co. Ltd.

   This file is written by Yutaka Oiwa in 2004-2006.

   This file is distributed under Apache License 2.0 with a special exception.
   See the file LICENSE contained in the distribution.
*/
/**
 * @file stdlib/ctype_inline.c
 */
#if defined(FORCE_COMPILE_INLINE) || !defined(FSC_DEBUG_RUNTIME)
#include <stdio.h>

#if 0
#include <wrapper/fscw.h>
#include <wrapper/stdlib/ctype.h>
#endif

#ifdef FORCE_COMPILE_INLINE
#define INLINE
#else
#define INLINE static inline
#endif

#if 1
#define __DEFINE_CTYPE_FUNC(s) \
INLINE value FS_Fi_i_##s(base_t b, word v) {    \
    return value_of_int(s(v == EOF ? EOF : (unsigned char)v)); \
}
#else
/*
 *  parameter should be fscw_int_param, but prototype generated by trans
 *  causes type conflict. ( value s(base_t, unsigned int); )
 */
#define __DEFINE_CTYPE_FUNC(s) \
INLINE fscw_int fscw_stdlib_ctype(s)(fscw_unsigned_int_param(v)) {    \
    return fscw_int_wrap(s(v == EOF ? EOF : (unsigned char)v)); \
}
#endif

#include <ctype.h>

/**
 * @fn int isascii(int c);
 * @author Yutaka Oiwa.
 */
INLINE value FS_Fi_i_isascii(base_t b, word v) {
    return value_of_int(((unsigned int)v) <= 0x7f);
}

/**
 * @fn int isalnum(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isalnum)

/**
 * @fn int isalpha(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isalpha)

/**
 * @fn int iscntrl(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(iscntrl)

/**
 * @fn int isdigit(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isdigit)

/**
 * @fn int islower(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(islower)

/**
 * @fn int isgraph(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isgraph)

/**
 * @fn int isprint(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isprint)

/**
 * @fn int ispunct(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(ispunct)

/**
 * @fn int isspace(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isspace)

/**
 * @fn int isupper(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isupper)

/**
 * @fn int tolower(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(tolower)

/**
 * @fn int toupper(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(toupper)

/**
 * @fn int _toupper(int c)
 * @param c must be lower case letter
 *
 * @author Lepidum Co., Ltd.
 */
INLINE value FS_Fi_i__toupper(base_t b, word v) {    \
    return value_of_int(toupper(v == EOF ? EOF : (unsigned char)v)); \
}

/**
 * @fn int _tolower(int c)
 * @param c must be upper case letter
 *
 * @author Lepidum Co., Ltd.
 */
INLINE value FS_Fi_i__tolower(base_t b, word v) {    \
    return value_of_int(tolower(v == EOF ? EOF : (unsigned char)v)); \
}

/**
 * @fn int toascii(int c)
 *
 * @author Lepidum Co., Ltd.
 */
INLINE value FS_Fi_i_toascii(base_t b, word v) {
    return value_of_int(v & 0x7F);
}

#ifndef isblank
extern int isblank(int c);
#endif

/**
 * @fn int isblank(int c)
 *
 * @author Lepidum Co., Ltd.
 */
__DEFINE_CTYPE_FUNC(isblank)

/**
 * @fn int isxdigit(int c);
 * @author Yutaka Oiwa.
 */
__DEFINE_CTYPE_FUNC(isxdigit)

#undef __DEFINE_CTYPE_FUNC
#undef INLINE
#endif
