/* 
   Part of Fail-Safe C Runtime. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2006 AIST.
   (c) 2006	 Lepidum Co. Ltd.

   This file is written by Lepidum Co. Ltd. in 2006.

   This file is distributed under Apache License 2.0 with a special exception.
   See the file LICENSE contained in the distribution.
*/
/**
 * @file stdlib/fstat.c
 */
#define FSC_RUNTIME_LIBRARY
#include <fsc_runtime.h>
#include <wrapper_helper.h>

#include <sys/stat.h>

#if 0

static void copy_stat(base_t dst_b, ofs_t dst_o, struct stat *src)
{
  write_word(dst_b, dst_o     , value_of_int(src->st_dev),     NULL);
  write_word(dst_b, dst_o +  4, value_of_int(src->st_ino),     NULL);
  write_word(dst_b, dst_o +  8, value_of_int(src->st_mode),    NULL);
  write_word(dst_b, dst_o + 12, value_of_int(src->st_nlink),   NULL);
  write_word(dst_b, dst_o + 16, value_of_int(src->st_uid),     NULL);
  write_word(dst_b, dst_o + 20, value_of_int(src->st_gid),     NULL);
  write_word(dst_b, dst_o + 24, value_of_int(src->st_rdev),    NULL);
  write_word(dst_b, dst_o + 28, value_of_int(src->st_size),    NULL);
  write_word(dst_b, dst_o + 32, value_of_int(src->st_atime),   NULL);
  write_word(dst_b, dst_o + 36, value_of_int(src->st_mtime),   NULL);
  write_word(dst_b, dst_o + 40, value_of_int(src->st_ctime),   NULL);
  write_word(dst_b, dst_o + 44, value_of_int(src->st_blksize), NULL);
  write_word(dst_b, dst_o + 48, value_of_int(src->st_blocks),  NULL);
}

/**
 * @fn int fstat(int fd, struct stat *buf)
 * @brief retrieve file status information.
 * @param fd file descriptor to examine.
 * @param buf result stored here.
 * @retval 0 success.
 * @retval -1 error occured.
 *
 * @crashcase illegal pointer, buffer overflow
 * @fsctype atomic, struct(atomic+)
 *
 * @author Lepidum Co., Ltd.
 */
value FS_FiPSn11stdlib_stat__i_fstat(base_t fd_b, unsigned int fd_o,
                                     base_t buf_b, ofs_t buf_o)
{
  struct stat buf;
  int ret;

  ret = fstat(fd_o, &buf);
  if(ret == 0){
    copy_stat(buf_b, buf_o, &buf);
  }
  return value_of_int(ret);
}

/**
 * @fn int stat(const char *path, struct stat *buf)
 * @brief retrieve file status information.
 * @param path file name to examine.
 * @param buf result stored here.
 * @retval 0 success.
 * @retval -1 error occured.
 *
 * @crashcase illegal pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), struct(atomic+)
 *
 * @author Lepidum Co., Ltd.
 */
value FS_FPcPSn11stdlib_stat__i_stat(base_t filename_b, ofs_t filename_o,
                                     base_t buf_b, ofs_t buf_o)
{
  struct stat buf;
  int ret;
  void *tmp;
  char *filename = wrapper_get_string_z(filename_b, filename_o, &tmp, "stat");
  ret = stat(filename, &buf);
  wrapper_release_tmpbuf(tmp);
  
  if (ret == 0) {
    copy_stat(buf_b, buf_o, &buf);
  }
  return value_of_int(ret);
}

/**
 * @fn int lstat(const char *path, struct stat *buf)
 * @brief retrieve symbolic link status information.
 * @param path file or symbolic link name to examine.
 * @param buf result stored here.
 * @retval 0 success.
 * @retval -1 error occured.
 *
 * @crashcase illegal pointer, buffer overflow, non-terminated string
 * @fsctype string(ro), struct(atomic+)
 *
 * @author Lepidum Co., Ltd.
 */
value FS_FPcPSn11stdlib_stat__i_lstat(base_t filename_b, ofs_t filename_o,
                                      base_t buf_b, ofs_t buf_o)
{
  struct stat buf;
  int ret;
  void *tmp;
  char *filename = wrapper_get_string_z(filename_b, filename_o, &tmp, "lstat");
  ret = lstat(filename, &buf);
  wrapper_release_tmpbuf(tmp);

  if (ret == 0) {
    copy_stat(buf_b, buf_o, &buf);
  }
  return value_of_int(ret);
}

#endif
