/* 
   Part of Fail-Safe C Runtime. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2006 AIST.

   This file is written by Yutaka Oiwa in 2003-2004.

   This file is distributed under Apache License 2.0 with a special exception.
   See the file LICENSE contained in the distribution.
*/
#define FSC_RUNTIME_LIBRARY
#include <stdlib.h>
#include <stdio.h>
#include "fsc_runtime.h"
#include "wrapper_helper.h"
#include "fsc_stackrec.h"

#ifdef CALL_SPECIFIC_MAIN
extern value FS_FiPPc_i_main(base_t, int, base_t, ofs_t);
#else
extern dvalue FG_main(base_t,base_t);
#endif

extern struct typeinfo_init fsc_typeinfo_Pc;

#include "main_globalvars.safe.c"

extern const char **environ;

int main(int argc, char **argv)
{
    int i, envc;
#ifdef FSC_CALL_SPECIFIC_MAIN
    value v;
#else
    value *varg;
    dvalue dv;
#endif
    ptrvalue *copied_argv;
    ptrvalue *copied_envp;

    struct fsc_stack_frame s;

    fsc_init_runtime();

    enter_stack_unwind_area(&s, "__start_main", 0);

    copied_argv = (ptrvalue *)fsc_alloc_static_block(&fsc_typeinfo_Pc.val, argc + 1);
    for(i = 0; i < argc; i++) {
	copied_argv[i] = wrapper_make_new_static_string(argv[i]);
    }
    copied_argv[argc] = ptrvalue_of_base_ofs(0, 0);
    GV_Pc___progname.val.cv = copied_argv[0];

    for (envc = 0;; envc++) {
	if (environ[envc] == NULL) break;
    }

    copied_envp = (ptrvalue *)fsc_alloc_static_block(&fsc_typeinfo_Pc.val, envc + 1);
    for(i = 0; i < envc; i++) {
	copied_envp[i] = wrapper_make_new_static_string(environ[i]);
    }
    copied_envp[envc] = ptrvalue_of_base_ofs(0, 0);

    GV_PPc_environ.val.cv = ptrvalue_of_base_ofs((base_t)copied_envp, 0);

#ifdef FSC_CALL_SPECIFIC_MAIN
    v = FS_FiPPc_i_main(0, argc, (base_t)copied_argv, 0);
#ifdef FSC_DEBUG_RUNTIME
    assert_all_buffers_are_released();
#endif
    return vaddr_of_value(v);
#else
    varg = (value *)fsc_alloc_varargs(2);
    varg[0] = value_of_int(argc);
    varg[1] = value_of_base_ofs((base_t)copied_argv, 0);
    dv = FG_main(0,(base_t)varg);

    exit_stack_unwind_area();

#ifdef FSC_DEBUG_RUNTIME
    assert_all_buffers_are_released();
#endif
    return (int)vaddr_of_dvalue(dv);
#endif
}

