/* 
   Part of Fail-Safe C Runtime. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2006 AIST.
   (c) 2006	 Lepidum Co. Ltd.

   This file is written by Lepidum Co. Ltd. in 2006.

   This file is distributed under Apache License 2.0 with a special exception.
   See the file LICENSE contained in the distribution.
*/
/**
 * @file stdlib/unistd/crypt.c
 */
#define _XOPEN_SOURCE 600

#define FSC_RUNTIME_LIBRARY
#include <fsc_runtime.h>
#include <wrapper_helper.h>

/* for Linux */
#include <unistd.h>

/**
 * @fn char *crypt(const char *key, const char *salt)
 * @brief encrypt string.
 * @param key key string.
 * @param salt salt string.
 * @return encoded string.
 *
 * @crashcase invalid pointer, non-terminated string, buffer overflow
 * @fsctype string(ro)
 *
 * @author Lepidum Co., Ltd.
 */
ptrvalue FS_FPcPc_Pc_crypt(base_t key_b, ofs_t key_o,
                           base_t salt_b, ofs_t salt_o)
{
  void *t1, *t2;
  char *key  = wrapper_get_string_z(key_b, key_o, &t1, "crypt");
  char *salt = wrapper_get_string_zn(salt_b, salt_o, &t2, 2, "crypt");
  char *ret = crypt(key, salt);
  wrapper_release_tmpbuf(t1);
  wrapper_release_tmpbuf(t2);
  return wrapper_make_new_static_string(ret);
}
