/*
   This file is automatically generated.
 */

/**
 * @file math/atan2.test.c
 */
#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

#ifndef NAN
#define NAN (0.0f/0.0f)
#endif

#ifndef INF
#define INF (1.0f/0.0f)
#endif

#define ABS_FLOAT       fabsf
#define ABS_DOUBLE      fabs
#define ABS_LONG_DOUBLE fabsl

#define TOLERANCE_FLOAT       0x1p-20
#define TOLERANCE_DOUBLE      0x1p-50
#define TOLERANCE_LONG_DOUBLE 0x1p-60

#define RET_MINIMUM_FLOAT       0x1p-126
#define RET_MINIMUM_DOUBLE      0x1p-1022
#define RET_MINIMUM_LONG_DOUBLE 0x1p-16382

#define max(a, b) ((a > b) ? a : b)

#define CMP_FLOAT(x, t) \
  static int compare_##x(t actual, t expected) { \
    if (isnan(actual) && isnan(expected)) \
      return 1; \
    else if (isinf(actual) && isinf(expected)) \
      return ((actual > (t)0.0) == (expected > (t)0.0)); \
    else \
      return (ABS_##x(actual - expected) < max(ABS_##x(expected * TOLERANCE_##x), RET_MINIMUM_##x)); \
  }

#define CMP_INT(x, t) \
  static int compare_##x(t actual, t expected) { return (actual == expected); }

CMP_FLOAT(FLOAT, float)
CMP_FLOAT(DOUBLE, double)
/* CMP_FLOAT(LONG_DOUBLE, long double) */

CMP_INT(INT, int)
CMP_INT(LONG, long)
CMP_INT(LONG_LONG, long long)

#define _CMP(t, a, e) compare_##t(a, e)
#define CMP(t, a, e) _CMP(t, a, e)

/**
 * @testname atan2_2
 * @testfor atan2
 */
TEST_CASE(atan2_2)
{

  static struct {
    double a;

    double a2;

  } args[] = {
    { 0x0p+0, 0x0p+0 },
    { 0x0p+0, -0x0p+0 },
    { 0x0p+0, 0x1p+0 },
    { 0x0p+0, -0x1p+0 },
    { 0x0p+0, 0x1p-1022 },
    { 0x0p+0, -0x1p-1022 },
    { 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, INF },
    { 0x0p+0, -INF },
    { 0x0p+0, NAN },
    { 0x0p+0, NAN },
    { 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x0p+0 },
    { -0x0p+0, -0x0p+0 },
    { -0x0p+0, 0x1p+0 },
    { -0x0p+0, -0x1p+0 },
    { -0x0p+0, 0x1p-1022 },
    { -0x0p+0, -0x1p-1022 },
    { -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, INF },
    { -0x0p+0, -INF },
    { -0x0p+0, NAN },
    { -0x0p+0, NAN },
    { -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x0p+0 },
    { 0x1p+0, -0x0p+0 },
    { 0x1p+0, 0x1p+0 },
    { 0x1p+0, -0x1p+0 },
    { 0x1p+0, 0x1p-1022 },
    { 0x1p+0, -0x1p-1022 },
    { 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, INF },
    { 0x1p+0, -INF },
    { 0x1p+0, NAN },
    { 0x1p+0, NAN },
    { 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x0p+0 },
    { -0x1p+0, -0x0p+0 },
    { -0x1p+0, 0x1p+0 },
    { -0x1p+0, -0x1p+0 },
    { -0x1p+0, 0x1p-1022 },
    { -0x1p+0, -0x1p-1022 },
    { -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, INF },
    { -0x1p+0, -INF },
    { -0x1p+0, NAN },
    { -0x1p+0, NAN },
    { -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x0p+0 },
    { 0x1p-1022, -0x0p+0 },
    { 0x1p-1022, 0x1p+0 },
    { 0x1p-1022, -0x1p+0 },
    { 0x1p-1022, 0x1p-1022 },
    { 0x1p-1022, -0x1p-1022 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, INF },
    { 0x1p-1022, -INF },
    { 0x1p-1022, NAN },
    { 0x1p-1022, NAN },
    { 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x0p+0 },
    { -0x1p-1022, -0x0p+0 },
    { -0x1p-1022, 0x1p+0 },
    { -0x1p-1022, -0x1p+0 },
    { -0x1p-1022, 0x1p-1022 },
    { -0x1p-1022, -0x1p-1022 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, INF },
    { -0x1p-1022, -INF },
    { -0x1p-1022, NAN },
    { -0x1p-1022, NAN },
    { -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, INF },
    { 0x1.fffffffffffffp+1023, -INF },
    { 0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, INF },
    { -0x1.fffffffffffffp+1023, -INF },
    { -0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { INF, 0x0p+0 },
    { INF, -0x0p+0 },
    { INF, 0x1p+0 },
    { INF, -0x1p+0 },
    { INF, 0x1p-1022 },
    { INF, -0x1p-1022 },
    { INF, 0x1.fffffffffffffp+1023 },
    { INF, -0x1.fffffffffffffp+1023 },
    { INF, INF },
    { INF, -INF },
    { INF, NAN },
    { INF, NAN },
    { INF, 0x1.ccccccccccccdp-1 },
    { INF, 0x1.199999999999ap+0 },
    { INF, -0x1.ccccccccccccdp-1 },
    { INF, -0x1.199999999999ap+0 },
    { -INF, 0x0p+0 },
    { -INF, -0x0p+0 },
    { -INF, 0x1p+0 },
    { -INF, -0x1p+0 },
    { -INF, 0x1p-1022 },
    { -INF, -0x1p-1022 },
    { -INF, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1.fffffffffffffp+1023 },
    { -INF, INF },
    { -INF, -INF },
    { -INF, NAN },
    { -INF, NAN },
    { -INF, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1.199999999999ap+0 },
    { -INF, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1.199999999999ap+0 },
    { NAN, 0x0p+0 },
    { NAN, -0x0p+0 },
    { NAN, 0x1p+0 },
    { NAN, -0x1p+0 },
    { NAN, 0x1p-1022 },
    { NAN, -0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023 },
    { NAN, INF },
    { NAN, -INF },
    { NAN, NAN },
    { NAN, NAN },
    { NAN, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0 },
    { NAN, 0x0p+0 },
    { NAN, -0x0p+0 },
    { NAN, 0x1p+0 },
    { NAN, -0x1p+0 },
    { NAN, 0x1p-1022 },
    { NAN, -0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023 },
    { NAN, INF },
    { NAN, -INF },
    { NAN, NAN },
    { NAN, NAN },
    { NAN, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, INF },
    { 0x1.ccccccccccccdp-1, -INF },
    { 0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, INF },
    { 0x1.199999999999ap+0, -INF },
    { 0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, INF },
    { -0x1.ccccccccccccdp-1, -INF },
    { -0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, INF },
    { -0x1.199999999999ap+0, -INF },
    { -0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0 },

  };
  static struct {
    double e;

  } expected[] = {
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x1.921fb54442d18p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x0.4p-1022 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ad0be00825672p-1 } ,
    { 0x1.79c2e5436d663p-1 } ,
    { 0x1.26dcbd423977cp+1 } ,
    { 0x1.33aefbf36777fp+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x0.4p-1022 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.ad0be00825672p-1 } ,
    { -0x1.79c2e5436d663p-1 } ,
    { -0x1.26dcbd423977cp+1 } ,
    { -0x1.33aefbf36777fp+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1p-1022 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.1c71c71c71c72p-1022 } ,
    { 0x0.e8ba2e8ba2e8bp-1022 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x1.921fb54442d18p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1p-1022 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.1c71c71c71c72p-1022 } ,
    { -0x0.e8ba2e8ba2e8bp-1022 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x1.921fb54442d18p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.77338a80603bep-1 } ,
    { 0x1.3452d2a42ac29p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x0.399999999999ap-1022 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.5f17f03e7d38ep-1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { 0x1.3a59b934a3835p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.aa7c8545183cdp-1 } ,
    { 0x1.278093f2fcc25p+1 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x1.921fb54442d18p+0 } ,
    { 0x0.4666666666667p-1022 } ,
    { 0x1.921fb54442d18p+1 } ,
    { 0x0p+0 } ,
    { 0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { 0x1.c5277a4a086a2p-1 } ,
    { 0x1.921fb54442d18p-1 } ,
    { 0x1.20d5d6b1c0b7p+1 } ,
    { 0x1.2d97c7f3321d2p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.77338a80603bep-1 } ,
    { -0x1.3452d2a42ac29p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x0.399999999999ap-1022 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.5f17f03e7d38ep-1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,
    { -0x1.3a59b934a3835p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.aa7c8545183cdp-1 } ,
    { -0x1.278093f2fcc25p+1 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x1.921fb54442d18p+0 } ,
    { -0x0.4666666666667p-1022 } ,
    { -0x1.921fb54442d18p+1 } ,
    { -0x0p+0 } ,
    { -0x1.921fb54442d18p+1 } ,
    { NAN } ,
    { NAN } ,
    { -0x1.c5277a4a086a2p-1 } ,
    { -0x1.921fb54442d18p-1 } ,
    { -0x1.20d5d6b1c0b7p+1 } ,
    { -0x1.2d97c7f3321d2p+1 } ,

  };
  int i;
  int ok = 1;
  for (i = 0; i < sizeof(args) / sizeof(args[0]); ++i) {

    double result = atan2(args[i].a

                                , args[i].a2


                                );

    if (!CMP(DOUBLE, result, expected[i].e)

        ) {
      fprintf(stderr,
              "atan2(%.16e"

              ", %.16e"


              ")\n",
              args[i].a

              , args[i].a2

              );

      fprintf(stderr,
              "returned: %.16e, expected: %.16e\n",
              result,
              expected[i].e
              );

      ok = 0;
    }
  }
  TEST(ok);
}
