/*
   This file is automatically generated.
 */

/**
 * @file math/remquof.test.c
 */
#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

#ifndef NAN
#define NAN (0.0f/0.0f)
#endif

#ifndef INF
#define INF (1.0f/0.0f)
#endif

#define ABS_FLOAT       fabsf
#define ABS_DOUBLE      fabs
#define ABS_LONG_DOUBLE fabsl

#define TOLERANCE_FLOAT       0x1p-20
#define TOLERANCE_DOUBLE      0x1p-50
#define TOLERANCE_LONG_DOUBLE 0x1p-60

#define RET_MINIMUM_FLOAT       0x1p-126
#define RET_MINIMUM_DOUBLE      0x1p-1022
#define RET_MINIMUM_LONG_DOUBLE 0x1p-16382

#define max(a, b) ((a > b) ? a : b)

#define CMP_FLOAT(x, t) \
  static int compare_##x(t actual, t expected) { \
    if (isnan(actual) && isnan(expected)) \
      return 1; \
    else if (isinf(actual) && isinf(expected)) \
      return ((actual > (t)0.0) == (expected > (t)0.0)); \
    else \
      return (ABS_##x(actual - expected) < max(ABS_##x(expected * TOLERANCE_##x), RET_MINIMUM_##x)); \
  }

#define CMP_INT(x, t) \
  static int compare_##x(t actual, t expected) { return (actual == expected); }

CMP_FLOAT(FLOAT, float)
CMP_FLOAT(DOUBLE, double)
/* CMP_FLOAT(LONG_DOUBLE, long double) */

CMP_INT(INT, int)
CMP_INT(LONG, long)
CMP_INT(LONG_LONG, long long)

#define _CMP(t, a, e) compare_##t(a, e)
#define CMP(t, a, e) _CMP(t, a, e)

/**
 * @testname remquof_2
 * @testfor remquof
 */
TEST_CASE(remquof_2)
{

  static struct {
    float a;

    float a2;

  } args[] = {
    { 0x0p+0F, 0x0p+0F },
    { 0x0p+0F, -0x0p+0F },
    { 0x0p+0F, 0x1p+0F },
    { 0x0p+0F, -0x1p+0F },
    { 0x0p+0F, 0x1p-126F },
    { 0x0p+0F, -0x1p-126F },
    { 0x0p+0F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1.fffffep+127F },
    { 0x0p+0F, INF },
    { 0x0p+0F, -INF },
    { 0x0p+0F, NAN },
    { 0x0p+0F, NAN },
    { 0x0p+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x0p+0F },
    { -0x0p+0F, -0x0p+0F },
    { -0x0p+0F, 0x1p+0F },
    { -0x0p+0F, -0x1p+0F },
    { -0x0p+0F, 0x1p-126F },
    { -0x0p+0F, -0x1p-126F },
    { -0x0p+0F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1.fffffep+127F },
    { -0x0p+0F, INF },
    { -0x0p+0F, -INF },
    { -0x0p+0F, NAN },
    { -0x0p+0F, NAN },
    { -0x0p+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x0p+0F },
    { 0x1p+0F, -0x0p+0F },
    { 0x1p+0F, 0x1p+0F },
    { 0x1p+0F, -0x1p+0F },
    { 0x1p+0F, 0x1p-126F },
    { 0x1p+0F, -0x1p-126F },
    { 0x1p+0F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1.fffffep+127F },
    { 0x1p+0F, INF },
    { 0x1p+0F, -INF },
    { 0x1p+0F, NAN },
    { 0x1p+0F, NAN },
    { 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x0p+0F },
    { -0x1p+0F, -0x0p+0F },
    { -0x1p+0F, 0x1p+0F },
    { -0x1p+0F, -0x1p+0F },
    { -0x1p+0F, 0x1p-126F },
    { -0x1p+0F, -0x1p-126F },
    { -0x1p+0F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1.fffffep+127F },
    { -0x1p+0F, INF },
    { -0x1p+0F, -INF },
    { -0x1p+0F, NAN },
    { -0x1p+0F, NAN },
    { -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x0p+0F },
    { 0x1p-126F, -0x0p+0F },
    { 0x1p-126F, 0x1p+0F },
    { 0x1p-126F, -0x1p+0F },
    { 0x1p-126F, 0x1p-126F },
    { 0x1p-126F, -0x1p-126F },
    { 0x1p-126F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1.fffffep+127F },
    { 0x1p-126F, INF },
    { 0x1p-126F, -INF },
    { 0x1p-126F, NAN },
    { 0x1p-126F, NAN },
    { 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x0p+0F },
    { -0x1p-126F, -0x0p+0F },
    { -0x1p-126F, 0x1p+0F },
    { -0x1p-126F, -0x1p+0F },
    { -0x1p-126F, 0x1p-126F },
    { -0x1p-126F, -0x1p-126F },
    { -0x1p-126F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1.fffffep+127F },
    { -0x1p-126F, INF },
    { -0x1p-126F, -INF },
    { -0x1p-126F, NAN },
    { -0x1p-126F, NAN },
    { -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, INF },
    { 0x1.fffffep+127F, -INF },
    { 0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, INF },
    { -0x1.fffffep+127F, -INF },
    { -0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F },
    { INF, 0x0p+0F },
    { INF, -0x0p+0F },
    { INF, 0x1p+0F },
    { INF, -0x1p+0F },
    { INF, 0x1p-126F },
    { INF, -0x1p-126F },
    { INF, 0x1.fffffep+127F },
    { INF, -0x1.fffffep+127F },
    { INF, INF },
    { INF, -INF },
    { INF, NAN },
    { INF, NAN },
    { INF, 0x1.ccccccp-1F },
    { INF, 0x1.19999ap+0F },
    { INF, -0x1.ccccccp-1F },
    { INF, -0x1.19999ap+0F },
    { -INF, 0x0p+0F },
    { -INF, -0x0p+0F },
    { -INF, 0x1p+0F },
    { -INF, -0x1p+0F },
    { -INF, 0x1p-126F },
    { -INF, -0x1p-126F },
    { -INF, 0x1.fffffep+127F },
    { -INF, -0x1.fffffep+127F },
    { -INF, INF },
    { -INF, -INF },
    { -INF, NAN },
    { -INF, NAN },
    { -INF, 0x1.ccccccp-1F },
    { -INF, 0x1.19999ap+0F },
    { -INF, -0x1.ccccccp-1F },
    { -INF, -0x1.19999ap+0F },
    { NAN, 0x0p+0F },
    { NAN, -0x0p+0F },
    { NAN, 0x1p+0F },
    { NAN, -0x1p+0F },
    { NAN, 0x1p-126F },
    { NAN, -0x1p-126F },
    { NAN, 0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F },
    { NAN, INF },
    { NAN, -INF },
    { NAN, NAN },
    { NAN, NAN },
    { NAN, 0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F },
    { NAN, 0x0p+0F },
    { NAN, -0x0p+0F },
    { NAN, 0x1p+0F },
    { NAN, -0x1p+0F },
    { NAN, 0x1p-126F },
    { NAN, -0x1p-126F },
    { NAN, 0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F },
    { NAN, INF },
    { NAN, -INF },
    { NAN, NAN },
    { NAN, NAN },
    { NAN, 0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, INF },
    { 0x1.ccccccp-1F, -INF },
    { 0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, INF },
    { 0x1.19999ap+0F, -INF },
    { 0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, INF },
    { -0x1.ccccccp-1F, -INF },
    { -0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, INF },
    { -0x1.19999ap+0F, -INF },
    { -0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F },

  };
  static struct {
    float e;

    int e2;

  } expected[] = {
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x0p+0F, 1 } ,
    { 0x0p+0F, -1 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x1p+0F, 0 } ,
    { 0x1p+0F, 0 } ,
    { 0x1p+0F, 0 } ,
    { 0x1p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1.9999ap-4F, 1 } ,
    { -0x1.9999ap-4F, 1 } ,
    { 0x1.9999ap-4F, -1 } ,
    { -0x1.9999ap-4F, -1 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x0p+0F, -1 } ,
    { -0x0p+0F, 1 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x1p+0F, 0 } ,
    { -0x1p+0F, 0 } ,
    { -0x1p+0F, 0 } ,
    { -0x1p+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1.9999ap-4F, -1 } ,
    { 0x1.9999ap-4F, -1 } ,
    { -0x1.9999ap-4F, 1 } ,
    { 0x1.9999ap-4F, 1 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x0p+0F, 1 } ,
    { 0x0p+0F, -1 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { 0x1p-126F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x0p+0F, -1 } ,
    { -0x0p+0F, 1 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { -0x1p-126F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 1 } ,
    { 0x0p+0F, -1 } ,
    { 0x1.fffffep+127F, 0 } ,
    { 0x1.fffffep+127F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1.5f1dcp-5F, 3 } ,
    { -0x1.c1568p-5F, 2 } ,
    { 0x1.5f1dcp-5F, -3 } ,
    { -0x1.c1568p-5F, -2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, -1 } ,
    { -0x0p+0F, 1 } ,
    { -0x1.fffffep+127F, 0 } ,
    { -0x1.fffffep+127F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1.5f1dcp-5F, -3 } ,
    { 0x1.c1568p-5F, -2 } ,
    { -0x1.5f1dcp-5F, 3 } ,
    { 0x1.c1568p-5F, 2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { NAN, -2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, 2 } ,
    { NAN, -2 } ,
    { -0x1.9999ap-4F, 1 } ,
    { -0x1.9999ap-4F, -1 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x1.ccccccp-1F, 0 } ,
    { 0x1.ccccccp-1F, 0 } ,
    { 0x1.ccccccp-1F, 0 } ,
    { 0x1.ccccccp-1F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x0p+0F, 1 } ,
    { -0x1.9999ap-3F, 1 } ,
    { 0x0p+0F, -1 } ,
    { -0x1.9999ap-3F, -1 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1.9999ap-4F, 1 } ,
    { 0x1.9999ap-4F, -1 } ,
    { 0x0p+0F, 0 } ,
    { 0x0p+0F, 0 } ,
    { 0x1.19999ap+0F, 0 } ,
    { 0x1.19999ap+0F, 0 } ,
    { 0x1.19999ap+0F, 0 } ,
    { 0x1.19999ap+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1.9999ap-3F, 1 } ,
    { 0x0p+0F, 1 } ,
    { 0x1.9999ap-3F, -1 } ,
    { 0x0p+0F, -1 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { 0x1.9999ap-4F, -1 } ,
    { 0x1.9999ap-4F, 1 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x1.ccccccp-1F, 0 } ,
    { -0x1.ccccccp-1F, 0 } ,
    { -0x1.ccccccp-1F, 0 } ,
    { -0x1.ccccccp-1F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x0p+0F, -1 } ,
    { 0x1.9999ap-3F, -1 } ,
    { -0x0p+0F, 1 } ,
    { 0x1.9999ap-3F, 1 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1.9999ap-4F, -1 } ,
    { -0x1.9999ap-4F, 1 } ,
    { -0x0p+0F, 0 } ,
    { -0x0p+0F, 0 } ,
    { -0x1.19999ap+0F, 0 } ,
    { -0x1.19999ap+0F, 0 } ,
    { -0x1.19999ap+0F, 0 } ,
    { -0x1.19999ap+0F, 0 } ,
    { NAN, 0 } ,
    { NAN, 0 } ,
    { -0x1.9999ap-3F, -1 } ,
    { -0x0p+0F, -1 } ,
    { -0x1.9999ap-3F, 1 } ,
    { -0x0p+0F, 1 } ,

  };
  int i;
  int ok = 1;
  for (i = 0; i < sizeof(args) / sizeof(args[0]); ++i) {

    int result2;

    float result = remquof(args[i].a

                                , args[i].a2


                                , &result2

                                );

    if (!CMP(FLOAT, result, expected[i].e)

        || !CMP(INT, result2, expected[i].e2)

        ) {
      fprintf(stderr,
              "remquof(%.8eF"

              ", %.8eF"


              ", &r2"

              ")\n",
              args[i].a

              , args[i].a2

              );

      fprintf(stderr,
              "returned: (%.8eF, %d), "
              "expected: (%.8eF, %d)\n",
              result, result2,
              expected[i].e, expected[i].e2
              );

      ok = 0;
    }
  }
  TEST(ok);
}
