/*
   This file is automatically generated.
 */

/**
 * @file math/scalbln.test.c
 */
#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

#ifndef NAN
#define NAN (0.0f/0.0f)
#endif

#ifndef INF
#define INF (1.0f/0.0f)
#endif

#define ABS_FLOAT       fabsf
#define ABS_DOUBLE      fabs
#define ABS_LONG_DOUBLE fabsl

#define TOLERANCE_FLOAT       0x1p-20
#define TOLERANCE_DOUBLE      0x1p-50
#define TOLERANCE_LONG_DOUBLE 0x1p-60

#define RET_MINIMUM_FLOAT       0x1p-126
#define RET_MINIMUM_DOUBLE      0x1p-1022
#define RET_MINIMUM_LONG_DOUBLE 0x1p-16382

#define max(a, b) ((a > b) ? a : b)

#define CMP_FLOAT(x, t) \
  static int compare_##x(t actual, t expected) { \
    if (isnan(actual) && isnan(expected)) \
      return 1; \
    else if (isinf(actual) && isinf(expected)) \
      return ((actual > (t)0.0) == (expected > (t)0.0)); \
    else \
      return (ABS_##x(actual - expected) < max(ABS_##x(expected * TOLERANCE_##x), RET_MINIMUM_##x)); \
  }

#define CMP_INT(x, t) \
  static int compare_##x(t actual, t expected) { return (actual == expected); }

CMP_FLOAT(FLOAT, float)
CMP_FLOAT(DOUBLE, double)
/* CMP_FLOAT(LONG_DOUBLE, long double) */

CMP_INT(INT, int)
CMP_INT(LONG, long)
CMP_INT(LONG_LONG, long long)

#define _CMP(t, a, e) compare_##t(a, e)
#define CMP(t, a, e) _CMP(t, a, e)

/**
 * @testname scalbln_2
 * @testfor scalbln
 */
TEST_CASE(scalbln_2)
{

  static struct {
    double a;

    long a2;

  } args[] = {
    { 0x0p+0, -2046L },
    { 0x0p+0, -1027L },
    { 0x0p+0, -1L },
    { 0x0p+0, 0L },
    { 0x0p+0, 1L },
    { 0x0p+0, 1027L },
    { 0x0p+0, 2046L },
    { -0x0p+0, -2046L },
    { -0x0p+0, -1027L },
    { -0x0p+0, -1L },
    { -0x0p+0, 0L },
    { -0x0p+0, 1L },
    { -0x0p+0, 1027L },
    { -0x0p+0, 2046L },
    { 0x1p+0, -2046L },
    { 0x1p+0, -1027L },
    { 0x1p+0, -1L },
    { 0x1p+0, 0L },
    { 0x1p+0, 1L },
    { 0x1p+0, 1027L },
    { 0x1p+0, 2046L },
    { -0x1p+0, -2046L },
    { -0x1p+0, -1027L },
    { -0x1p+0, -1L },
    { -0x1p+0, 0L },
    { -0x1p+0, 1L },
    { -0x1p+0, 1027L },
    { -0x1p+0, 2046L },
    { 0x1p-1022, -2046L },
    { 0x1p-1022, -1027L },
    { 0x1p-1022, -1L },
    { 0x1p-1022, 0L },
    { 0x1p-1022, 1L },
    { 0x1p-1022, 1027L },
    { 0x1p-1022, 2046L },
    { -0x1p-1022, -2046L },
    { -0x1p-1022, -1027L },
    { -0x1p-1022, -1L },
    { -0x1p-1022, 0L },
    { -0x1p-1022, 1L },
    { -0x1p-1022, 1027L },
    { -0x1p-1022, 2046L },
    { 0x1.fffffffffffffp+1023, -2046L },
    { 0x1.fffffffffffffp+1023, -1027L },
    { 0x1.fffffffffffffp+1023, -1L },
    { 0x1.fffffffffffffp+1023, 0L },
    { 0x1.fffffffffffffp+1023, 1L },
    { 0x1.fffffffffffffp+1023, 1027L },
    { 0x1.fffffffffffffp+1023, 2046L },
    { -0x1.fffffffffffffp+1023, -2046L },
    { -0x1.fffffffffffffp+1023, -1027L },
    { -0x1.fffffffffffffp+1023, -1L },
    { -0x1.fffffffffffffp+1023, 0L },
    { -0x1.fffffffffffffp+1023, 1L },
    { -0x1.fffffffffffffp+1023, 1027L },
    { -0x1.fffffffffffffp+1023, 2046L },
    { INF, -2046L },
    { INF, -1027L },
    { INF, -1L },
    { INF, 0L },
    { INF, 1L },
    { INF, 1027L },
    { INF, 2046L },
    { -INF, -2046L },
    { -INF, -1027L },
    { -INF, -1L },
    { -INF, 0L },
    { -INF, 1L },
    { -INF, 1027L },
    { -INF, 2046L },
    { NAN, -2046L },
    { NAN, -1027L },
    { NAN, -1L },
    { NAN, 0L },
    { NAN, 1L },
    { NAN, 1027L },
    { NAN, 2046L },
    { NAN, -2046L },
    { NAN, -1027L },
    { NAN, -1L },
    { NAN, 0L },
    { NAN, 1L },
    { NAN, 1027L },
    { NAN, 2046L },
    { 0x1.ccccccccccccdp-1, -2046L },
    { 0x1.ccccccccccccdp-1, -1027L },
    { 0x1.ccccccccccccdp-1, -1L },
    { 0x1.ccccccccccccdp-1, 0L },
    { 0x1.ccccccccccccdp-1, 1L },
    { 0x1.ccccccccccccdp-1, 1027L },
    { 0x1.ccccccccccccdp-1, 2046L },
    { 0x1.199999999999ap+0, -2046L },
    { 0x1.199999999999ap+0, -1027L },
    { 0x1.199999999999ap+0, -1L },
    { 0x1.199999999999ap+0, 0L },
    { 0x1.199999999999ap+0, 1L },
    { 0x1.199999999999ap+0, 1027L },
    { 0x1.199999999999ap+0, 2046L },
    { -0x1.ccccccccccccdp-1, -2046L },
    { -0x1.ccccccccccccdp-1, -1027L },
    { -0x1.ccccccccccccdp-1, -1L },
    { -0x1.ccccccccccccdp-1, 0L },
    { -0x1.ccccccccccccdp-1, 1L },
    { -0x1.ccccccccccccdp-1, 1027L },
    { -0x1.ccccccccccccdp-1, 2046L },
    { -0x1.199999999999ap+0, -2046L },
    { -0x1.199999999999ap+0, -1027L },
    { -0x1.199999999999ap+0, -1L },
    { -0x1.199999999999ap+0, 0L },
    { -0x1.199999999999ap+0, 1L },
    { -0x1.199999999999ap+0, 1027L },
    { -0x1.199999999999ap+0, 2046L },

  };
  static struct {
    double e;

  } expected[] = {
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0.08p-1022 } ,
    { 0x1p-1 } ,
    { 0x1p+0 } ,
    { 0x1p+1 } ,
    { INF } ,
    { INF } ,
    { -0x0p+0 } ,
    { -0x0.08p-1022 } ,
    { -0x1p-1 } ,
    { -0x1p+0 } ,
    { -0x1p+1 } ,
    { -INF } ,
    { -INF } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0.8p-1022 } ,
    { 0x1p-1022 } ,
    { 0x1p-1021 } ,
    { 0x1p+5 } ,
    { INF } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { -0x0.8p-1022 } ,
    { -0x1p-1022 } ,
    { -0x1p-1021 } ,
    { -0x1p+5 } ,
    { -INF } ,
    { 0x1p-1022 } ,
    { 0x1.fffffffffffffp-4 } ,
    { 0x1.fffffffffffffp+1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -0x1p-1022 } ,
    { -0x1.fffffffffffffp-4 } ,
    { -0x1.fffffffffffffp+1022 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x0.0733333333333p-1022 } ,
    { 0x1.ccccccccccccdp-2 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp+0 } ,
    { INF } ,
    { INF } ,
    { 0x0p+0 } ,
    { 0x0.08ccccccccccdp-1022 } ,
    { 0x1.199999999999ap-1 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+1 } ,
    { INF } ,
    { INF } ,
    { -0x0p+0 } ,
    { -0x0.0733333333333p-1022 } ,
    { -0x1.ccccccccccccdp-2 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp+0 } ,
    { -INF } ,
    { -INF } ,
    { -0x0p+0 } ,
    { -0x0.08ccccccccccdp-1022 } ,
    { -0x1.199999999999ap-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+1 } ,
    { -INF } ,
    { -INF } ,

  };
  int i;
  int ok = 1;
  for (i = 0; i < sizeof(args) / sizeof(args[0]); ++i) {

    double result = scalbln(args[i].a

                                , args[i].a2


                                );

    if (!CMP(DOUBLE, result, expected[i].e)

        ) {
      fprintf(stderr,
              "scalbln(%.16e"

              ", %ldL"


              ")\n",
              args[i].a

              , args[i].a2

              );

      fprintf(stderr,
              "returned: %.16e, expected: %.16e\n",
              result,
              expected[i].e
              );

      ok = 0;
    }
  }
  TEST(ok);
}
