(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2007 AIST.

   This file is written by Yutaka Oiwa in 2007. *)

(* pretty-formatting functions for IL family *)

open Il
open Format
open Ctt_printer

let pp_ctt_type ppf ct = 
  pp_print_string ppf (Ctt_printer.strof_c_type ct) (* temporary *)
  

let rec pp_list pp_elem sep ppf v = 
  match v with
    [] -> ()
  | [v] -> pp_elem ppf v
  | h::t -> pp_elem ppf h; pp_print_string ppf sep; pp_list pp_elem sep ppf t

let pp_temp_id ppf v = fprintf ppf "%%%d" v
let pp_identifier = pp_print_string

let pp_field ppf (id, ct) = fprintf ppf ".(%s : %s)" id (Ctt_printer.strof_c_type ct)

let pp_il_switch_label ppf =
   function 
       CASE bi -> fprintf ppf "case %s:" (Big_int.string_of_big_int bi)
     | DEFAULT -> pp_print_string ppf "default:"
	   
let pp_il_if_type ppf s = 
  pp_print_string ppf (if s = IFNOT then "ifnot" else "if")

let pp_il_binop ppf s = 
  pp_print_string ppf
    (match s with
      ILbinTimes -> "*"
    | ILbinDiv -> "/"
    | ILbinPlusVV -> "+" 
    | ILbinMinusVV -> "-"
    | ILbinPlusPV -> "p+" 
    | ILbinMinusPP -> "p-p" 
    | ILbinMinusPV -> "p-" 
    | ILbinModulo -> "%"
    | ILbinLshift -> "<<" 
    | ILbinRshift -> ">>" 
    | ILbinLogAnd -> "&&" 
    | ILbinLogOr -> "||" 
    | ILbinIntAnd -> "&" 
    | ILbinIntOr -> "|" 
    | ILbinIntXor -> "^" 
    | ILbinLessThan -> "<" 
    | ILbinLessEqual -> "<=" 
    | ILbinGtrThan -> ">" 
    | ILbinGtrEqual -> ">=" 
    | ILbinEqual -> "=="
    | ILbinNotEqual -> "!=")

let pp_il_lvalue ppf = function
    ILlvPtr t1 -> fprintf ppf "*%a" pp_temp_id t1
  | ILlvVar(id, ct) -> fprintf ppf "(%s : %s)" id (strof_c_type ct)
  | ILlvTemp t1 -> fprintf ppf "temp(%a)" pp_temp_id t1

let pp_il_expr ppf = function
  | ILexpCoerce(ct, t1) -> fprintf ppf "(%s)%a" (strof_c_type ct) pp_temp_id t1
  | ILexpConstant cc -> pp_print_string ppf (Ctt_printer.strof_ctt_constant cc)
  | ILexpUndefined -> pp_print_string ppf "(undefined)"
  | ILexpBinop(bop, t1, t2) -> fprintf ppf "%a %a %a" pp_temp_id t1 pp_il_binop bop pp_temp_id t2
  | ILexpUnaryop(uop, t1) -> fprintf ppf "%s%a" (Ctt_printer.strof_unaryop uop) pp_temp_id t1
  | ILexpInvoke(lv, tl) -> fprintf ppf "%a(%a)" pp_il_lvalue lv (pp_list pp_temp_id ", ") tl
  | ILexpAddress(lv, flds) -> fprintf ppf "&%a%a" pp_il_lvalue lv (pp_list pp_field "") flds
  | ILexpArgument i -> fprintf ppf "argument(%d)" i
  | ILexpIdent t1 -> pp_temp_id ppf t1
(*
type il_constant_exp_desc = Il1.il_constant_exp_desc =
    ILcexpCoerce of c_type * il_constant_exp
  | ILcexpConstant of c_constants
  | ILcexpBinop of il_binop * il_constant_exp * il_constant_exp
  | ILcexpUnaryop of unaryop * il_constant_exp
  | ILcexpAddress of il_lvalue * field list
  | ILcexpAddFieldOfs of il_constant_exp * field list

and il_constant_exp = Il1.il_constant_exp = 
    { il_cexp_type : c_type; il_cexp_t : il_constant_exp_desc }

type il_initializer = Il1.il_initializer =
    ILinitConstantExp of c_type * il_constant_exp
  | ILinitPostFill
  | ILinitStruct of c_type * (identifier * il_initializer) list
  | ILinitArray of c_type * il_initializer list
  | ILinitAbstractCtt of Cttm_abstree.mexpr (* only for macro emit *)
*)
let pp_il_initalizer ppf i = pp_print_string ppf "(initializer(unimp))"

let rec pp_il ppf = function
    ILstmtIf(ift, tid, l)
    -> fprintf ppf "%a %a then goto %d"
	pp_il_if_type ift pp_temp_id tid l
  | ILstmtSwitch(tid, tlist) ->
      fprintf ppf "switch %a:@\n    @[<v>" pp_temp_id tid;
      List.iter
	(function l, t -> fprintf ppf "%a => %d@," pp_il_switch_label l t) tlist;
      fprintf ppf "@]"
  | ILstmtGoto t -> fprintf ppf "goto %d" t
  | ILstmtReturn None -> fprintf ppf "return"
  | ILstmtReturn (Some t1) -> fprintf ppf "return %a" pp_temp_id t1
  | ILstmtAssign (t1, ct, e1) 
    -> fprintf ppf "%a : %s = %a"
	pp_temp_id t1 (Ctt_printer.strof_c_type ct) pp_il_expr e1
  | ILstmtRead(t1, ct, lv, flds)
    -> fprintf ppf "%a : %s = READ(%a%a)"
	pp_temp_id t1 (Ctt_printer.strof_c_type ct)
	pp_il_lvalue lv (pp_list pp_field "") flds
  | ILstmtWrite(lv, flds, t1)
    -> fprintf ppf "WRITE(%a%a) = %a"
	pp_il_lvalue lv (pp_list pp_field "") flds
	pp_temp_id t1
  | ILstmtInitialize(t1, ct, None)
      -> fprintf ppf "INITIALIZE(%a : %s)"
	  pp_temp_id t1 (Ctt_printer.strof_c_type ct)
  | ILstmtInitialize(t1, ct, Some inits)
      -> fprintf ppf "INITIALIZE(%a : %s) = %a"
	  pp_temp_id t1 (Ctt_printer.strof_c_type ct) pp_il_initalizer inits
  | ILstmtSequence l
      -> fprintf ppf "SEQUENCE:@\n    @[<v>";
	List.iter (fun i -> fprintf ppf "%a@;" pp_il i) l;
	fprintf ppf "@]"
  | ILstmtParallel l
      -> fprintf ppf "PARALLEL:@\n    @[<v>";
	List.iter (fun i -> fprintf ppf "%a@;" pp_il i) l;
	fprintf ppf "@]"

let pp_il_basic_block_base pp_instr bnum ppf b =
  fprintf ppf "BLOCK %d:@\n" bnum;
  fprintf ppf "  from:%a@\n" (pp_list pp_print_int ", ") b.predecessor;
  fprintf ppf "  to  :%a@\n" (pp_list pp_print_int ", ") b.successor;
  if b.immediate_dominator >= 0 then fprintf ppf "  idom:%d@\n" b.immediate_dominator;
  if b.nest_level >= 0 then fprintf ppf "  nlvl:%d@\n" b.nest_level;
  List.iter
    (function i, p ->
      fprintf ppf "    PHI: %a := {%a}@\n" 
	pp_temp_id i (pp_list pp_print_int ", ") (Array.to_list p)) b.phi_function;
  List.iter (fun i -> fprintf ppf "    %a@\n@." pp_instr i) b.code

let pp_il_basic_block = pp_il_basic_block_base pp_il

(*type 'typ il_variable_attribute_base =
    {
      original_name : string option;
      variable_type : 'typ;
      storage_class : local_storage_class
    }

type il_variable_attribute = c_type il_variable_attribute_base
*)

let pp_il_function_base pp_instr pp_typ ppf f =
  fprintf ppf "FUNCTION:@\n";
  fprintf ppf "  maxvar:%d@\n" f.max_variable_number;

  Array.iteri
    (fun i b -> pp_il_basic_block_base pp_instr i ppf b)
    f.body

let pp_il_function ppf f =
  pp_il_function_base pp_il pp_ctt_type f

let pp_il_global_declaration_base pp_instr pp_typ pp_init ppf =
  function
      ILdeclFunction(gs, ct, id, argids, b) ->
	fprintf ppf "%s %s %s(%a): "
	  (Ctt_printer.strof_global_storage_class gs)
	  (Ctt_printer.strof_c_type ct)
	  id
	  (pp_list pp_print_string ", ") argids;
	pp_il_function_base pp_instr pp_typ ppf b
  | ILdeclVariable(gs, ct, id, initopt) ->
      fprintf ppf "%s %s %s"
	(Ctt_printer.strof_global_storage_class gs)
	(Ctt_printer.strof_c_type ct) id;
      match initopt with
	None -> fprintf ppf " : DECL@."
      | Some i -> fprintf ppf " = %a@." pp_init i

let pp_il_global_declaration ppf d = 
  pp_il_global_declaration_base pp_il 
    pp_ctt_type pp_il_initalizer
