(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2007 AIST.

   This file is written by Yutaka Oiwa in 2003-2006. *)

(** ILC0: un-SSAed representation of ILC *)

open Ctt_abstree
open Cttm_abstree
open Il
open Ils
open Ilc

type temp_id = int

type ilc0_desc =
(* basically from statement *)
  | ILC0stmtLabel of identifier
  | ILC0stmtDeclScalar of ils_vartype * c_type * identifier
  | ILC0stmtDeclBulk of ils_vartype * c_type * identifier
  | ILC0stmtInitialize of ils_vartype * c_type * identifier * 
	ils_initializer (* bulk assignment *)
  | ILC0stmtIf of il_if_type * temp_id * string
  | ILC0stmtSwitch of temp_id * ( il_switch_label * string ) list
  | ILC0stmtGoto of string
  | ILC0stmtReturn0
  | ILC0stmtReturn1 of temp_id
  | ILC0stmtReturn2 of temp_id * temp_id
(* basically from expression *)
  | ILC0stmtDefTemp of temp_id * ilc_expr
  | ILC0stmtDefTemp2 of temp_id * temp_id * ilc_expr
  | ILC0stmtReadToTemp1 of temp_id * ils_lvalue * field list * check_attribute
  | ILC0stmtReadToTemp2 of temp_id * temp_id * ils_lvalue * field list * check_attribute
  | ILC0stmtWrite1 of ils_lvalue * field list * temp_id * check_attribute
  | ILC0stmtWrite2 of ils_lvalue * field list * temp_id * temp_id * check_attribute

  | ILC0stmtSequence of ilc0 list
  | ILC0stmtParallel of ilc0 list (* Data-flow parallel *)
  | ILC0stmtAbort of ils_error
  | ILC0stmtAbortIf of ilc_expr * ils_error
and ilc0 = ilc0_desc

type ilc0_function = 
    {
     body : ilc0;
     max_variable_number : int;
     variable_environment : ilr_type il_variable_attribute_base array;
     arguments : (identifier * c_type) array;
     function_attributes : Il.function_attributes
   }

type ilc0_global_declaration = 
    ILC0declFunction of global_storage_class * c_type 
	* identifier * identifier list * ilc0_function
  | ILC0declVariable of 
      global_storage_class * c_type * identifier * ils_initializer option

