(* 
   Part of Fail-Safe C Compiler. Produced by Yutaka Oiwa.
   (c) 2001-2005 Yutaka Oiwa.
   (c) 2005-2008 AIST.

   This file is written by Yutaka Oiwa in 2008. *)

(** Main translator of Fail-Safe C (ILC0 to IL0) *)

(**{6 Main function} *)

(* Mode Switches *)
val clear_castflag_at_memop : bool ref

type compiler_mode =
    MultiModule (** multi-module compilation *)
  | MultiLinker (** linker *)
  | StandAlone  (** stand-alone compilation: single module only, not using FSC linker *)
  | TrustInterModule (** unsafe, linkerless mode of multi-module compilation *)
  | StdlibImplementation (** special mode for compiling standard library *)
val compiler_mode : compiler_mode ref
(** Mode of the compiler. Affects generated codes. *)

val max_local_var_size : Big_int_infix.big_int ref (** The limit of local variable size. *)
val use_signed_compare_for_offset : bool ref (** Use signed comparison for pointer offsets. *)
val force_emit_stack_unwind : bool ref (** Generate stack management code in all functions. *)
val use_optimized_narrow_cast : bool ref (** Use optimized semantics for pointer cast with narrow builtin types. *)

val f :
  genv:Ctt_abstree.environment ->
  Ilc0.ilc0_global_declaration list ->
  Cttm_abstree.cttm_initializer Il0.il0_global_declaration list
(** The main function. *)

(**{6 support routines used in other modules} *)

type size_info =
    Size_byte
  | Size_hword
  | Size_word
  | Size_dword
  | Size_other of Big_int_infix.big_int option
type convfuncs = (Ctt_abstree.c_type * Il.il_lvalue option) list

val encoded_name_of_btype : Ctt_abstree.builtin_type -> string
val encoded_name_of_struct : genv:Ctt_abstree.environment -> int -> string
val encoded_name_of_type_genv :
  genv:Ctt_abstree.environment ->
  ?array_squash:bool -> Ctt_abstree.c_type -> string

type type_properties = {
  size : size_info;
  is_struct : bool;
  is_pointer : bool;
  is_fat : bool;
  is_floating : bool;
  is_indirect_ret : bool;
  packed_t : Ctt_abstree.c_type;
  cdr_t : Ctt_abstree.c_type;
  unpacked_types : Ctt_abstree.c_type list;
  initializer_type : Ctt_abstree.c_type;
  packed_inj_cdr : convfuncs;
  generic_t : Ctt_abstree.c_type;
  samesized_word_t : Ctt_abstree.c_type;
  packed_to_generic : Il.il_lvalue option;
  generic_to_packed : Il.il_lvalue option;
  generic_reader : Il.il_lvalue;
  generic_writer : Il.il_lvalue;
  generic_cdr_t : Ctt_abstree.c_type;
  gen_inj_car : convfuncs;
  gen_inj_cdr : convfuncs;
}
val parse_type_genv :
  genv:Ctt_abstree.environment -> Ctt_abstree.c_type -> type_properties

val translate_c_type_genv :
  genv:Ctt_abstree.environment -> Ctt_abstree.c_type -> Ctt_abstree.c_type

val translate_global_function_name :
  genv:Ctt_abstree.environment -> string -> Ctt_abstree.c_type -> string

val get_global_string_storage_name : genv:Ctt_abstree.environment -> int -> string

exception FG_not_supported

val generate_bridge_function :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.global_storage_class ->
  string ->
  Ctt_abstree.c_type -> string list -> Cttm_abstree.cttm_initializer Il0.il0_global_declaration
val generate_dummy_bridge_function :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.global_storage_class ->
  string ->
  Ctt_abstree.c_type -> string list -> Cttm_abstree.cttm_initializer Il0.il0_global_declaration
val generate_function_stub :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.global_storage_class ->
  string ->
  Ctt_abstree.c_type ->
  string list -> Cttm_abstree.cttm_initializer Il0.il0_global_declaration

(*
val constant_zero : Ctt_abstree.c_constants
val constant_one : Ctt_abstree.c_constants
val make_cttm_expr :
  Cttm_abstree.mexpr_desc -> Ctt_abstree.c_type -> Cttm_abstree.mexpr
val rettype_of_functype : Ctt_abstree.c_type -> Ctt_abstree.c_type
val new_temp_label : int -> int -> string
val sizeof_type_genv :
  genv:Ctt_abstree.environment -> Ctt_abstree.c_type -> Big_int.big_int
val is_numeric : Ctt_abstree.c_type -> bool
val is_pointer : Ctt_abstree.c_type -> bool
val size_is_dword : Big_int_infix.big_int -> bool
*)
val type_base_t : Ctt_abstree.c_type
val type_ofs_t : Ctt_abstree.c_type
(*
val type_signed_ofs_t : Ctt_abstree.c_type
val type_vaddr_t : Ctt_abstree.c_type
val type_word : Ctt_abstree.c_type
val type_dword : Ctt_abstree.c_type
val type_byte : Ctt_abstree.c_type
val type_hword : Ctt_abstree.c_type
val type_value : Ctt_abstree.c_type
val type_dvalue : Ctt_abstree.c_type
val type_ptrvalue : Ctt_abstree.c_type
val type_ptrvalue_ptr : Ctt_abstree.c_type
val type_void_ptr : Ctt_abstree.c_type
val type_ptrinit : Ctt_abstree.c_type
val type_valinit : Ctt_abstree.c_type
val type_dvalinit : Ctt_abstree.c_type
val type_fsc_error : Ctt_abstree.c_type
val type_typeinfo_init : Ctt_abstree.c_type
val type_typeinfo_struct : Ctt_abstree.c_type
val type_function_stub_init : Ctt_abstree.c_type
val type_function_stub : Ctt_abstree.c_type
val type_genfunc : Ctt_abstree.c_type
val make_func_lv :
  Il.identifier ->
  Ctt_abstree.c_type list -> Ctt_abstree.c_type -> Il.il_lvalue
val make_func_lv_R :
  Il.identifier ->
  Ctt_abstree.c_type list -> Ctt_abstree.c_type -> Il.il_lvalue
val lv_cons_val : Il.il_lvalue
val lv_value_of_ptrvalue : Il.il_lvalue
val lv_ptrvalue_of_value : Il.il_lvalue
val lv_cons_dval : Il.il_lvalue
val lv_cons_ptrval : Il.il_lvalue
val lv_value_of_ptr : Il.il_lvalue
val lv_nullexpand_value : Il.il_lvalue
val lv_nullexpand_dvalue : Il.il_lvalue
val lv_base_of_value : Il.il_lvalue
val lv_base_of_dvalue : Il.il_lvalue
val lv_base_of_ptrvalue : Il.il_lvalue
val lv_vaddr_of_value : Il.il_lvalue
val lv_vaddr_of_dvalue : Il.il_lvalue
val lv_ofs_of_ptrvalue : Il.il_lvalue
val lv_ofs_of_value : Il.il_lvalue
val lv_vaddr_of_base_ofs : Il.il_lvalue
val lv_ofs_of_base_vaddr : Il.il_lvalue
val lv_base_remove_castflag : Il.il_lvalue
val lv_base_put_castflag : Il.il_lvalue
val lv_check_cast : Il.il_lvalue
val lv_check_ofs_ok : Il.il_lvalue
val lv_read_byte : Il.il_lvalue
val lv_read_hword : Il.il_lvalue
val lv_read_word : Il.il_lvalue
val lv_read_dword : Il.il_lvalue
val lv_write_byte : Il.il_lvalue
val lv_write_hword : Il.il_lvalue
val lv_write_word : Il.il_lvalue
val lv_write_dword : Il.il_lvalue
val lv_abort : Il.il_lvalue
val lv_alloc_varargs : Il.il_lvalue
val lv_va_end : Il.il_lvalue
val lv_dealloc_varargs_finished : Il.il_lvalue
val lv_put_varargs : Il.il_lvalue
val lv_put_varargs_2 : Il.il_lvalue
val lv_create_stack_frame : Il.il_lvalue
val lv_free_stack_frame : Il.il_lvalue
val lv_double_of_dword : Il.il_lvalue
val lv_float_of_word : Il.il_lvalue
val lv_dword_of_double : Il.il_lvalue
val lv_word_of_float : Il.il_lvalue
val lv_double_of_dvalue : Il.il_lvalue
val lv_float_of_value : Il.il_lvalue
val lv_dvalue_of_double : Il.il_lvalue
val lv_value_of_float : Il.il_lvalue
val lv_get_realoffset_ptr : Il.il_lvalue
val lv_get_funcptr : Il.il_lvalue
val lv_invoke_genfunc : Il.il_lvalue
val lv_value_of_dvalue : Il.il_lvalue
val lv_dvalue_of_value : Il.il_lvalue
val type_fsc_header : Ctt_abstree.c_type
val make_cttm_func_lv :
  Ctt_abstree.identifier ->
  Ctt_abstree.c_type list -> Ctt_abstree.c_type -> Cttm_abstree.mem_object
val lv_cttm_macro_emit_header : Cttm_abstree.mem_object
val encoded_typeinfo_name :
  genv:Ctt_abstree.environment -> Ctt_abstree.c_type -> string
val get_storage_type_array :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.c_type -> Big_int_infix.big_int option -> Ctt_abstree.c_type
val size_and_elemtype_of_onmemory_type :
  ?incomplete_ok:bool ->
  Ctt_abstree.c_type -> Big_int.big_int option * Ctt_abstree.c_type
val storage_for_typeinfo_constant :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.c_type ->
  Ctt_abstree.c_type * Ctt_abstree.c_type * Il.il_lvalue *
  (string * Ctt_abstree.c_type) list
val lv_alloc_hvar : Il.il_lvalue
val check_size_of_local_variable :
  genv:Ctt_abstree.environment -> string -> Ctt_abstree.c_type -> unit
val make_coerce_from_vaddr : Ctt_abstree.c_type -> Il.temp_id -> Il.il_expr
val make_coerce_to_vaddr :
  Ctt_abstree.c_type -> Ctt_abstree.c_type -> Il.temp_id -> Il.il_expr
val make_coerce_to_value2 :
  Ctt_abstree.c_type -> 'a -> Il.temp_id * Il.temp_id -> Il.il_expr
val original_type_from_val_type : Ils.ilr_type -> Ctt_abstree.c_type
val is_type_polymorphic_func : genv:'a -> Ctt_abstree.c_type -> bool
val translate_c_type_funcarg :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.c_type -> Ctt_abstree.c_type list -> Ctt_abstree.c_type list
val get_storage_type :
  genv:Ctt_abstree.environment ->
  ?incomplete_ok:bool -> Ctt_abstree.c_type -> Ctt_abstree.c_type
val translate_global_bridge_function_name : genv:'a -> string -> 'b -> string
val translate_ptr_local_heap_variable_name :
  genv:Ctt_abstree.environment -> string -> Ctt_abstree.c_type -> string
val translate_local_stack_variable_name :
  genv:Ctt_abstree.environment -> string -> Ctt_abstree.c_type -> string
val translate_fields :
  genv:Ctt_abstree.environment ->
  (string * Ctt_abstree.c_type) list -> (string * Ctt_abstree.c_type) list
val get_type_of_field : Ctt_abstree.c_type -> (string * Ctt_abstree.c_type) list -> Ctt_abstree.c_type
val cttm_storage_for_global_string :
  genv:Ctt_abstree.environment ->
  string ->
  Ctt_abstree.c_type * Ctt_abstree.c_type * Cttm_abstree.mem_object *
  (string * Ctt_abstree.c_type) list
val storage_for_global_string :
  genv:Ctt_abstree.environment ->
  string ->
  Ctt_abstree.c_type * Ctt_abstree.c_type * Il.il_lvalue *
  (string * Ctt_abstree.c_type) list
val get_field_offset :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.c_type ->
  (Ctt_abstree.identifier * Ctt_abstree.c_type) list ->
  Ctt_abstree.struct_ofsinfo
val cttm_macro_for_blockheader :
  genv:Ctt_abstree.environment ->
  Ctt_abstree.c_type -> Big_int_infix.big_int option -> Cttm_abstree.mexpr
val exp_of_error_constant : Ils.ils_error -> Il.il_expr
val resolve_reference_type :
  Ils.ilr_type -> Ctt_abstree.c_type * Ctt_abstree.c_type
val constant_of_ilsinitofs :
  Ils.ils_initializer_offset -> Ctt_abstree.c_constants
val is_pointer_cast :
  region_target_type:Ctt_abstree.c_type ->
  value_target_type:Ctt_abstree.c_type ->
  unknown_p:bool -> offset:Big_int_infix.big_int -> bool
val select_macro_emit :
  genv:Ctt_abstree.environment ->
  is_global:bool ->
  Ctt_abstree.c_type ->
  Ctt_abstree.c_type option * Ctt_abstree.c_type *
  Cttm_abstree.mem_object option * Ctt_abstree.c_type
val is_empty_initialization : Ils.ils_initializer -> bool
val translate_funcdecl_to_prototype :
  'a Il0.il0_global_declaration -> 'b Il0.il0_global_declaration
*)
