/*
   This file is automatically generated.
 */

/**
 * @file math/fma.test.c
 */
#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

#ifndef NAN
#define NAN (0.0f/0.0f)
#endif

#ifndef INF
#define INF (1.0f/0.0f)
#endif

#define ABS_FLOAT       fabsf
#define ABS_DOUBLE      fabs
#define ABS_LONG_DOUBLE fabsl

#define TOLERANCE_FLOAT       0x1p-20
#define TOLERANCE_DOUBLE      0x1p-50
#define TOLERANCE_LONG_DOUBLE 0x1p-60

#define RET_MINIMUM_FLOAT       0x1p-126
#define RET_MINIMUM_DOUBLE      0x1p-1022
#define RET_MINIMUM_LONG_DOUBLE 0x1p-16382

#define max(a, b) ((a > b) ? a : b)

#define CMP_FLOAT(x, t) \
  static int compare_##x(t actual, t expected) { \
    if (isnan(actual) && isnan(expected)) \
      return 1; \
    else if (isinf(actual) && isinf(expected)) \
      return ((actual > (t)0.0) == (expected > (t)0.0)); \
    else \
      return (ABS_##x(actual - expected) < max(ABS_##x(expected * TOLERANCE_##x), RET_MINIMUM_##x)); \
  }

#define CMP_INT(x, t) \
  static int compare_##x(t actual, t expected) { return (actual == expected); }

CMP_FLOAT(FLOAT, float)
CMP_FLOAT(DOUBLE, double)
/* CMP_FLOAT(LONG_DOUBLE, long double) */

CMP_INT(INT, int)
CMP_INT(LONG, long)
CMP_INT(LONG_LONG, long long)

#define _CMP(t, a, e) compare_##t(a, e)
#define CMP(t, a, e) _CMP(t, a, e)

/**
 * @testname fma_2
 * @testfor fma
 */
TEST_CASE(fma_2)
{

  static struct {
    double a;

    double a2;

    double a3;

  } args[] = {
    { 0x0p+0, 0x0p+0, 0x0p+0 },
    { 0x0p+0, 0x0p+0, -0x0p+0 },
    { 0x0p+0, 0x0p+0, 0x1p+0 },
    { 0x0p+0, 0x0p+0, -0x1p+0 },
    { 0x0p+0, 0x0p+0, 0x1p-1022 },
    { 0x0p+0, 0x0p+0, -0x1p-1022 },
    { 0x0p+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x0p+0, INF },
    { 0x0p+0, 0x0p+0, -INF },
    { 0x0p+0, 0x0p+0, NAN },
    { 0x0p+0, 0x0p+0, NAN },
    { 0x0p+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x0p+0, 0x0p+0 },
    { 0x0p+0, -0x0p+0, -0x0p+0 },
    { 0x0p+0, -0x0p+0, 0x1p+0 },
    { 0x0p+0, -0x0p+0, -0x1p+0 },
    { 0x0p+0, -0x0p+0, 0x1p-1022 },
    { 0x0p+0, -0x0p+0, -0x1p-1022 },
    { 0x0p+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x0p+0, INF },
    { 0x0p+0, -0x0p+0, -INF },
    { 0x0p+0, -0x0p+0, NAN },
    { 0x0p+0, -0x0p+0, NAN },
    { 0x0p+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x0p+0, 0x1p+0, 0x0p+0 },
    { 0x0p+0, 0x1p+0, -0x0p+0 },
    { 0x0p+0, 0x1p+0, 0x1p+0 },
    { 0x0p+0, 0x1p+0, -0x1p+0 },
    { 0x0p+0, 0x1p+0, 0x1p-1022 },
    { 0x0p+0, 0x1p+0, -0x1p-1022 },
    { 0x0p+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1p+0, INF },
    { 0x0p+0, 0x1p+0, -INF },
    { 0x0p+0, 0x1p+0, NAN },
    { 0x0p+0, 0x1p+0, NAN },
    { 0x0p+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x1p+0, 0x0p+0 },
    { 0x0p+0, -0x1p+0, -0x0p+0 },
    { 0x0p+0, -0x1p+0, 0x1p+0 },
    { 0x0p+0, -0x1p+0, -0x1p+0 },
    { 0x0p+0, -0x1p+0, 0x1p-1022 },
    { 0x0p+0, -0x1p+0, -0x1p-1022 },
    { 0x0p+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1p+0, INF },
    { 0x0p+0, -0x1p+0, -INF },
    { 0x0p+0, -0x1p+0, NAN },
    { 0x0p+0, -0x1p+0, NAN },
    { 0x0p+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x0p+0, 0x1p-1022, 0x0p+0 },
    { 0x0p+0, 0x1p-1022, -0x0p+0 },
    { 0x0p+0, 0x1p-1022, 0x1p+0 },
    { 0x0p+0, 0x1p-1022, -0x1p+0 },
    { 0x0p+0, 0x1p-1022, 0x1p-1022 },
    { 0x0p+0, 0x1p-1022, -0x1p-1022 },
    { 0x0p+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1p-1022, INF },
    { 0x0p+0, 0x1p-1022, -INF },
    { 0x0p+0, 0x1p-1022, NAN },
    { 0x0p+0, 0x1p-1022, NAN },
    { 0x0p+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x1p-1022, 0x0p+0 },
    { 0x0p+0, -0x1p-1022, -0x0p+0 },
    { 0x0p+0, -0x1p-1022, 0x1p+0 },
    { 0x0p+0, -0x1p-1022, -0x1p+0 },
    { 0x0p+0, -0x1p-1022, 0x1p-1022 },
    { 0x0p+0, -0x1p-1022, -0x1p-1022 },
    { 0x0p+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1p-1022, INF },
    { 0x0p+0, -0x1p-1022, -INF },
    { 0x0p+0, -0x1p-1022, NAN },
    { 0x0p+0, -0x1p-1022, NAN },
    { 0x0p+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, INF },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -INF },
    { 0x0p+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x0p+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, INF },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -INF },
    { 0x0p+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x0p+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x0p+0, INF, 0x0p+0 },
    { 0x0p+0, INF, -0x0p+0 },
    { 0x0p+0, INF, 0x1p+0 },
    { 0x0p+0, INF, -0x1p+0 },
    { 0x0p+0, INF, 0x1p-1022 },
    { 0x0p+0, INF, -0x1p-1022 },
    { 0x0p+0, INF, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, INF, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, INF, INF },
    { 0x0p+0, INF, -INF },
    { 0x0p+0, INF, NAN },
    { 0x0p+0, INF, NAN },
    { 0x0p+0, INF, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, INF, 0x1.199999999999ap+0 },
    { 0x0p+0, INF, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, INF, -0x1.199999999999ap+0 },
    { 0x0p+0, -INF, 0x0p+0 },
    { 0x0p+0, -INF, -0x0p+0 },
    { 0x0p+0, -INF, 0x1p+0 },
    { 0x0p+0, -INF, -0x1p+0 },
    { 0x0p+0, -INF, 0x1p-1022 },
    { 0x0p+0, -INF, -0x1p-1022 },
    { 0x0p+0, -INF, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -INF, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -INF, INF },
    { 0x0p+0, -INF, -INF },
    { 0x0p+0, -INF, NAN },
    { 0x0p+0, -INF, NAN },
    { 0x0p+0, -INF, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -INF, 0x1.199999999999ap+0 },
    { 0x0p+0, -INF, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -INF, -0x1.199999999999ap+0 },
    { 0x0p+0, NAN, 0x0p+0 },
    { 0x0p+0, NAN, -0x0p+0 },
    { 0x0p+0, NAN, 0x1p+0 },
    { 0x0p+0, NAN, -0x1p+0 },
    { 0x0p+0, NAN, 0x1p-1022 },
    { 0x0p+0, NAN, -0x1p-1022 },
    { 0x0p+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, NAN, INF },
    { 0x0p+0, NAN, -INF },
    { 0x0p+0, NAN, NAN },
    { 0x0p+0, NAN, NAN },
    { 0x0p+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, NAN, 0x1.199999999999ap+0 },
    { 0x0p+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, NAN, -0x1.199999999999ap+0 },
    { 0x0p+0, NAN, 0x0p+0 },
    { 0x0p+0, NAN, -0x0p+0 },
    { 0x0p+0, NAN, 0x1p+0 },
    { 0x0p+0, NAN, -0x1p+0 },
    { 0x0p+0, NAN, 0x1p-1022 },
    { 0x0p+0, NAN, -0x1p-1022 },
    { 0x0p+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, NAN, INF },
    { 0x0p+0, NAN, -INF },
    { 0x0p+0, NAN, NAN },
    { 0x0p+0, NAN, NAN },
    { 0x0p+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, NAN, 0x1.199999999999ap+0 },
    { 0x0p+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, NAN, -0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, INF },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -INF },
    { 0x0p+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x0p+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x0p+0, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x0p+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x0p+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, 0x1.199999999999ap+0, INF },
    { 0x0p+0, 0x1.199999999999ap+0, -INF },
    { 0x0p+0, 0x1.199999999999ap+0, NAN },
    { 0x0p+0, 0x1.199999999999ap+0, NAN },
    { 0x0p+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, INF },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -INF },
    { 0x0p+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x0p+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x0p+0, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x0p+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x0p+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x0p+0, -0x1.199999999999ap+0, INF },
    { 0x0p+0, -0x1.199999999999ap+0, -INF },
    { 0x0p+0, -0x1.199999999999ap+0, NAN },
    { 0x0p+0, -0x1.199999999999ap+0, NAN },
    { 0x0p+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x0p+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x0p+0, 0x0p+0 },
    { -0x0p+0, 0x0p+0, -0x0p+0 },
    { -0x0p+0, 0x0p+0, 0x1p+0 },
    { -0x0p+0, 0x0p+0, -0x1p+0 },
    { -0x0p+0, 0x0p+0, 0x1p-1022 },
    { -0x0p+0, 0x0p+0, -0x1p-1022 },
    { -0x0p+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x0p+0, INF },
    { -0x0p+0, 0x0p+0, -INF },
    { -0x0p+0, 0x0p+0, NAN },
    { -0x0p+0, 0x0p+0, NAN },
    { -0x0p+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x0p+0, 0x0p+0 },
    { -0x0p+0, -0x0p+0, -0x0p+0 },
    { -0x0p+0, -0x0p+0, 0x1p+0 },
    { -0x0p+0, -0x0p+0, -0x1p+0 },
    { -0x0p+0, -0x0p+0, 0x1p-1022 },
    { -0x0p+0, -0x0p+0, -0x1p-1022 },
    { -0x0p+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x0p+0, INF },
    { -0x0p+0, -0x0p+0, -INF },
    { -0x0p+0, -0x0p+0, NAN },
    { -0x0p+0, -0x0p+0, NAN },
    { -0x0p+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x1p+0, 0x0p+0 },
    { -0x0p+0, 0x1p+0, -0x0p+0 },
    { -0x0p+0, 0x1p+0, 0x1p+0 },
    { -0x0p+0, 0x1p+0, -0x1p+0 },
    { -0x0p+0, 0x1p+0, 0x1p-1022 },
    { -0x0p+0, 0x1p+0, -0x1p-1022 },
    { -0x0p+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1p+0, INF },
    { -0x0p+0, 0x1p+0, -INF },
    { -0x0p+0, 0x1p+0, NAN },
    { -0x0p+0, 0x1p+0, NAN },
    { -0x0p+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x1p+0, 0x0p+0 },
    { -0x0p+0, -0x1p+0, -0x0p+0 },
    { -0x0p+0, -0x1p+0, 0x1p+0 },
    { -0x0p+0, -0x1p+0, -0x1p+0 },
    { -0x0p+0, -0x1p+0, 0x1p-1022 },
    { -0x0p+0, -0x1p+0, -0x1p-1022 },
    { -0x0p+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1p+0, INF },
    { -0x0p+0, -0x1p+0, -INF },
    { -0x0p+0, -0x1p+0, NAN },
    { -0x0p+0, -0x1p+0, NAN },
    { -0x0p+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x1p-1022, 0x0p+0 },
    { -0x0p+0, 0x1p-1022, -0x0p+0 },
    { -0x0p+0, 0x1p-1022, 0x1p+0 },
    { -0x0p+0, 0x1p-1022, -0x1p+0 },
    { -0x0p+0, 0x1p-1022, 0x1p-1022 },
    { -0x0p+0, 0x1p-1022, -0x1p-1022 },
    { -0x0p+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1p-1022, INF },
    { -0x0p+0, 0x1p-1022, -INF },
    { -0x0p+0, 0x1p-1022, NAN },
    { -0x0p+0, 0x1p-1022, NAN },
    { -0x0p+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x1p-1022, 0x0p+0 },
    { -0x0p+0, -0x1p-1022, -0x0p+0 },
    { -0x0p+0, -0x1p-1022, 0x1p+0 },
    { -0x0p+0, -0x1p-1022, -0x1p+0 },
    { -0x0p+0, -0x1p-1022, 0x1p-1022 },
    { -0x0p+0, -0x1p-1022, -0x1p-1022 },
    { -0x0p+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1p-1022, INF },
    { -0x0p+0, -0x1p-1022, -INF },
    { -0x0p+0, -0x1p-1022, NAN },
    { -0x0p+0, -0x1p-1022, NAN },
    { -0x0p+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, INF },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -INF },
    { -0x0p+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x0p+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, INF },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -INF },
    { -0x0p+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x0p+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x0p+0, INF, 0x0p+0 },
    { -0x0p+0, INF, -0x0p+0 },
    { -0x0p+0, INF, 0x1p+0 },
    { -0x0p+0, INF, -0x1p+0 },
    { -0x0p+0, INF, 0x1p-1022 },
    { -0x0p+0, INF, -0x1p-1022 },
    { -0x0p+0, INF, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, INF, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, INF, INF },
    { -0x0p+0, INF, -INF },
    { -0x0p+0, INF, NAN },
    { -0x0p+0, INF, NAN },
    { -0x0p+0, INF, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, INF, 0x1.199999999999ap+0 },
    { -0x0p+0, INF, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, INF, -0x1.199999999999ap+0 },
    { -0x0p+0, -INF, 0x0p+0 },
    { -0x0p+0, -INF, -0x0p+0 },
    { -0x0p+0, -INF, 0x1p+0 },
    { -0x0p+0, -INF, -0x1p+0 },
    { -0x0p+0, -INF, 0x1p-1022 },
    { -0x0p+0, -INF, -0x1p-1022 },
    { -0x0p+0, -INF, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -INF, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -INF, INF },
    { -0x0p+0, -INF, -INF },
    { -0x0p+0, -INF, NAN },
    { -0x0p+0, -INF, NAN },
    { -0x0p+0, -INF, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -INF, 0x1.199999999999ap+0 },
    { -0x0p+0, -INF, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -INF, -0x1.199999999999ap+0 },
    { -0x0p+0, NAN, 0x0p+0 },
    { -0x0p+0, NAN, -0x0p+0 },
    { -0x0p+0, NAN, 0x1p+0 },
    { -0x0p+0, NAN, -0x1p+0 },
    { -0x0p+0, NAN, 0x1p-1022 },
    { -0x0p+0, NAN, -0x1p-1022 },
    { -0x0p+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, NAN, INF },
    { -0x0p+0, NAN, -INF },
    { -0x0p+0, NAN, NAN },
    { -0x0p+0, NAN, NAN },
    { -0x0p+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, NAN, 0x1.199999999999ap+0 },
    { -0x0p+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, NAN, -0x1.199999999999ap+0 },
    { -0x0p+0, NAN, 0x0p+0 },
    { -0x0p+0, NAN, -0x0p+0 },
    { -0x0p+0, NAN, 0x1p+0 },
    { -0x0p+0, NAN, -0x1p+0 },
    { -0x0p+0, NAN, 0x1p-1022 },
    { -0x0p+0, NAN, -0x1p-1022 },
    { -0x0p+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, NAN, INF },
    { -0x0p+0, NAN, -INF },
    { -0x0p+0, NAN, NAN },
    { -0x0p+0, NAN, NAN },
    { -0x0p+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, NAN, 0x1.199999999999ap+0 },
    { -0x0p+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, NAN, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, INF },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -INF },
    { -0x0p+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x0p+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x0p+0, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x0p+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x0p+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, 0x1.199999999999ap+0, INF },
    { -0x0p+0, 0x1.199999999999ap+0, -INF },
    { -0x0p+0, 0x1.199999999999ap+0, NAN },
    { -0x0p+0, 0x1.199999999999ap+0, NAN },
    { -0x0p+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, INF },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -INF },
    { -0x0p+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x0p+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x0p+0, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x0p+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x0p+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x0p+0, -0x1.199999999999ap+0, INF },
    { -0x0p+0, -0x1.199999999999ap+0, -INF },
    { -0x0p+0, -0x1.199999999999ap+0, NAN },
    { -0x0p+0, -0x1.199999999999ap+0, NAN },
    { -0x0p+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x0p+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x0p+0, 0x0p+0 },
    { 0x1p+0, 0x0p+0, -0x0p+0 },
    { 0x1p+0, 0x0p+0, 0x1p+0 },
    { 0x1p+0, 0x0p+0, -0x1p+0 },
    { 0x1p+0, 0x0p+0, 0x1p-1022 },
    { 0x1p+0, 0x0p+0, -0x1p-1022 },
    { 0x1p+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x0p+0, INF },
    { 0x1p+0, 0x0p+0, -INF },
    { 0x1p+0, 0x0p+0, NAN },
    { 0x1p+0, 0x0p+0, NAN },
    { 0x1p+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x0p+0, 0x0p+0 },
    { 0x1p+0, -0x0p+0, -0x0p+0 },
    { 0x1p+0, -0x0p+0, 0x1p+0 },
    { 0x1p+0, -0x0p+0, -0x1p+0 },
    { 0x1p+0, -0x0p+0, 0x1p-1022 },
    { 0x1p+0, -0x0p+0, -0x1p-1022 },
    { 0x1p+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x0p+0, INF },
    { 0x1p+0, -0x0p+0, -INF },
    { 0x1p+0, -0x0p+0, NAN },
    { 0x1p+0, -0x0p+0, NAN },
    { 0x1p+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x1p+0, 0x0p+0 },
    { 0x1p+0, 0x1p+0, -0x0p+0 },
    { 0x1p+0, 0x1p+0, 0x1p+0 },
    { 0x1p+0, 0x1p+0, -0x1p+0 },
    { 0x1p+0, 0x1p+0, 0x1p-1022 },
    { 0x1p+0, 0x1p+0, -0x1p-1022 },
    { 0x1p+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1p+0, INF },
    { 0x1p+0, 0x1p+0, -INF },
    { 0x1p+0, 0x1p+0, NAN },
    { 0x1p+0, 0x1p+0, NAN },
    { 0x1p+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x1p+0, 0x0p+0 },
    { 0x1p+0, -0x1p+0, -0x0p+0 },
    { 0x1p+0, -0x1p+0, 0x1p+0 },
    { 0x1p+0, -0x1p+0, -0x1p+0 },
    { 0x1p+0, -0x1p+0, 0x1p-1022 },
    { 0x1p+0, -0x1p+0, -0x1p-1022 },
    { 0x1p+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1p+0, INF },
    { 0x1p+0, -0x1p+0, -INF },
    { 0x1p+0, -0x1p+0, NAN },
    { 0x1p+0, -0x1p+0, NAN },
    { 0x1p+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x1p-1022, 0x0p+0 },
    { 0x1p+0, 0x1p-1022, -0x0p+0 },
    { 0x1p+0, 0x1p-1022, 0x1p+0 },
    { 0x1p+0, 0x1p-1022, -0x1p+0 },
    { 0x1p+0, 0x1p-1022, 0x1p-1022 },
    { 0x1p+0, 0x1p-1022, -0x1p-1022 },
    { 0x1p+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1p-1022, INF },
    { 0x1p+0, 0x1p-1022, -INF },
    { 0x1p+0, 0x1p-1022, NAN },
    { 0x1p+0, 0x1p-1022, NAN },
    { 0x1p+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x1p-1022, 0x0p+0 },
    { 0x1p+0, -0x1p-1022, -0x0p+0 },
    { 0x1p+0, -0x1p-1022, 0x1p+0 },
    { 0x1p+0, -0x1p-1022, -0x1p+0 },
    { 0x1p+0, -0x1p-1022, 0x1p-1022 },
    { 0x1p+0, -0x1p-1022, -0x1p-1022 },
    { 0x1p+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1p-1022, INF },
    { 0x1p+0, -0x1p-1022, -INF },
    { 0x1p+0, -0x1p-1022, NAN },
    { 0x1p+0, -0x1p-1022, NAN },
    { 0x1p+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, INF },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -INF },
    { 0x1p+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x1p+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, INF },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -INF },
    { 0x1p+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x1p+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1p+0, INF, 0x0p+0 },
    { 0x1p+0, INF, -0x0p+0 },
    { 0x1p+0, INF, 0x1p+0 },
    { 0x1p+0, INF, -0x1p+0 },
    { 0x1p+0, INF, 0x1p-1022 },
    { 0x1p+0, INF, -0x1p-1022 },
    { 0x1p+0, INF, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, INF, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, INF, INF },
    { 0x1p+0, INF, -INF },
    { 0x1p+0, INF, NAN },
    { 0x1p+0, INF, NAN },
    { 0x1p+0, INF, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, INF, 0x1.199999999999ap+0 },
    { 0x1p+0, INF, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, INF, -0x1.199999999999ap+0 },
    { 0x1p+0, -INF, 0x0p+0 },
    { 0x1p+0, -INF, -0x0p+0 },
    { 0x1p+0, -INF, 0x1p+0 },
    { 0x1p+0, -INF, -0x1p+0 },
    { 0x1p+0, -INF, 0x1p-1022 },
    { 0x1p+0, -INF, -0x1p-1022 },
    { 0x1p+0, -INF, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -INF, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -INF, INF },
    { 0x1p+0, -INF, -INF },
    { 0x1p+0, -INF, NAN },
    { 0x1p+0, -INF, NAN },
    { 0x1p+0, -INF, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -INF, 0x1.199999999999ap+0 },
    { 0x1p+0, -INF, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -INF, -0x1.199999999999ap+0 },
    { 0x1p+0, NAN, 0x0p+0 },
    { 0x1p+0, NAN, -0x0p+0 },
    { 0x1p+0, NAN, 0x1p+0 },
    { 0x1p+0, NAN, -0x1p+0 },
    { 0x1p+0, NAN, 0x1p-1022 },
    { 0x1p+0, NAN, -0x1p-1022 },
    { 0x1p+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, NAN, INF },
    { 0x1p+0, NAN, -INF },
    { 0x1p+0, NAN, NAN },
    { 0x1p+0, NAN, NAN },
    { 0x1p+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, NAN, 0x1.199999999999ap+0 },
    { 0x1p+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, NAN, -0x1.199999999999ap+0 },
    { 0x1p+0, NAN, 0x0p+0 },
    { 0x1p+0, NAN, -0x0p+0 },
    { 0x1p+0, NAN, 0x1p+0 },
    { 0x1p+0, NAN, -0x1p+0 },
    { 0x1p+0, NAN, 0x1p-1022 },
    { 0x1p+0, NAN, -0x1p-1022 },
    { 0x1p+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, NAN, INF },
    { 0x1p+0, NAN, -INF },
    { 0x1p+0, NAN, NAN },
    { 0x1p+0, NAN, NAN },
    { 0x1p+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, NAN, 0x1.199999999999ap+0 },
    { 0x1p+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, NAN, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, INF },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -INF },
    { 0x1p+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x1p+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1p+0, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1p+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1p+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, 0x1.199999999999ap+0, INF },
    { 0x1p+0, 0x1.199999999999ap+0, -INF },
    { 0x1p+0, 0x1.199999999999ap+0, NAN },
    { 0x1p+0, 0x1.199999999999ap+0, NAN },
    { 0x1p+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, INF },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -INF },
    { 0x1p+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x1p+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x1p+0, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x1p+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1p+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1p+0, -0x1.199999999999ap+0, INF },
    { 0x1p+0, -0x1.199999999999ap+0, -INF },
    { 0x1p+0, -0x1.199999999999ap+0, NAN },
    { 0x1p+0, -0x1.199999999999ap+0, NAN },
    { 0x1p+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1p+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x0p+0, 0x0p+0 },
    { -0x1p+0, 0x0p+0, -0x0p+0 },
    { -0x1p+0, 0x0p+0, 0x1p+0 },
    { -0x1p+0, 0x0p+0, -0x1p+0 },
    { -0x1p+0, 0x0p+0, 0x1p-1022 },
    { -0x1p+0, 0x0p+0, -0x1p-1022 },
    { -0x1p+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x0p+0, INF },
    { -0x1p+0, 0x0p+0, -INF },
    { -0x1p+0, 0x0p+0, NAN },
    { -0x1p+0, 0x0p+0, NAN },
    { -0x1p+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x0p+0, 0x0p+0 },
    { -0x1p+0, -0x0p+0, -0x0p+0 },
    { -0x1p+0, -0x0p+0, 0x1p+0 },
    { -0x1p+0, -0x0p+0, -0x1p+0 },
    { -0x1p+0, -0x0p+0, 0x1p-1022 },
    { -0x1p+0, -0x0p+0, -0x1p-1022 },
    { -0x1p+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x0p+0, INF },
    { -0x1p+0, -0x0p+0, -INF },
    { -0x1p+0, -0x0p+0, NAN },
    { -0x1p+0, -0x0p+0, NAN },
    { -0x1p+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x1p+0, 0x0p+0 },
    { -0x1p+0, 0x1p+0, -0x0p+0 },
    { -0x1p+0, 0x1p+0, 0x1p+0 },
    { -0x1p+0, 0x1p+0, -0x1p+0 },
    { -0x1p+0, 0x1p+0, 0x1p-1022 },
    { -0x1p+0, 0x1p+0, -0x1p-1022 },
    { -0x1p+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1p+0, INF },
    { -0x1p+0, 0x1p+0, -INF },
    { -0x1p+0, 0x1p+0, NAN },
    { -0x1p+0, 0x1p+0, NAN },
    { -0x1p+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x1p+0, 0x0p+0 },
    { -0x1p+0, -0x1p+0, -0x0p+0 },
    { -0x1p+0, -0x1p+0, 0x1p+0 },
    { -0x1p+0, -0x1p+0, -0x1p+0 },
    { -0x1p+0, -0x1p+0, 0x1p-1022 },
    { -0x1p+0, -0x1p+0, -0x1p-1022 },
    { -0x1p+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1p+0, INF },
    { -0x1p+0, -0x1p+0, -INF },
    { -0x1p+0, -0x1p+0, NAN },
    { -0x1p+0, -0x1p+0, NAN },
    { -0x1p+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x1p-1022, 0x0p+0 },
    { -0x1p+0, 0x1p-1022, -0x0p+0 },
    { -0x1p+0, 0x1p-1022, 0x1p+0 },
    { -0x1p+0, 0x1p-1022, -0x1p+0 },
    { -0x1p+0, 0x1p-1022, 0x1p-1022 },
    { -0x1p+0, 0x1p-1022, -0x1p-1022 },
    { -0x1p+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1p-1022, INF },
    { -0x1p+0, 0x1p-1022, -INF },
    { -0x1p+0, 0x1p-1022, NAN },
    { -0x1p+0, 0x1p-1022, NAN },
    { -0x1p+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x1p-1022, 0x0p+0 },
    { -0x1p+0, -0x1p-1022, -0x0p+0 },
    { -0x1p+0, -0x1p-1022, 0x1p+0 },
    { -0x1p+0, -0x1p-1022, -0x1p+0 },
    { -0x1p+0, -0x1p-1022, 0x1p-1022 },
    { -0x1p+0, -0x1p-1022, -0x1p-1022 },
    { -0x1p+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1p-1022, INF },
    { -0x1p+0, -0x1p-1022, -INF },
    { -0x1p+0, -0x1p-1022, NAN },
    { -0x1p+0, -0x1p-1022, NAN },
    { -0x1p+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, INF },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -INF },
    { -0x1p+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x1p+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, INF },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -INF },
    { -0x1p+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x1p+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1p+0, INF, 0x0p+0 },
    { -0x1p+0, INF, -0x0p+0 },
    { -0x1p+0, INF, 0x1p+0 },
    { -0x1p+0, INF, -0x1p+0 },
    { -0x1p+0, INF, 0x1p-1022 },
    { -0x1p+0, INF, -0x1p-1022 },
    { -0x1p+0, INF, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, INF, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, INF, INF },
    { -0x1p+0, INF, -INF },
    { -0x1p+0, INF, NAN },
    { -0x1p+0, INF, NAN },
    { -0x1p+0, INF, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, INF, 0x1.199999999999ap+0 },
    { -0x1p+0, INF, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, INF, -0x1.199999999999ap+0 },
    { -0x1p+0, -INF, 0x0p+0 },
    { -0x1p+0, -INF, -0x0p+0 },
    { -0x1p+0, -INF, 0x1p+0 },
    { -0x1p+0, -INF, -0x1p+0 },
    { -0x1p+0, -INF, 0x1p-1022 },
    { -0x1p+0, -INF, -0x1p-1022 },
    { -0x1p+0, -INF, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -INF, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -INF, INF },
    { -0x1p+0, -INF, -INF },
    { -0x1p+0, -INF, NAN },
    { -0x1p+0, -INF, NAN },
    { -0x1p+0, -INF, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -INF, 0x1.199999999999ap+0 },
    { -0x1p+0, -INF, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -INF, -0x1.199999999999ap+0 },
    { -0x1p+0, NAN, 0x0p+0 },
    { -0x1p+0, NAN, -0x0p+0 },
    { -0x1p+0, NAN, 0x1p+0 },
    { -0x1p+0, NAN, -0x1p+0 },
    { -0x1p+0, NAN, 0x1p-1022 },
    { -0x1p+0, NAN, -0x1p-1022 },
    { -0x1p+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, NAN, INF },
    { -0x1p+0, NAN, -INF },
    { -0x1p+0, NAN, NAN },
    { -0x1p+0, NAN, NAN },
    { -0x1p+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, NAN, 0x1.199999999999ap+0 },
    { -0x1p+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, NAN, -0x1.199999999999ap+0 },
    { -0x1p+0, NAN, 0x0p+0 },
    { -0x1p+0, NAN, -0x0p+0 },
    { -0x1p+0, NAN, 0x1p+0 },
    { -0x1p+0, NAN, -0x1p+0 },
    { -0x1p+0, NAN, 0x1p-1022 },
    { -0x1p+0, NAN, -0x1p-1022 },
    { -0x1p+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, NAN, INF },
    { -0x1p+0, NAN, -INF },
    { -0x1p+0, NAN, NAN },
    { -0x1p+0, NAN, NAN },
    { -0x1p+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, NAN, 0x1.199999999999ap+0 },
    { -0x1p+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, NAN, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, INF },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -INF },
    { -0x1p+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x1p+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x1p+0, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x1p+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1p+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, 0x1.199999999999ap+0, INF },
    { -0x1p+0, 0x1.199999999999ap+0, -INF },
    { -0x1p+0, 0x1.199999999999ap+0, NAN },
    { -0x1p+0, 0x1.199999999999ap+0, NAN },
    { -0x1p+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, INF },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -INF },
    { -0x1p+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x1p+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1p+0, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1p+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1p+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1p+0, -0x1.199999999999ap+0, INF },
    { -0x1p+0, -0x1.199999999999ap+0, -INF },
    { -0x1p+0, -0x1.199999999999ap+0, NAN },
    { -0x1p+0, -0x1.199999999999ap+0, NAN },
    { -0x1p+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1p+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x0p+0, 0x0p+0 },
    { 0x1p-1022, 0x0p+0, -0x0p+0 },
    { 0x1p-1022, 0x0p+0, 0x1p+0 },
    { 0x1p-1022, 0x0p+0, -0x1p+0 },
    { 0x1p-1022, 0x0p+0, 0x1p-1022 },
    { 0x1p-1022, 0x0p+0, -0x1p-1022 },
    { 0x1p-1022, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x0p+0, INF },
    { 0x1p-1022, 0x0p+0, -INF },
    { 0x1p-1022, 0x0p+0, NAN },
    { 0x1p-1022, 0x0p+0, NAN },
    { 0x1p-1022, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x0p+0, 0x0p+0 },
    { 0x1p-1022, -0x0p+0, -0x0p+0 },
    { 0x1p-1022, -0x0p+0, 0x1p+0 },
    { 0x1p-1022, -0x0p+0, -0x1p+0 },
    { 0x1p-1022, -0x0p+0, 0x1p-1022 },
    { 0x1p-1022, -0x0p+0, -0x1p-1022 },
    { 0x1p-1022, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x0p+0, INF },
    { 0x1p-1022, -0x0p+0, -INF },
    { 0x1p-1022, -0x0p+0, NAN },
    { 0x1p-1022, -0x0p+0, NAN },
    { 0x1p-1022, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1p+0, 0x0p+0 },
    { 0x1p-1022, 0x1p+0, -0x0p+0 },
    { 0x1p-1022, 0x1p+0, 0x1p+0 },
    { 0x1p-1022, 0x1p+0, -0x1p+0 },
    { 0x1p-1022, 0x1p+0, 0x1p-1022 },
    { 0x1p-1022, 0x1p+0, -0x1p-1022 },
    { 0x1p-1022, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1p+0, INF },
    { 0x1p-1022, 0x1p+0, -INF },
    { 0x1p-1022, 0x1p+0, NAN },
    { 0x1p-1022, 0x1p+0, NAN },
    { 0x1p-1022, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1p+0, 0x0p+0 },
    { 0x1p-1022, -0x1p+0, -0x0p+0 },
    { 0x1p-1022, -0x1p+0, 0x1p+0 },
    { 0x1p-1022, -0x1p+0, -0x1p+0 },
    { 0x1p-1022, -0x1p+0, 0x1p-1022 },
    { 0x1p-1022, -0x1p+0, -0x1p-1022 },
    { 0x1p-1022, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1p+0, INF },
    { 0x1p-1022, -0x1p+0, -INF },
    { 0x1p-1022, -0x1p+0, NAN },
    { 0x1p-1022, -0x1p+0, NAN },
    { 0x1p-1022, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1p-1022, 0x0p+0 },
    { 0x1p-1022, 0x1p-1022, -0x0p+0 },
    { 0x1p-1022, 0x1p-1022, 0x1p+0 },
    { 0x1p-1022, 0x1p-1022, -0x1p+0 },
    { 0x1p-1022, 0x1p-1022, 0x1p-1022 },
    { 0x1p-1022, 0x1p-1022, -0x1p-1022 },
    { 0x1p-1022, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1p-1022, INF },
    { 0x1p-1022, 0x1p-1022, -INF },
    { 0x1p-1022, 0x1p-1022, NAN },
    { 0x1p-1022, 0x1p-1022, NAN },
    { 0x1p-1022, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1p-1022, 0x0p+0 },
    { 0x1p-1022, -0x1p-1022, -0x0p+0 },
    { 0x1p-1022, -0x1p-1022, 0x1p+0 },
    { 0x1p-1022, -0x1p-1022, -0x1p+0 },
    { 0x1p-1022, -0x1p-1022, 0x1p-1022 },
    { 0x1p-1022, -0x1p-1022, -0x1p-1022 },
    { 0x1p-1022, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1p-1022, INF },
    { 0x1p-1022, -0x1p-1022, -INF },
    { 0x1p-1022, -0x1p-1022, NAN },
    { 0x1p-1022, -0x1p-1022, NAN },
    { 0x1p-1022, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, INF },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -INF },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, NAN },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, NAN },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, INF },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -INF },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, NAN },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, NAN },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1p-1022, INF, 0x0p+0 },
    { 0x1p-1022, INF, -0x0p+0 },
    { 0x1p-1022, INF, 0x1p+0 },
    { 0x1p-1022, INF, -0x1p+0 },
    { 0x1p-1022, INF, 0x1p-1022 },
    { 0x1p-1022, INF, -0x1p-1022 },
    { 0x1p-1022, INF, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, INF, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, INF, INF },
    { 0x1p-1022, INF, -INF },
    { 0x1p-1022, INF, NAN },
    { 0x1p-1022, INF, NAN },
    { 0x1p-1022, INF, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, INF, 0x1.199999999999ap+0 },
    { 0x1p-1022, INF, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, INF, -0x1.199999999999ap+0 },
    { 0x1p-1022, -INF, 0x0p+0 },
    { 0x1p-1022, -INF, -0x0p+0 },
    { 0x1p-1022, -INF, 0x1p+0 },
    { 0x1p-1022, -INF, -0x1p+0 },
    { 0x1p-1022, -INF, 0x1p-1022 },
    { 0x1p-1022, -INF, -0x1p-1022 },
    { 0x1p-1022, -INF, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -INF, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -INF, INF },
    { 0x1p-1022, -INF, -INF },
    { 0x1p-1022, -INF, NAN },
    { 0x1p-1022, -INF, NAN },
    { 0x1p-1022, -INF, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -INF, 0x1.199999999999ap+0 },
    { 0x1p-1022, -INF, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -INF, -0x1.199999999999ap+0 },
    { 0x1p-1022, NAN, 0x0p+0 },
    { 0x1p-1022, NAN, -0x0p+0 },
    { 0x1p-1022, NAN, 0x1p+0 },
    { 0x1p-1022, NAN, -0x1p+0 },
    { 0x1p-1022, NAN, 0x1p-1022 },
    { 0x1p-1022, NAN, -0x1p-1022 },
    { 0x1p-1022, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, NAN, INF },
    { 0x1p-1022, NAN, -INF },
    { 0x1p-1022, NAN, NAN },
    { 0x1p-1022, NAN, NAN },
    { 0x1p-1022, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, NAN, 0x1.199999999999ap+0 },
    { 0x1p-1022, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, NAN, -0x1.199999999999ap+0 },
    { 0x1p-1022, NAN, 0x0p+0 },
    { 0x1p-1022, NAN, -0x0p+0 },
    { 0x1p-1022, NAN, 0x1p+0 },
    { 0x1p-1022, NAN, -0x1p+0 },
    { 0x1p-1022, NAN, 0x1p-1022 },
    { 0x1p-1022, NAN, -0x1p-1022 },
    { 0x1p-1022, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, NAN, INF },
    { 0x1p-1022, NAN, -INF },
    { 0x1p-1022, NAN, NAN },
    { 0x1p-1022, NAN, NAN },
    { 0x1p-1022, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, NAN, 0x1.199999999999ap+0 },
    { 0x1p-1022, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, NAN, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, INF },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -INF },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, NAN },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, NAN },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, 0x1.199999999999ap+0, INF },
    { 0x1p-1022, 0x1.199999999999ap+0, -INF },
    { 0x1p-1022, 0x1.199999999999ap+0, NAN },
    { 0x1p-1022, 0x1.199999999999ap+0, NAN },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, INF },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -INF },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, NAN },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, NAN },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1p-1022, -0x1.199999999999ap+0, INF },
    { 0x1p-1022, -0x1.199999999999ap+0, -INF },
    { 0x1p-1022, -0x1.199999999999ap+0, NAN },
    { 0x1p-1022, -0x1.199999999999ap+0, NAN },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1p-1022, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x0p+0, 0x0p+0 },
    { -0x1p-1022, 0x0p+0, -0x0p+0 },
    { -0x1p-1022, 0x0p+0, 0x1p+0 },
    { -0x1p-1022, 0x0p+0, -0x1p+0 },
    { -0x1p-1022, 0x0p+0, 0x1p-1022 },
    { -0x1p-1022, 0x0p+0, -0x1p-1022 },
    { -0x1p-1022, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x0p+0, INF },
    { -0x1p-1022, 0x0p+0, -INF },
    { -0x1p-1022, 0x0p+0, NAN },
    { -0x1p-1022, 0x0p+0, NAN },
    { -0x1p-1022, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x0p+0, 0x0p+0 },
    { -0x1p-1022, -0x0p+0, -0x0p+0 },
    { -0x1p-1022, -0x0p+0, 0x1p+0 },
    { -0x1p-1022, -0x0p+0, -0x1p+0 },
    { -0x1p-1022, -0x0p+0, 0x1p-1022 },
    { -0x1p-1022, -0x0p+0, -0x1p-1022 },
    { -0x1p-1022, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x0p+0, INF },
    { -0x1p-1022, -0x0p+0, -INF },
    { -0x1p-1022, -0x0p+0, NAN },
    { -0x1p-1022, -0x0p+0, NAN },
    { -0x1p-1022, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1p+0, 0x0p+0 },
    { -0x1p-1022, 0x1p+0, -0x0p+0 },
    { -0x1p-1022, 0x1p+0, 0x1p+0 },
    { -0x1p-1022, 0x1p+0, -0x1p+0 },
    { -0x1p-1022, 0x1p+0, 0x1p-1022 },
    { -0x1p-1022, 0x1p+0, -0x1p-1022 },
    { -0x1p-1022, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1p+0, INF },
    { -0x1p-1022, 0x1p+0, -INF },
    { -0x1p-1022, 0x1p+0, NAN },
    { -0x1p-1022, 0x1p+0, NAN },
    { -0x1p-1022, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1p+0, 0x0p+0 },
    { -0x1p-1022, -0x1p+0, -0x0p+0 },
    { -0x1p-1022, -0x1p+0, 0x1p+0 },
    { -0x1p-1022, -0x1p+0, -0x1p+0 },
    { -0x1p-1022, -0x1p+0, 0x1p-1022 },
    { -0x1p-1022, -0x1p+0, -0x1p-1022 },
    { -0x1p-1022, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1p+0, INF },
    { -0x1p-1022, -0x1p+0, -INF },
    { -0x1p-1022, -0x1p+0, NAN },
    { -0x1p-1022, -0x1p+0, NAN },
    { -0x1p-1022, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1p-1022, 0x0p+0 },
    { -0x1p-1022, 0x1p-1022, -0x0p+0 },
    { -0x1p-1022, 0x1p-1022, 0x1p+0 },
    { -0x1p-1022, 0x1p-1022, -0x1p+0 },
    { -0x1p-1022, 0x1p-1022, 0x1p-1022 },
    { -0x1p-1022, 0x1p-1022, -0x1p-1022 },
    { -0x1p-1022, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1p-1022, INF },
    { -0x1p-1022, 0x1p-1022, -INF },
    { -0x1p-1022, 0x1p-1022, NAN },
    { -0x1p-1022, 0x1p-1022, NAN },
    { -0x1p-1022, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1p-1022, 0x0p+0 },
    { -0x1p-1022, -0x1p-1022, -0x0p+0 },
    { -0x1p-1022, -0x1p-1022, 0x1p+0 },
    { -0x1p-1022, -0x1p-1022, -0x1p+0 },
    { -0x1p-1022, -0x1p-1022, 0x1p-1022 },
    { -0x1p-1022, -0x1p-1022, -0x1p-1022 },
    { -0x1p-1022, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1p-1022, INF },
    { -0x1p-1022, -0x1p-1022, -INF },
    { -0x1p-1022, -0x1p-1022, NAN },
    { -0x1p-1022, -0x1p-1022, NAN },
    { -0x1p-1022, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, INF },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -INF },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, NAN },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, NAN },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, INF },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -INF },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, NAN },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, NAN },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1p-1022, INF, 0x0p+0 },
    { -0x1p-1022, INF, -0x0p+0 },
    { -0x1p-1022, INF, 0x1p+0 },
    { -0x1p-1022, INF, -0x1p+0 },
    { -0x1p-1022, INF, 0x1p-1022 },
    { -0x1p-1022, INF, -0x1p-1022 },
    { -0x1p-1022, INF, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, INF, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, INF, INF },
    { -0x1p-1022, INF, -INF },
    { -0x1p-1022, INF, NAN },
    { -0x1p-1022, INF, NAN },
    { -0x1p-1022, INF, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, INF, 0x1.199999999999ap+0 },
    { -0x1p-1022, INF, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, INF, -0x1.199999999999ap+0 },
    { -0x1p-1022, -INF, 0x0p+0 },
    { -0x1p-1022, -INF, -0x0p+0 },
    { -0x1p-1022, -INF, 0x1p+0 },
    { -0x1p-1022, -INF, -0x1p+0 },
    { -0x1p-1022, -INF, 0x1p-1022 },
    { -0x1p-1022, -INF, -0x1p-1022 },
    { -0x1p-1022, -INF, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -INF, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -INF, INF },
    { -0x1p-1022, -INF, -INF },
    { -0x1p-1022, -INF, NAN },
    { -0x1p-1022, -INF, NAN },
    { -0x1p-1022, -INF, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -INF, 0x1.199999999999ap+0 },
    { -0x1p-1022, -INF, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -INF, -0x1.199999999999ap+0 },
    { -0x1p-1022, NAN, 0x0p+0 },
    { -0x1p-1022, NAN, -0x0p+0 },
    { -0x1p-1022, NAN, 0x1p+0 },
    { -0x1p-1022, NAN, -0x1p+0 },
    { -0x1p-1022, NAN, 0x1p-1022 },
    { -0x1p-1022, NAN, -0x1p-1022 },
    { -0x1p-1022, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, NAN, INF },
    { -0x1p-1022, NAN, -INF },
    { -0x1p-1022, NAN, NAN },
    { -0x1p-1022, NAN, NAN },
    { -0x1p-1022, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, NAN, 0x1.199999999999ap+0 },
    { -0x1p-1022, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, NAN, -0x1.199999999999ap+0 },
    { -0x1p-1022, NAN, 0x0p+0 },
    { -0x1p-1022, NAN, -0x0p+0 },
    { -0x1p-1022, NAN, 0x1p+0 },
    { -0x1p-1022, NAN, -0x1p+0 },
    { -0x1p-1022, NAN, 0x1p-1022 },
    { -0x1p-1022, NAN, -0x1p-1022 },
    { -0x1p-1022, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, NAN, INF },
    { -0x1p-1022, NAN, -INF },
    { -0x1p-1022, NAN, NAN },
    { -0x1p-1022, NAN, NAN },
    { -0x1p-1022, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, NAN, 0x1.199999999999ap+0 },
    { -0x1p-1022, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, NAN, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, INF },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -INF },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, NAN },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, NAN },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, 0x1.199999999999ap+0, INF },
    { -0x1p-1022, 0x1.199999999999ap+0, -INF },
    { -0x1p-1022, 0x1.199999999999ap+0, NAN },
    { -0x1p-1022, 0x1.199999999999ap+0, NAN },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, INF },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -INF },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, NAN },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, NAN },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1p-1022, -0x1.199999999999ap+0, INF },
    { -0x1p-1022, -0x1.199999999999ap+0, -INF },
    { -0x1p-1022, -0x1.199999999999ap+0, NAN },
    { -0x1p-1022, -0x1.199999999999ap+0, NAN },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1p-1022, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, INF },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -INF },
    { 0x1.fffffffffffffp+1023, 0x0p+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x0p+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, INF },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -INF },
    { 0x1.fffffffffffffp+1023, -0x0p+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x0p+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, INF },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -INF },
    { 0x1.fffffffffffffp+1023, 0x1p+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x1p+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, INF },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -INF },
    { 0x1.fffffffffffffp+1023, -0x1p+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x1p+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, INF },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -INF },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, NAN },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, NAN },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, INF },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -INF },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, NAN },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, NAN },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, INF },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -INF },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, INF },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -INF },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, INF, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, INF, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, INF, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, INF, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, INF, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, INF, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, INF, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, INF, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, INF, INF },
    { 0x1.fffffffffffffp+1023, INF, -INF },
    { 0x1.fffffffffffffp+1023, INF, NAN },
    { 0x1.fffffffffffffp+1023, INF, NAN },
    { 0x1.fffffffffffffp+1023, INF, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, INF, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, INF, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, INF, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -INF, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -INF, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -INF, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -INF, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -INF, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -INF, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -INF, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -INF, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -INF, INF },
    { 0x1.fffffffffffffp+1023, -INF, -INF },
    { 0x1.fffffffffffffp+1023, -INF, NAN },
    { 0x1.fffffffffffffp+1023, -INF, NAN },
    { 0x1.fffffffffffffp+1023, -INF, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -INF, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -INF, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -INF, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, NAN, INF },
    { 0x1.fffffffffffffp+1023, NAN, -INF },
    { 0x1.fffffffffffffp+1023, NAN, NAN },
    { 0x1.fffffffffffffp+1023, NAN, NAN },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, NAN, INF },
    { 0x1.fffffffffffffp+1023, NAN, -INF },
    { 0x1.fffffffffffffp+1023, NAN, NAN },
    { 0x1.fffffffffffffp+1023, NAN, NAN },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, NAN, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, NAN, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, INF },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -INF },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, INF },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -INF },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, NAN },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, INF },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -INF },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, INF },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -INF },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, NAN },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, INF },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -INF },
    { -0x1.fffffffffffffp+1023, 0x0p+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x0p+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, INF },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -INF },
    { -0x1.fffffffffffffp+1023, -0x0p+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x0p+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, INF },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -INF },
    { -0x1.fffffffffffffp+1023, 0x1p+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x1p+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, INF },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -INF },
    { -0x1.fffffffffffffp+1023, -0x1p+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x1p+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, INF },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -INF },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, NAN },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, NAN },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, INF },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -INF },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, NAN },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, NAN },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, INF },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -INF },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, INF },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -INF },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, INF, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, INF, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, INF, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, INF, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, INF, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, INF, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, INF, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, INF, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, INF, INF },
    { -0x1.fffffffffffffp+1023, INF, -INF },
    { -0x1.fffffffffffffp+1023, INF, NAN },
    { -0x1.fffffffffffffp+1023, INF, NAN },
    { -0x1.fffffffffffffp+1023, INF, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, INF, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, INF, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, INF, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -INF, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -INF, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -INF, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -INF, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -INF, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -INF, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -INF, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -INF, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -INF, INF },
    { -0x1.fffffffffffffp+1023, -INF, -INF },
    { -0x1.fffffffffffffp+1023, -INF, NAN },
    { -0x1.fffffffffffffp+1023, -INF, NAN },
    { -0x1.fffffffffffffp+1023, -INF, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -INF, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -INF, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -INF, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, NAN, INF },
    { -0x1.fffffffffffffp+1023, NAN, -INF },
    { -0x1.fffffffffffffp+1023, NAN, NAN },
    { -0x1.fffffffffffffp+1023, NAN, NAN },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, NAN, INF },
    { -0x1.fffffffffffffp+1023, NAN, -INF },
    { -0x1.fffffffffffffp+1023, NAN, NAN },
    { -0x1.fffffffffffffp+1023, NAN, NAN },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, NAN, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, NAN, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, INF },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -INF },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, INF },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -INF },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, NAN },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, INF },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -INF },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, INF },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -INF },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, NAN },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { INF, 0x0p+0, 0x0p+0 },
    { INF, 0x0p+0, -0x0p+0 },
    { INF, 0x0p+0, 0x1p+0 },
    { INF, 0x0p+0, -0x1p+0 },
    { INF, 0x0p+0, 0x1p-1022 },
    { INF, 0x0p+0, -0x1p-1022 },
    { INF, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { INF, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { INF, 0x0p+0, INF },
    { INF, 0x0p+0, -INF },
    { INF, 0x0p+0, NAN },
    { INF, 0x0p+0, NAN },
    { INF, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { INF, 0x0p+0, 0x1.199999999999ap+0 },
    { INF, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { INF, 0x0p+0, -0x1.199999999999ap+0 },
    { INF, -0x0p+0, 0x0p+0 },
    { INF, -0x0p+0, -0x0p+0 },
    { INF, -0x0p+0, 0x1p+0 },
    { INF, -0x0p+0, -0x1p+0 },
    { INF, -0x0p+0, 0x1p-1022 },
    { INF, -0x0p+0, -0x1p-1022 },
    { INF, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { INF, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { INF, -0x0p+0, INF },
    { INF, -0x0p+0, -INF },
    { INF, -0x0p+0, NAN },
    { INF, -0x0p+0, NAN },
    { INF, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { INF, -0x0p+0, 0x1.199999999999ap+0 },
    { INF, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { INF, -0x0p+0, -0x1.199999999999ap+0 },
    { INF, 0x1p+0, 0x0p+0 },
    { INF, 0x1p+0, -0x0p+0 },
    { INF, 0x1p+0, 0x1p+0 },
    { INF, 0x1p+0, -0x1p+0 },
    { INF, 0x1p+0, 0x1p-1022 },
    { INF, 0x1p+0, -0x1p-1022 },
    { INF, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { INF, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { INF, 0x1p+0, INF },
    { INF, 0x1p+0, -INF },
    { INF, 0x1p+0, NAN },
    { INF, 0x1p+0, NAN },
    { INF, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { INF, 0x1p+0, 0x1.199999999999ap+0 },
    { INF, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { INF, 0x1p+0, -0x1.199999999999ap+0 },
    { INF, -0x1p+0, 0x0p+0 },
    { INF, -0x1p+0, -0x0p+0 },
    { INF, -0x1p+0, 0x1p+0 },
    { INF, -0x1p+0, -0x1p+0 },
    { INF, -0x1p+0, 0x1p-1022 },
    { INF, -0x1p+0, -0x1p-1022 },
    { INF, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { INF, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { INF, -0x1p+0, INF },
    { INF, -0x1p+0, -INF },
    { INF, -0x1p+0, NAN },
    { INF, -0x1p+0, NAN },
    { INF, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { INF, -0x1p+0, 0x1.199999999999ap+0 },
    { INF, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { INF, -0x1p+0, -0x1.199999999999ap+0 },
    { INF, 0x1p-1022, 0x0p+0 },
    { INF, 0x1p-1022, -0x0p+0 },
    { INF, 0x1p-1022, 0x1p+0 },
    { INF, 0x1p-1022, -0x1p+0 },
    { INF, 0x1p-1022, 0x1p-1022 },
    { INF, 0x1p-1022, -0x1p-1022 },
    { INF, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { INF, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { INF, 0x1p-1022, INF },
    { INF, 0x1p-1022, -INF },
    { INF, 0x1p-1022, NAN },
    { INF, 0x1p-1022, NAN },
    { INF, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { INF, 0x1p-1022, 0x1.199999999999ap+0 },
    { INF, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { INF, 0x1p-1022, -0x1.199999999999ap+0 },
    { INF, -0x1p-1022, 0x0p+0 },
    { INF, -0x1p-1022, -0x0p+0 },
    { INF, -0x1p-1022, 0x1p+0 },
    { INF, -0x1p-1022, -0x1p+0 },
    { INF, -0x1p-1022, 0x1p-1022 },
    { INF, -0x1p-1022, -0x1p-1022 },
    { INF, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { INF, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { INF, -0x1p-1022, INF },
    { INF, -0x1p-1022, -INF },
    { INF, -0x1p-1022, NAN },
    { INF, -0x1p-1022, NAN },
    { INF, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { INF, -0x1p-1022, 0x1.199999999999ap+0 },
    { INF, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { INF, -0x1p-1022, -0x1.199999999999ap+0 },
    { INF, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { INF, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { INF, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { INF, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { INF, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { INF, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { INF, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { INF, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { INF, 0x1.fffffffffffffp+1023, INF },
    { INF, 0x1.fffffffffffffp+1023, -INF },
    { INF, 0x1.fffffffffffffp+1023, NAN },
    { INF, 0x1.fffffffffffffp+1023, NAN },
    { INF, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { INF, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { INF, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { INF, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { INF, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { INF, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { INF, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { INF, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { INF, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { INF, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { INF, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { INF, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { INF, -0x1.fffffffffffffp+1023, INF },
    { INF, -0x1.fffffffffffffp+1023, -INF },
    { INF, -0x1.fffffffffffffp+1023, NAN },
    { INF, -0x1.fffffffffffffp+1023, NAN },
    { INF, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { INF, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { INF, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { INF, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { INF, INF, 0x0p+0 },
    { INF, INF, -0x0p+0 },
    { INF, INF, 0x1p+0 },
    { INF, INF, -0x1p+0 },
    { INF, INF, 0x1p-1022 },
    { INF, INF, -0x1p-1022 },
    { INF, INF, 0x1.fffffffffffffp+1023 },
    { INF, INF, -0x1.fffffffffffffp+1023 },
    { INF, INF, INF },
    { INF, INF, -INF },
    { INF, INF, NAN },
    { INF, INF, NAN },
    { INF, INF, 0x1.ccccccccccccdp-1 },
    { INF, INF, 0x1.199999999999ap+0 },
    { INF, INF, -0x1.ccccccccccccdp-1 },
    { INF, INF, -0x1.199999999999ap+0 },
    { INF, -INF, 0x0p+0 },
    { INF, -INF, -0x0p+0 },
    { INF, -INF, 0x1p+0 },
    { INF, -INF, -0x1p+0 },
    { INF, -INF, 0x1p-1022 },
    { INF, -INF, -0x1p-1022 },
    { INF, -INF, 0x1.fffffffffffffp+1023 },
    { INF, -INF, -0x1.fffffffffffffp+1023 },
    { INF, -INF, INF },
    { INF, -INF, -INF },
    { INF, -INF, NAN },
    { INF, -INF, NAN },
    { INF, -INF, 0x1.ccccccccccccdp-1 },
    { INF, -INF, 0x1.199999999999ap+0 },
    { INF, -INF, -0x1.ccccccccccccdp-1 },
    { INF, -INF, -0x1.199999999999ap+0 },
    { INF, NAN, 0x0p+0 },
    { INF, NAN, -0x0p+0 },
    { INF, NAN, 0x1p+0 },
    { INF, NAN, -0x1p+0 },
    { INF, NAN, 0x1p-1022 },
    { INF, NAN, -0x1p-1022 },
    { INF, NAN, 0x1.fffffffffffffp+1023 },
    { INF, NAN, -0x1.fffffffffffffp+1023 },
    { INF, NAN, INF },
    { INF, NAN, -INF },
    { INF, NAN, NAN },
    { INF, NAN, NAN },
    { INF, NAN, 0x1.ccccccccccccdp-1 },
    { INF, NAN, 0x1.199999999999ap+0 },
    { INF, NAN, -0x1.ccccccccccccdp-1 },
    { INF, NAN, -0x1.199999999999ap+0 },
    { INF, NAN, 0x0p+0 },
    { INF, NAN, -0x0p+0 },
    { INF, NAN, 0x1p+0 },
    { INF, NAN, -0x1p+0 },
    { INF, NAN, 0x1p-1022 },
    { INF, NAN, -0x1p-1022 },
    { INF, NAN, 0x1.fffffffffffffp+1023 },
    { INF, NAN, -0x1.fffffffffffffp+1023 },
    { INF, NAN, INF },
    { INF, NAN, -INF },
    { INF, NAN, NAN },
    { INF, NAN, NAN },
    { INF, NAN, 0x1.ccccccccccccdp-1 },
    { INF, NAN, 0x1.199999999999ap+0 },
    { INF, NAN, -0x1.ccccccccccccdp-1 },
    { INF, NAN, -0x1.199999999999ap+0 },
    { INF, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { INF, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { INF, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { INF, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { INF, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { INF, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { INF, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { INF, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { INF, 0x1.ccccccccccccdp-1, INF },
    { INF, 0x1.ccccccccccccdp-1, -INF },
    { INF, 0x1.ccccccccccccdp-1, NAN },
    { INF, 0x1.ccccccccccccdp-1, NAN },
    { INF, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { INF, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { INF, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { INF, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { INF, 0x1.199999999999ap+0, 0x0p+0 },
    { INF, 0x1.199999999999ap+0, -0x0p+0 },
    { INF, 0x1.199999999999ap+0, 0x1p+0 },
    { INF, 0x1.199999999999ap+0, -0x1p+0 },
    { INF, 0x1.199999999999ap+0, 0x1p-1022 },
    { INF, 0x1.199999999999ap+0, -0x1p-1022 },
    { INF, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { INF, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { INF, 0x1.199999999999ap+0, INF },
    { INF, 0x1.199999999999ap+0, -INF },
    { INF, 0x1.199999999999ap+0, NAN },
    { INF, 0x1.199999999999ap+0, NAN },
    { INF, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { INF, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { INF, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { INF, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { INF, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { INF, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { INF, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { INF, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { INF, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { INF, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { INF, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { INF, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { INF, -0x1.ccccccccccccdp-1, INF },
    { INF, -0x1.ccccccccccccdp-1, -INF },
    { INF, -0x1.ccccccccccccdp-1, NAN },
    { INF, -0x1.ccccccccccccdp-1, NAN },
    { INF, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { INF, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { INF, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { INF, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { INF, -0x1.199999999999ap+0, 0x0p+0 },
    { INF, -0x1.199999999999ap+0, -0x0p+0 },
    { INF, -0x1.199999999999ap+0, 0x1p+0 },
    { INF, -0x1.199999999999ap+0, -0x1p+0 },
    { INF, -0x1.199999999999ap+0, 0x1p-1022 },
    { INF, -0x1.199999999999ap+0, -0x1p-1022 },
    { INF, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { INF, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { INF, -0x1.199999999999ap+0, INF },
    { INF, -0x1.199999999999ap+0, -INF },
    { INF, -0x1.199999999999ap+0, NAN },
    { INF, -0x1.199999999999ap+0, NAN },
    { INF, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { INF, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { INF, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { INF, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -INF, 0x0p+0, 0x0p+0 },
    { -INF, 0x0p+0, -0x0p+0 },
    { -INF, 0x0p+0, 0x1p+0 },
    { -INF, 0x0p+0, -0x1p+0 },
    { -INF, 0x0p+0, 0x1p-1022 },
    { -INF, 0x0p+0, -0x1p-1022 },
    { -INF, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -INF, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -INF, 0x0p+0, INF },
    { -INF, 0x0p+0, -INF },
    { -INF, 0x0p+0, NAN },
    { -INF, 0x0p+0, NAN },
    { -INF, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -INF, 0x0p+0, 0x1.199999999999ap+0 },
    { -INF, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -INF, 0x0p+0, -0x1.199999999999ap+0 },
    { -INF, -0x0p+0, 0x0p+0 },
    { -INF, -0x0p+0, -0x0p+0 },
    { -INF, -0x0p+0, 0x1p+0 },
    { -INF, -0x0p+0, -0x1p+0 },
    { -INF, -0x0p+0, 0x1p-1022 },
    { -INF, -0x0p+0, -0x1p-1022 },
    { -INF, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -INF, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -INF, -0x0p+0, INF },
    { -INF, -0x0p+0, -INF },
    { -INF, -0x0p+0, NAN },
    { -INF, -0x0p+0, NAN },
    { -INF, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -INF, -0x0p+0, 0x1.199999999999ap+0 },
    { -INF, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -INF, -0x0p+0, -0x1.199999999999ap+0 },
    { -INF, 0x1p+0, 0x0p+0 },
    { -INF, 0x1p+0, -0x0p+0 },
    { -INF, 0x1p+0, 0x1p+0 },
    { -INF, 0x1p+0, -0x1p+0 },
    { -INF, 0x1p+0, 0x1p-1022 },
    { -INF, 0x1p+0, -0x1p-1022 },
    { -INF, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -INF, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -INF, 0x1p+0, INF },
    { -INF, 0x1p+0, -INF },
    { -INF, 0x1p+0, NAN },
    { -INF, 0x1p+0, NAN },
    { -INF, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1p+0, 0x1.199999999999ap+0 },
    { -INF, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -INF, 0x1p+0, -0x1.199999999999ap+0 },
    { -INF, -0x1p+0, 0x0p+0 },
    { -INF, -0x1p+0, -0x0p+0 },
    { -INF, -0x1p+0, 0x1p+0 },
    { -INF, -0x1p+0, -0x1p+0 },
    { -INF, -0x1p+0, 0x1p-1022 },
    { -INF, -0x1p+0, -0x1p-1022 },
    { -INF, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -INF, -0x1p+0, INF },
    { -INF, -0x1p+0, -INF },
    { -INF, -0x1p+0, NAN },
    { -INF, -0x1p+0, NAN },
    { -INF, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -INF, -0x1p+0, 0x1.199999999999ap+0 },
    { -INF, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1p+0, -0x1.199999999999ap+0 },
    { -INF, 0x1p-1022, 0x0p+0 },
    { -INF, 0x1p-1022, -0x0p+0 },
    { -INF, 0x1p-1022, 0x1p+0 },
    { -INF, 0x1p-1022, -0x1p+0 },
    { -INF, 0x1p-1022, 0x1p-1022 },
    { -INF, 0x1p-1022, -0x1p-1022 },
    { -INF, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -INF, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -INF, 0x1p-1022, INF },
    { -INF, 0x1p-1022, -INF },
    { -INF, 0x1p-1022, NAN },
    { -INF, 0x1p-1022, NAN },
    { -INF, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1p-1022, 0x1.199999999999ap+0 },
    { -INF, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -INF, 0x1p-1022, -0x1.199999999999ap+0 },
    { -INF, -0x1p-1022, 0x0p+0 },
    { -INF, -0x1p-1022, -0x0p+0 },
    { -INF, -0x1p-1022, 0x1p+0 },
    { -INF, -0x1p-1022, -0x1p+0 },
    { -INF, -0x1p-1022, 0x1p-1022 },
    { -INF, -0x1p-1022, -0x1p-1022 },
    { -INF, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -INF, -0x1p-1022, INF },
    { -INF, -0x1p-1022, -INF },
    { -INF, -0x1p-1022, NAN },
    { -INF, -0x1p-1022, NAN },
    { -INF, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -INF, -0x1p-1022, 0x1.199999999999ap+0 },
    { -INF, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1p-1022, -0x1.199999999999ap+0 },
    { -INF, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -INF, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -INF, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -INF, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -INF, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -INF, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -INF, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -INF, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -INF, 0x1.fffffffffffffp+1023, INF },
    { -INF, 0x1.fffffffffffffp+1023, -INF },
    { -INF, 0x1.fffffffffffffp+1023, NAN },
    { -INF, 0x1.fffffffffffffp+1023, NAN },
    { -INF, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -INF, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -INF, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -INF, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -INF, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -INF, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -INF, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -INF, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -INF, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -INF, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -INF, -0x1.fffffffffffffp+1023, INF },
    { -INF, -0x1.fffffffffffffp+1023, -INF },
    { -INF, -0x1.fffffffffffffp+1023, NAN },
    { -INF, -0x1.fffffffffffffp+1023, NAN },
    { -INF, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -INF, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -INF, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -INF, INF, 0x0p+0 },
    { -INF, INF, -0x0p+0 },
    { -INF, INF, 0x1p+0 },
    { -INF, INF, -0x1p+0 },
    { -INF, INF, 0x1p-1022 },
    { -INF, INF, -0x1p-1022 },
    { -INF, INF, 0x1.fffffffffffffp+1023 },
    { -INF, INF, -0x1.fffffffffffffp+1023 },
    { -INF, INF, INF },
    { -INF, INF, -INF },
    { -INF, INF, NAN },
    { -INF, INF, NAN },
    { -INF, INF, 0x1.ccccccccccccdp-1 },
    { -INF, INF, 0x1.199999999999ap+0 },
    { -INF, INF, -0x1.ccccccccccccdp-1 },
    { -INF, INF, -0x1.199999999999ap+0 },
    { -INF, -INF, 0x0p+0 },
    { -INF, -INF, -0x0p+0 },
    { -INF, -INF, 0x1p+0 },
    { -INF, -INF, -0x1p+0 },
    { -INF, -INF, 0x1p-1022 },
    { -INF, -INF, -0x1p-1022 },
    { -INF, -INF, 0x1.fffffffffffffp+1023 },
    { -INF, -INF, -0x1.fffffffffffffp+1023 },
    { -INF, -INF, INF },
    { -INF, -INF, -INF },
    { -INF, -INF, NAN },
    { -INF, -INF, NAN },
    { -INF, -INF, 0x1.ccccccccccccdp-1 },
    { -INF, -INF, 0x1.199999999999ap+0 },
    { -INF, -INF, -0x1.ccccccccccccdp-1 },
    { -INF, -INF, -0x1.199999999999ap+0 },
    { -INF, NAN, 0x0p+0 },
    { -INF, NAN, -0x0p+0 },
    { -INF, NAN, 0x1p+0 },
    { -INF, NAN, -0x1p+0 },
    { -INF, NAN, 0x1p-1022 },
    { -INF, NAN, -0x1p-1022 },
    { -INF, NAN, 0x1.fffffffffffffp+1023 },
    { -INF, NAN, -0x1.fffffffffffffp+1023 },
    { -INF, NAN, INF },
    { -INF, NAN, -INF },
    { -INF, NAN, NAN },
    { -INF, NAN, NAN },
    { -INF, NAN, 0x1.ccccccccccccdp-1 },
    { -INF, NAN, 0x1.199999999999ap+0 },
    { -INF, NAN, -0x1.ccccccccccccdp-1 },
    { -INF, NAN, -0x1.199999999999ap+0 },
    { -INF, NAN, 0x0p+0 },
    { -INF, NAN, -0x0p+0 },
    { -INF, NAN, 0x1p+0 },
    { -INF, NAN, -0x1p+0 },
    { -INF, NAN, 0x1p-1022 },
    { -INF, NAN, -0x1p-1022 },
    { -INF, NAN, 0x1.fffffffffffffp+1023 },
    { -INF, NAN, -0x1.fffffffffffffp+1023 },
    { -INF, NAN, INF },
    { -INF, NAN, -INF },
    { -INF, NAN, NAN },
    { -INF, NAN, NAN },
    { -INF, NAN, 0x1.ccccccccccccdp-1 },
    { -INF, NAN, 0x1.199999999999ap+0 },
    { -INF, NAN, -0x1.ccccccccccccdp-1 },
    { -INF, NAN, -0x1.199999999999ap+0 },
    { -INF, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -INF, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -INF, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -INF, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -INF, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -INF, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -INF, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -INF, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -INF, 0x1.ccccccccccccdp-1, INF },
    { -INF, 0x1.ccccccccccccdp-1, -INF },
    { -INF, 0x1.ccccccccccccdp-1, NAN },
    { -INF, 0x1.ccccccccccccdp-1, NAN },
    { -INF, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -INF, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -INF, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -INF, 0x1.199999999999ap+0, 0x0p+0 },
    { -INF, 0x1.199999999999ap+0, -0x0p+0 },
    { -INF, 0x1.199999999999ap+0, 0x1p+0 },
    { -INF, 0x1.199999999999ap+0, -0x1p+0 },
    { -INF, 0x1.199999999999ap+0, 0x1p-1022 },
    { -INF, 0x1.199999999999ap+0, -0x1p-1022 },
    { -INF, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -INF, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -INF, 0x1.199999999999ap+0, INF },
    { -INF, 0x1.199999999999ap+0, -INF },
    { -INF, 0x1.199999999999ap+0, NAN },
    { -INF, 0x1.199999999999ap+0, NAN },
    { -INF, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -INF, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -INF, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -INF, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -INF, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -INF, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -INF, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -INF, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -INF, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -INF, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -INF, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -INF, -0x1.ccccccccccccdp-1, INF },
    { -INF, -0x1.ccccccccccccdp-1, -INF },
    { -INF, -0x1.ccccccccccccdp-1, NAN },
    { -INF, -0x1.ccccccccccccdp-1, NAN },
    { -INF, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -INF, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -INF, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -INF, -0x1.199999999999ap+0, 0x0p+0 },
    { -INF, -0x1.199999999999ap+0, -0x0p+0 },
    { -INF, -0x1.199999999999ap+0, 0x1p+0 },
    { -INF, -0x1.199999999999ap+0, -0x1p+0 },
    { -INF, -0x1.199999999999ap+0, 0x1p-1022 },
    { -INF, -0x1.199999999999ap+0, -0x1p-1022 },
    { -INF, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -INF, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -INF, -0x1.199999999999ap+0, INF },
    { -INF, -0x1.199999999999ap+0, -INF },
    { -INF, -0x1.199999999999ap+0, NAN },
    { -INF, -0x1.199999999999ap+0, NAN },
    { -INF, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -INF, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -INF, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -INF, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { NAN, 0x0p+0, 0x0p+0 },
    { NAN, 0x0p+0, -0x0p+0 },
    { NAN, 0x0p+0, 0x1p+0 },
    { NAN, 0x0p+0, -0x1p+0 },
    { NAN, 0x0p+0, 0x1p-1022 },
    { NAN, 0x0p+0, -0x1p-1022 },
    { NAN, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x0p+0, INF },
    { NAN, 0x0p+0, -INF },
    { NAN, 0x0p+0, NAN },
    { NAN, 0x0p+0, NAN },
    { NAN, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x0p+0, 0x1.199999999999ap+0 },
    { NAN, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x0p+0, -0x1.199999999999ap+0 },
    { NAN, -0x0p+0, 0x0p+0 },
    { NAN, -0x0p+0, -0x0p+0 },
    { NAN, -0x0p+0, 0x1p+0 },
    { NAN, -0x0p+0, -0x1p+0 },
    { NAN, -0x0p+0, 0x1p-1022 },
    { NAN, -0x0p+0, -0x1p-1022 },
    { NAN, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x0p+0, INF },
    { NAN, -0x0p+0, -INF },
    { NAN, -0x0p+0, NAN },
    { NAN, -0x0p+0, NAN },
    { NAN, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x0p+0, 0x1.199999999999ap+0 },
    { NAN, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x0p+0, -0x1.199999999999ap+0 },
    { NAN, 0x1p+0, 0x0p+0 },
    { NAN, 0x1p+0, -0x0p+0 },
    { NAN, 0x1p+0, 0x1p+0 },
    { NAN, 0x1p+0, -0x1p+0 },
    { NAN, 0x1p+0, 0x1p-1022 },
    { NAN, 0x1p+0, -0x1p-1022 },
    { NAN, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1p+0, INF },
    { NAN, 0x1p+0, -INF },
    { NAN, 0x1p+0, NAN },
    { NAN, 0x1p+0, NAN },
    { NAN, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1p+0, 0x1.199999999999ap+0 },
    { NAN, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1p+0, -0x1.199999999999ap+0 },
    { NAN, -0x1p+0, 0x0p+0 },
    { NAN, -0x1p+0, -0x0p+0 },
    { NAN, -0x1p+0, 0x1p+0 },
    { NAN, -0x1p+0, -0x1p+0 },
    { NAN, -0x1p+0, 0x1p-1022 },
    { NAN, -0x1p+0, -0x1p-1022 },
    { NAN, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1p+0, INF },
    { NAN, -0x1p+0, -INF },
    { NAN, -0x1p+0, NAN },
    { NAN, -0x1p+0, NAN },
    { NAN, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1p+0, 0x1.199999999999ap+0 },
    { NAN, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1p+0, -0x1.199999999999ap+0 },
    { NAN, 0x1p-1022, 0x0p+0 },
    { NAN, 0x1p-1022, -0x0p+0 },
    { NAN, 0x1p-1022, 0x1p+0 },
    { NAN, 0x1p-1022, -0x1p+0 },
    { NAN, 0x1p-1022, 0x1p-1022 },
    { NAN, 0x1p-1022, -0x1p-1022 },
    { NAN, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1p-1022, INF },
    { NAN, 0x1p-1022, -INF },
    { NAN, 0x1p-1022, NAN },
    { NAN, 0x1p-1022, NAN },
    { NAN, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1p-1022, 0x1.199999999999ap+0 },
    { NAN, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1p-1022, -0x1.199999999999ap+0 },
    { NAN, -0x1p-1022, 0x0p+0 },
    { NAN, -0x1p-1022, -0x0p+0 },
    { NAN, -0x1p-1022, 0x1p+0 },
    { NAN, -0x1p-1022, -0x1p+0 },
    { NAN, -0x1p-1022, 0x1p-1022 },
    { NAN, -0x1p-1022, -0x1p-1022 },
    { NAN, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1p-1022, INF },
    { NAN, -0x1p-1022, -INF },
    { NAN, -0x1p-1022, NAN },
    { NAN, -0x1p-1022, NAN },
    { NAN, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1p-1022, 0x1.199999999999ap+0 },
    { NAN, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1p-1022, -0x1.199999999999ap+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.fffffffffffffp+1023, INF },
    { NAN, 0x1.fffffffffffffp+1023, -INF },
    { NAN, 0x1.fffffffffffffp+1023, NAN },
    { NAN, 0x1.fffffffffffffp+1023, NAN },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023, INF },
    { NAN, -0x1.fffffffffffffp+1023, -INF },
    { NAN, -0x1.fffffffffffffp+1023, NAN },
    { NAN, -0x1.fffffffffffffp+1023, NAN },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { NAN, INF, 0x0p+0 },
    { NAN, INF, -0x0p+0 },
    { NAN, INF, 0x1p+0 },
    { NAN, INF, -0x1p+0 },
    { NAN, INF, 0x1p-1022 },
    { NAN, INF, -0x1p-1022 },
    { NAN, INF, 0x1.fffffffffffffp+1023 },
    { NAN, INF, -0x1.fffffffffffffp+1023 },
    { NAN, INF, INF },
    { NAN, INF, -INF },
    { NAN, INF, NAN },
    { NAN, INF, NAN },
    { NAN, INF, 0x1.ccccccccccccdp-1 },
    { NAN, INF, 0x1.199999999999ap+0 },
    { NAN, INF, -0x1.ccccccccccccdp-1 },
    { NAN, INF, -0x1.199999999999ap+0 },
    { NAN, -INF, 0x0p+0 },
    { NAN, -INF, -0x0p+0 },
    { NAN, -INF, 0x1p+0 },
    { NAN, -INF, -0x1p+0 },
    { NAN, -INF, 0x1p-1022 },
    { NAN, -INF, -0x1p-1022 },
    { NAN, -INF, 0x1.fffffffffffffp+1023 },
    { NAN, -INF, -0x1.fffffffffffffp+1023 },
    { NAN, -INF, INF },
    { NAN, -INF, -INF },
    { NAN, -INF, NAN },
    { NAN, -INF, NAN },
    { NAN, -INF, 0x1.ccccccccccccdp-1 },
    { NAN, -INF, 0x1.199999999999ap+0 },
    { NAN, -INF, -0x1.ccccccccccccdp-1 },
    { NAN, -INF, -0x1.199999999999ap+0 },
    { NAN, NAN, 0x0p+0 },
    { NAN, NAN, -0x0p+0 },
    { NAN, NAN, 0x1p+0 },
    { NAN, NAN, -0x1p+0 },
    { NAN, NAN, 0x1p-1022 },
    { NAN, NAN, -0x1p-1022 },
    { NAN, NAN, 0x1.fffffffffffffp+1023 },
    { NAN, NAN, -0x1.fffffffffffffp+1023 },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccccccccdp-1 },
    { NAN, NAN, 0x1.199999999999ap+0 },
    { NAN, NAN, -0x1.ccccccccccccdp-1 },
    { NAN, NAN, -0x1.199999999999ap+0 },
    { NAN, NAN, 0x0p+0 },
    { NAN, NAN, -0x0p+0 },
    { NAN, NAN, 0x1p+0 },
    { NAN, NAN, -0x1p+0 },
    { NAN, NAN, 0x1p-1022 },
    { NAN, NAN, -0x1p-1022 },
    { NAN, NAN, 0x1.fffffffffffffp+1023 },
    { NAN, NAN, -0x1.fffffffffffffp+1023 },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccccccccdp-1 },
    { NAN, NAN, 0x1.199999999999ap+0 },
    { NAN, NAN, -0x1.ccccccccccccdp-1 },
    { NAN, NAN, -0x1.199999999999ap+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.ccccccccccccdp-1, INF },
    { NAN, 0x1.ccccccccccccdp-1, -INF },
    { NAN, 0x1.ccccccccccccdp-1, NAN },
    { NAN, 0x1.ccccccccccccdp-1, NAN },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { NAN, 0x1.199999999999ap+0, 0x0p+0 },
    { NAN, 0x1.199999999999ap+0, -0x0p+0 },
    { NAN, 0x1.199999999999ap+0, 0x1p+0 },
    { NAN, 0x1.199999999999ap+0, -0x1p+0 },
    { NAN, 0x1.199999999999ap+0, 0x1p-1022 },
    { NAN, 0x1.199999999999ap+0, -0x1p-1022 },
    { NAN, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.199999999999ap+0, INF },
    { NAN, 0x1.199999999999ap+0, -INF },
    { NAN, 0x1.199999999999ap+0, NAN },
    { NAN, 0x1.199999999999ap+0, NAN },
    { NAN, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { NAN, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.ccccccccccccdp-1, INF },
    { NAN, -0x1.ccccccccccccdp-1, -INF },
    { NAN, -0x1.ccccccccccccdp-1, NAN },
    { NAN, -0x1.ccccccccccccdp-1, NAN },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { NAN, -0x1.199999999999ap+0, 0x0p+0 },
    { NAN, -0x1.199999999999ap+0, -0x0p+0 },
    { NAN, -0x1.199999999999ap+0, 0x1p+0 },
    { NAN, -0x1.199999999999ap+0, -0x1p+0 },
    { NAN, -0x1.199999999999ap+0, 0x1p-1022 },
    { NAN, -0x1.199999999999ap+0, -0x1p-1022 },
    { NAN, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.199999999999ap+0, INF },
    { NAN, -0x1.199999999999ap+0, -INF },
    { NAN, -0x1.199999999999ap+0, NAN },
    { NAN, -0x1.199999999999ap+0, NAN },
    { NAN, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { NAN, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { NAN, 0x0p+0, 0x0p+0 },
    { NAN, 0x0p+0, -0x0p+0 },
    { NAN, 0x0p+0, 0x1p+0 },
    { NAN, 0x0p+0, -0x1p+0 },
    { NAN, 0x0p+0, 0x1p-1022 },
    { NAN, 0x0p+0, -0x1p-1022 },
    { NAN, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x0p+0, INF },
    { NAN, 0x0p+0, -INF },
    { NAN, 0x0p+0, NAN },
    { NAN, 0x0p+0, NAN },
    { NAN, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x0p+0, 0x1.199999999999ap+0 },
    { NAN, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x0p+0, -0x1.199999999999ap+0 },
    { NAN, -0x0p+0, 0x0p+0 },
    { NAN, -0x0p+0, -0x0p+0 },
    { NAN, -0x0p+0, 0x1p+0 },
    { NAN, -0x0p+0, -0x1p+0 },
    { NAN, -0x0p+0, 0x1p-1022 },
    { NAN, -0x0p+0, -0x1p-1022 },
    { NAN, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x0p+0, INF },
    { NAN, -0x0p+0, -INF },
    { NAN, -0x0p+0, NAN },
    { NAN, -0x0p+0, NAN },
    { NAN, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x0p+0, 0x1.199999999999ap+0 },
    { NAN, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x0p+0, -0x1.199999999999ap+0 },
    { NAN, 0x1p+0, 0x0p+0 },
    { NAN, 0x1p+0, -0x0p+0 },
    { NAN, 0x1p+0, 0x1p+0 },
    { NAN, 0x1p+0, -0x1p+0 },
    { NAN, 0x1p+0, 0x1p-1022 },
    { NAN, 0x1p+0, -0x1p-1022 },
    { NAN, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1p+0, INF },
    { NAN, 0x1p+0, -INF },
    { NAN, 0x1p+0, NAN },
    { NAN, 0x1p+0, NAN },
    { NAN, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1p+0, 0x1.199999999999ap+0 },
    { NAN, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1p+0, -0x1.199999999999ap+0 },
    { NAN, -0x1p+0, 0x0p+0 },
    { NAN, -0x1p+0, -0x0p+0 },
    { NAN, -0x1p+0, 0x1p+0 },
    { NAN, -0x1p+0, -0x1p+0 },
    { NAN, -0x1p+0, 0x1p-1022 },
    { NAN, -0x1p+0, -0x1p-1022 },
    { NAN, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1p+0, INF },
    { NAN, -0x1p+0, -INF },
    { NAN, -0x1p+0, NAN },
    { NAN, -0x1p+0, NAN },
    { NAN, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1p+0, 0x1.199999999999ap+0 },
    { NAN, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1p+0, -0x1.199999999999ap+0 },
    { NAN, 0x1p-1022, 0x0p+0 },
    { NAN, 0x1p-1022, -0x0p+0 },
    { NAN, 0x1p-1022, 0x1p+0 },
    { NAN, 0x1p-1022, -0x1p+0 },
    { NAN, 0x1p-1022, 0x1p-1022 },
    { NAN, 0x1p-1022, -0x1p-1022 },
    { NAN, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1p-1022, INF },
    { NAN, 0x1p-1022, -INF },
    { NAN, 0x1p-1022, NAN },
    { NAN, 0x1p-1022, NAN },
    { NAN, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1p-1022, 0x1.199999999999ap+0 },
    { NAN, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1p-1022, -0x1.199999999999ap+0 },
    { NAN, -0x1p-1022, 0x0p+0 },
    { NAN, -0x1p-1022, -0x0p+0 },
    { NAN, -0x1p-1022, 0x1p+0 },
    { NAN, -0x1p-1022, -0x1p+0 },
    { NAN, -0x1p-1022, 0x1p-1022 },
    { NAN, -0x1p-1022, -0x1p-1022 },
    { NAN, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1p-1022, INF },
    { NAN, -0x1p-1022, -INF },
    { NAN, -0x1p-1022, NAN },
    { NAN, -0x1p-1022, NAN },
    { NAN, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1p-1022, 0x1.199999999999ap+0 },
    { NAN, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1p-1022, -0x1.199999999999ap+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.fffffffffffffp+1023, INF },
    { NAN, 0x1.fffffffffffffp+1023, -INF },
    { NAN, 0x1.fffffffffffffp+1023, NAN },
    { NAN, 0x1.fffffffffffffp+1023, NAN },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.fffffffffffffp+1023, INF },
    { NAN, -0x1.fffffffffffffp+1023, -INF },
    { NAN, -0x1.fffffffffffffp+1023, NAN },
    { NAN, -0x1.fffffffffffffp+1023, NAN },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { NAN, INF, 0x0p+0 },
    { NAN, INF, -0x0p+0 },
    { NAN, INF, 0x1p+0 },
    { NAN, INF, -0x1p+0 },
    { NAN, INF, 0x1p-1022 },
    { NAN, INF, -0x1p-1022 },
    { NAN, INF, 0x1.fffffffffffffp+1023 },
    { NAN, INF, -0x1.fffffffffffffp+1023 },
    { NAN, INF, INF },
    { NAN, INF, -INF },
    { NAN, INF, NAN },
    { NAN, INF, NAN },
    { NAN, INF, 0x1.ccccccccccccdp-1 },
    { NAN, INF, 0x1.199999999999ap+0 },
    { NAN, INF, -0x1.ccccccccccccdp-1 },
    { NAN, INF, -0x1.199999999999ap+0 },
    { NAN, -INF, 0x0p+0 },
    { NAN, -INF, -0x0p+0 },
    { NAN, -INF, 0x1p+0 },
    { NAN, -INF, -0x1p+0 },
    { NAN, -INF, 0x1p-1022 },
    { NAN, -INF, -0x1p-1022 },
    { NAN, -INF, 0x1.fffffffffffffp+1023 },
    { NAN, -INF, -0x1.fffffffffffffp+1023 },
    { NAN, -INF, INF },
    { NAN, -INF, -INF },
    { NAN, -INF, NAN },
    { NAN, -INF, NAN },
    { NAN, -INF, 0x1.ccccccccccccdp-1 },
    { NAN, -INF, 0x1.199999999999ap+0 },
    { NAN, -INF, -0x1.ccccccccccccdp-1 },
    { NAN, -INF, -0x1.199999999999ap+0 },
    { NAN, NAN, 0x0p+0 },
    { NAN, NAN, -0x0p+0 },
    { NAN, NAN, 0x1p+0 },
    { NAN, NAN, -0x1p+0 },
    { NAN, NAN, 0x1p-1022 },
    { NAN, NAN, -0x1p-1022 },
    { NAN, NAN, 0x1.fffffffffffffp+1023 },
    { NAN, NAN, -0x1.fffffffffffffp+1023 },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccccccccdp-1 },
    { NAN, NAN, 0x1.199999999999ap+0 },
    { NAN, NAN, -0x1.ccccccccccccdp-1 },
    { NAN, NAN, -0x1.199999999999ap+0 },
    { NAN, NAN, 0x0p+0 },
    { NAN, NAN, -0x0p+0 },
    { NAN, NAN, 0x1p+0 },
    { NAN, NAN, -0x1p+0 },
    { NAN, NAN, 0x1p-1022 },
    { NAN, NAN, -0x1p-1022 },
    { NAN, NAN, 0x1.fffffffffffffp+1023 },
    { NAN, NAN, -0x1.fffffffffffffp+1023 },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccccccccdp-1 },
    { NAN, NAN, 0x1.199999999999ap+0 },
    { NAN, NAN, -0x1.ccccccccccccdp-1 },
    { NAN, NAN, -0x1.199999999999ap+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.ccccccccccccdp-1, INF },
    { NAN, 0x1.ccccccccccccdp-1, -INF },
    { NAN, 0x1.ccccccccccccdp-1, NAN },
    { NAN, 0x1.ccccccccccccdp-1, NAN },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { NAN, 0x1.199999999999ap+0, 0x0p+0 },
    { NAN, 0x1.199999999999ap+0, -0x0p+0 },
    { NAN, 0x1.199999999999ap+0, 0x1p+0 },
    { NAN, 0x1.199999999999ap+0, -0x1p+0 },
    { NAN, 0x1.199999999999ap+0, 0x1p-1022 },
    { NAN, 0x1.199999999999ap+0, -0x1p-1022 },
    { NAN, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { NAN, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { NAN, 0x1.199999999999ap+0, INF },
    { NAN, 0x1.199999999999ap+0, -INF },
    { NAN, 0x1.199999999999ap+0, NAN },
    { NAN, 0x1.199999999999ap+0, NAN },
    { NAN, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { NAN, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { NAN, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.ccccccccccccdp-1, INF },
    { NAN, -0x1.ccccccccccccdp-1, -INF },
    { NAN, -0x1.ccccccccccccdp-1, NAN },
    { NAN, -0x1.ccccccccccccdp-1, NAN },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { NAN, -0x1.199999999999ap+0, 0x0p+0 },
    { NAN, -0x1.199999999999ap+0, -0x0p+0 },
    { NAN, -0x1.199999999999ap+0, 0x1p+0 },
    { NAN, -0x1.199999999999ap+0, -0x1p+0 },
    { NAN, -0x1.199999999999ap+0, 0x1p-1022 },
    { NAN, -0x1.199999999999ap+0, -0x1p-1022 },
    { NAN, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { NAN, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { NAN, -0x1.199999999999ap+0, INF },
    { NAN, -0x1.199999999999ap+0, -INF },
    { NAN, -0x1.199999999999ap+0, NAN },
    { NAN, -0x1.199999999999ap+0, NAN },
    { NAN, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { NAN, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { NAN, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, INF },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -INF },
    { 0x1.ccccccccccccdp-1, 0x0p+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x0p+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, INF },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -INF },
    { 0x1.ccccccccccccdp-1, -0x0p+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x0p+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, INF },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -INF },
    { 0x1.ccccccccccccdp-1, 0x1p+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x1p+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, INF },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -INF },
    { 0x1.ccccccccccccdp-1, -0x1p+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x1p+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, INF },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -INF },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, NAN },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, NAN },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, INF },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -INF },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, NAN },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, NAN },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, INF },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -INF },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, INF },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -INF },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, INF, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, INF, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, INF, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, INF, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, INF, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, INF, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, INF, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, INF, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, INF, INF },
    { 0x1.ccccccccccccdp-1, INF, -INF },
    { 0x1.ccccccccccccdp-1, INF, NAN },
    { 0x1.ccccccccccccdp-1, INF, NAN },
    { 0x1.ccccccccccccdp-1, INF, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, INF, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, INF, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, INF, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -INF, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -INF, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -INF, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -INF, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -INF, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -INF, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -INF, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -INF, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -INF, INF },
    { 0x1.ccccccccccccdp-1, -INF, -INF },
    { 0x1.ccccccccccccdp-1, -INF, NAN },
    { 0x1.ccccccccccccdp-1, -INF, NAN },
    { 0x1.ccccccccccccdp-1, -INF, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -INF, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -INF, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -INF, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, NAN, INF },
    { 0x1.ccccccccccccdp-1, NAN, -INF },
    { 0x1.ccccccccccccdp-1, NAN, NAN },
    { 0x1.ccccccccccccdp-1, NAN, NAN },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, NAN, INF },
    { 0x1.ccccccccccccdp-1, NAN, -INF },
    { 0x1.ccccccccccccdp-1, NAN, NAN },
    { 0x1.ccccccccccccdp-1, NAN, NAN },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, NAN, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, NAN, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, INF },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -INF },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, INF },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -INF },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, NAN },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, INF },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -INF },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, INF },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -INF },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, NAN },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x0p+0, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x0p+0, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x0p+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x0p+0, INF },
    { 0x1.199999999999ap+0, 0x0p+0, -INF },
    { 0x1.199999999999ap+0, 0x0p+0, NAN },
    { 0x1.199999999999ap+0, 0x0p+0, NAN },
    { 0x1.199999999999ap+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x0p+0, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x0p+0, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x0p+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x0p+0, INF },
    { 0x1.199999999999ap+0, -0x0p+0, -INF },
    { 0x1.199999999999ap+0, -0x0p+0, NAN },
    { 0x1.199999999999ap+0, -0x0p+0, NAN },
    { 0x1.199999999999ap+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x0p+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x0p+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1p+0, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1p+0, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1p+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1p+0, INF },
    { 0x1.199999999999ap+0, 0x1p+0, -INF },
    { 0x1.199999999999ap+0, 0x1p+0, NAN },
    { 0x1.199999999999ap+0, 0x1p+0, NAN },
    { 0x1.199999999999ap+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1p+0, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x1p+0, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x1p+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1p+0, INF },
    { 0x1.199999999999ap+0, -0x1p+0, -INF },
    { 0x1.199999999999ap+0, -0x1p+0, NAN },
    { 0x1.199999999999ap+0, -0x1p+0, NAN },
    { 0x1.199999999999ap+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1p+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1p+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1p-1022, INF },
    { 0x1.199999999999ap+0, 0x1p-1022, -INF },
    { 0x1.199999999999ap+0, 0x1p-1022, NAN },
    { 0x1.199999999999ap+0, 0x1p-1022, NAN },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1p-1022, INF },
    { 0x1.199999999999ap+0, -0x1p-1022, -INF },
    { 0x1.199999999999ap+0, -0x1p-1022, NAN },
    { 0x1.199999999999ap+0, -0x1p-1022, NAN },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, INF },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -INF },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, NAN },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, INF },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -INF },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, NAN },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, INF, 0x0p+0 },
    { 0x1.199999999999ap+0, INF, -0x0p+0 },
    { 0x1.199999999999ap+0, INF, 0x1p+0 },
    { 0x1.199999999999ap+0, INF, -0x1p+0 },
    { 0x1.199999999999ap+0, INF, 0x1p-1022 },
    { 0x1.199999999999ap+0, INF, -0x1p-1022 },
    { 0x1.199999999999ap+0, INF, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, INF, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, INF, INF },
    { 0x1.199999999999ap+0, INF, -INF },
    { 0x1.199999999999ap+0, INF, NAN },
    { 0x1.199999999999ap+0, INF, NAN },
    { 0x1.199999999999ap+0, INF, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, INF, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, INF, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, INF, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -INF, 0x0p+0 },
    { 0x1.199999999999ap+0, -INF, -0x0p+0 },
    { 0x1.199999999999ap+0, -INF, 0x1p+0 },
    { 0x1.199999999999ap+0, -INF, -0x1p+0 },
    { 0x1.199999999999ap+0, -INF, 0x1p-1022 },
    { 0x1.199999999999ap+0, -INF, -0x1p-1022 },
    { 0x1.199999999999ap+0, -INF, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -INF, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -INF, INF },
    { 0x1.199999999999ap+0, -INF, -INF },
    { 0x1.199999999999ap+0, -INF, NAN },
    { 0x1.199999999999ap+0, -INF, NAN },
    { 0x1.199999999999ap+0, -INF, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -INF, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -INF, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -INF, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, NAN, 0x0p+0 },
    { 0x1.199999999999ap+0, NAN, -0x0p+0 },
    { 0x1.199999999999ap+0, NAN, 0x1p+0 },
    { 0x1.199999999999ap+0, NAN, -0x1p+0 },
    { 0x1.199999999999ap+0, NAN, 0x1p-1022 },
    { 0x1.199999999999ap+0, NAN, -0x1p-1022 },
    { 0x1.199999999999ap+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, NAN, INF },
    { 0x1.199999999999ap+0, NAN, -INF },
    { 0x1.199999999999ap+0, NAN, NAN },
    { 0x1.199999999999ap+0, NAN, NAN },
    { 0x1.199999999999ap+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, NAN, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, NAN, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, NAN, 0x0p+0 },
    { 0x1.199999999999ap+0, NAN, -0x0p+0 },
    { 0x1.199999999999ap+0, NAN, 0x1p+0 },
    { 0x1.199999999999ap+0, NAN, -0x1p+0 },
    { 0x1.199999999999ap+0, NAN, 0x1p-1022 },
    { 0x1.199999999999ap+0, NAN, -0x1p-1022 },
    { 0x1.199999999999ap+0, NAN, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, NAN, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, NAN, INF },
    { 0x1.199999999999ap+0, NAN, -INF },
    { 0x1.199999999999ap+0, NAN, NAN },
    { 0x1.199999999999ap+0, NAN, NAN },
    { 0x1.199999999999ap+0, NAN, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, NAN, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, NAN, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, NAN, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, INF },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -INF },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, NAN },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, INF },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -INF },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, INF },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -INF },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, NAN },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x0p+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1p+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, INF },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -INF },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, NAN },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { 0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, INF },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -INF },
    { -0x1.ccccccccccccdp-1, 0x0p+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x0p+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, INF },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -INF },
    { -0x1.ccccccccccccdp-1, -0x0p+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x0p+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, INF },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -INF },
    { -0x1.ccccccccccccdp-1, 0x1p+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x1p+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, INF },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -INF },
    { -0x1.ccccccccccccdp-1, -0x1p+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x1p+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, INF },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -INF },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, NAN },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, NAN },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, INF },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -INF },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, NAN },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, NAN },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, INF },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -INF },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, INF },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -INF },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, INF, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, INF, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, INF, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, INF, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, INF, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, INF, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, INF, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, INF, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, INF, INF },
    { -0x1.ccccccccccccdp-1, INF, -INF },
    { -0x1.ccccccccccccdp-1, INF, NAN },
    { -0x1.ccccccccccccdp-1, INF, NAN },
    { -0x1.ccccccccccccdp-1, INF, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, INF, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, INF, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, INF, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -INF, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -INF, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -INF, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -INF, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -INF, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -INF, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -INF, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -INF, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -INF, INF },
    { -0x1.ccccccccccccdp-1, -INF, -INF },
    { -0x1.ccccccccccccdp-1, -INF, NAN },
    { -0x1.ccccccccccccdp-1, -INF, NAN },
    { -0x1.ccccccccccccdp-1, -INF, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -INF, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -INF, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -INF, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, NAN, INF },
    { -0x1.ccccccccccccdp-1, NAN, -INF },
    { -0x1.ccccccccccccdp-1, NAN, NAN },
    { -0x1.ccccccccccccdp-1, NAN, NAN },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, NAN, INF },
    { -0x1.ccccccccccccdp-1, NAN, -INF },
    { -0x1.ccccccccccccdp-1, NAN, NAN },
    { -0x1.ccccccccccccdp-1, NAN, NAN },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, NAN, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, NAN, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, INF },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -INF },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, INF },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -INF },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, NAN },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, INF },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -INF },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, INF },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -INF },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, NAN },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x0p+0, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x0p+0, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x0p+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x0p+0, INF },
    { -0x1.199999999999ap+0, 0x0p+0, -INF },
    { -0x1.199999999999ap+0, 0x0p+0, NAN },
    { -0x1.199999999999ap+0, 0x0p+0, NAN },
    { -0x1.199999999999ap+0, 0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x0p+0, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x0p+0, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x0p+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x0p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x0p+0, INF },
    { -0x1.199999999999ap+0, -0x0p+0, -INF },
    { -0x1.199999999999ap+0, -0x0p+0, NAN },
    { -0x1.199999999999ap+0, -0x0p+0, NAN },
    { -0x1.199999999999ap+0, -0x0p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x0p+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x0p+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1p+0, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1p+0, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1p+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1p+0, INF },
    { -0x1.199999999999ap+0, 0x1p+0, -INF },
    { -0x1.199999999999ap+0, 0x1p+0, NAN },
    { -0x1.199999999999ap+0, 0x1p+0, NAN },
    { -0x1.199999999999ap+0, 0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1p+0, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x1p+0, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x1p+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1p+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1p+0, INF },
    { -0x1.199999999999ap+0, -0x1p+0, -INF },
    { -0x1.199999999999ap+0, -0x1p+0, NAN },
    { -0x1.199999999999ap+0, -0x1p+0, NAN },
    { -0x1.199999999999ap+0, -0x1p+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1p+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1p+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1p-1022, INF },
    { -0x1.199999999999ap+0, 0x1p-1022, -INF },
    { -0x1.199999999999ap+0, 0x1p-1022, NAN },
    { -0x1.199999999999ap+0, 0x1p-1022, NAN },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1p-1022, INF },
    { -0x1.199999999999ap+0, -0x1p-1022, -INF },
    { -0x1.199999999999ap+0, -0x1p-1022, NAN },
    { -0x1.199999999999ap+0, -0x1p-1022, NAN },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1p-1022, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1p-1022, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, INF },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -INF },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, NAN },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, INF },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -INF },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, NAN },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, INF, 0x0p+0 },
    { -0x1.199999999999ap+0, INF, -0x0p+0 },
    { -0x1.199999999999ap+0, INF, 0x1p+0 },
    { -0x1.199999999999ap+0, INF, -0x1p+0 },
    { -0x1.199999999999ap+0, INF, 0x1p-1022 },
    { -0x1.199999999999ap+0, INF, -0x1p-1022 },
    { -0x1.199999999999ap+0, INF, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, INF, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, INF, INF },
    { -0x1.199999999999ap+0, INF, -INF },
    { -0x1.199999999999ap+0, INF, NAN },
    { -0x1.199999999999ap+0, INF, NAN },
    { -0x1.199999999999ap+0, INF, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, INF, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, INF, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, INF, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -INF, 0x0p+0 },
    { -0x1.199999999999ap+0, -INF, -0x0p+0 },
    { -0x1.199999999999ap+0, -INF, 0x1p+0 },
    { -0x1.199999999999ap+0, -INF, -0x1p+0 },
    { -0x1.199999999999ap+0, -INF, 0x1p-1022 },
    { -0x1.199999999999ap+0, -INF, -0x1p-1022 },
    { -0x1.199999999999ap+0, -INF, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -INF, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -INF, INF },
    { -0x1.199999999999ap+0, -INF, -INF },
    { -0x1.199999999999ap+0, -INF, NAN },
    { -0x1.199999999999ap+0, -INF, NAN },
    { -0x1.199999999999ap+0, -INF, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -INF, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -INF, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -INF, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, NAN, 0x0p+0 },
    { -0x1.199999999999ap+0, NAN, -0x0p+0 },
    { -0x1.199999999999ap+0, NAN, 0x1p+0 },
    { -0x1.199999999999ap+0, NAN, -0x1p+0 },
    { -0x1.199999999999ap+0, NAN, 0x1p-1022 },
    { -0x1.199999999999ap+0, NAN, -0x1p-1022 },
    { -0x1.199999999999ap+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, NAN, INF },
    { -0x1.199999999999ap+0, NAN, -INF },
    { -0x1.199999999999ap+0, NAN, NAN },
    { -0x1.199999999999ap+0, NAN, NAN },
    { -0x1.199999999999ap+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, NAN, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, NAN, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, NAN, 0x0p+0 },
    { -0x1.199999999999ap+0, NAN, -0x0p+0 },
    { -0x1.199999999999ap+0, NAN, 0x1p+0 },
    { -0x1.199999999999ap+0, NAN, -0x1p+0 },
    { -0x1.199999999999ap+0, NAN, 0x1p-1022 },
    { -0x1.199999999999ap+0, NAN, -0x1p-1022 },
    { -0x1.199999999999ap+0, NAN, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, NAN, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, NAN, INF },
    { -0x1.199999999999ap+0, NAN, -INF },
    { -0x1.199999999999ap+0, NAN, NAN },
    { -0x1.199999999999ap+0, NAN, NAN },
    { -0x1.199999999999ap+0, NAN, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, NAN, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, NAN, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, NAN, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, INF },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -INF },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, NAN },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, INF },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -INF },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, 0x1.199999999999ap+0, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, INF },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -INF },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, NAN },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1, -0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x0p+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1p+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1p-1022 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.fffffffffffffp+1023 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, INF },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -INF },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, NAN },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, 0x1.199999999999ap+0 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.ccccccccccccdp-1 },
    { -0x1.199999999999ap+0, -0x1.199999999999ap+0, -0x1.199999999999ap+0 },

  };
  static struct {
    double e;

  } expected[] = {
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1p+0 } ,
    { 0x1p+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { 0x1p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e666666666666p+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.99999999999ap-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { 0x1p-1022 } ,
    { 0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1021 } ,
    { 0x0p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { 0x0p+0 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x0p+0 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.e666666666666p+0 } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { -0x1.999999999999cp-3 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.99999999999ap-4 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1 } ,
    { 0x1.199999999999ap+1 } ,
    { 0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x1.999999999999cp-3 } ,
    { -0x1.ccccccccccccdp+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+1 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.99999999999ap-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { 0x1p+0 } ,
    { 0x1p+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { 0x1p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e666666666666p+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1p-1022 } ,
    { 0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1021 } ,
    { 0x0p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x0p+0 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { 0x0p+0 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x1.999999999999cp-3 } ,
    { -0x1.ccccccccccccdp+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.e666666666666p+0 } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { -0x1.999999999999cp-3 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.99999999999ap-4 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1 } ,
    { 0x1.199999999999ap+1 } ,
    { 0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1p-1022 } ,
    { 0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1021 } ,
    { 0x0p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.4p+2 } ,
    { 0x1.7ffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.3999999999999p+2 } ,
    { 0x1.4666666666666p+2 } ,
    { 0x1.8ccccccccccccp+1 } ,
    { 0x1.7333333333332p+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.7ffffffffffffp+1 } ,
    { -0x1.4p+2 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8ccccccccccccp+1 } ,
    { -0x1.7333333333332p+1 } ,
    { -0x1.3999999999999p+2 } ,
    { -0x1.4666666666666p+2 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.e666666666666p-1022 } ,
    { -0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.0cccccccccccdp-1021 } ,
    { 0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x0.e666666666666p-1022 } ,
    { -0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0.199999999999ap-1022 } ,
    { -0x1.e666666666666p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap-1022 } ,
    { -0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { -0x0.199999999999ap-1022 } ,
    { -0x1.0cccccccccccdp-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0p+0 } ,
    { -0x1p-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1p-1022 } ,
    { 0x1p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1021 } ,
    { 0x0p+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.7ffffffffffffp+1 } ,
    { -0x1.4p+2 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8ccccccccccccp+1 } ,
    { -0x1.7333333333332p+1 } ,
    { -0x1.3999999999999p+2 } ,
    { -0x1.4666666666666p+2 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.4p+2 } ,
    { 0x1.7ffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.3999999999999p+2 } ,
    { 0x1.4666666666666p+2 } ,
    { 0x1.8ccccccccccccp+1 } ,
    { 0x1.7333333333332p+1 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x0.e666666666666p-1022 } ,
    { -0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0.199999999999ap-1022 } ,
    { -0x1.e666666666666p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap-1022 } ,
    { -0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { -0x0.199999999999ap-1022 } ,
    { -0x1.0cccccccccccdp-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.e666666666666p-1022 } ,
    { -0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.0cccccccccccdp-1021 } ,
    { 0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { 0x0p+0 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x0p+0 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.4p+2 } ,
    { 0x1.7ffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.3999999999999p+2 } ,
    { 0x1.4666666666666p+2 } ,
    { 0x1.8ccccccccccccp+1 } ,
    { 0x1.7333333333332p+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.7ffffffffffffp+1 } ,
    { -0x1.4p+2 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8ccccccccccccp+1 } ,
    { -0x1.7333333333332p+1 } ,
    { -0x1.3999999999999p+2 } ,
    { -0x1.4666666666666p+2 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { -0x1.9999999999997p+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.999999999999fp+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { 0x1.9999999999997p+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.999999999999fp+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x0p+0 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { 0x0p+0 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.7ffffffffffffp+1 } ,
    { -0x1.4p+2 } ,
    { -0x1.fffffffffffffp+1 } ,
    { -0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8ccccccccccccp+1 } ,
    { -0x1.7333333333332p+1 } ,
    { -0x1.3999999999999p+2 } ,
    { -0x1.4666666666666p+2 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.4p+2 } ,
    { 0x1.7ffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.3999999999999p+2 } ,
    { 0x1.4666666666666p+2 } ,
    { 0x1.8ccccccccccccp+1 } ,
    { 0x1.7333333333332p+1 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { 0x1.9999999999997p+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.999999999999fp+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { -0x1.9999999999997p+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.999999999999fp+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.e666666666666p+0 } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { -0x1.999999999999cp-3 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x1.999999999999cp-3 } ,
    { -0x1.ccccccccccccdp+0 } ,
    { -0x1p+1 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.e666666666666p-1022 } ,
    { -0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x0.e666666666666p-1022 } ,
    { -0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0.199999999999ap-1022 } ,
    { -0x1.e666666666666p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { -0x1.9999999999997p+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { 0x1.9999999999997p+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.cf5c28f5c28f6p+0 } ,
    { -0x1.851eb851eb85p-3 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.b5c28f5c28f5cp+0 } ,
    { 0x1.e8f5c28f5c29p+0 } ,
    { -0x1.70a3d70a3d709p-4 } ,
    { -0x1.28f5c28f5c29p-2 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.47ae147ae143fp-7 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70a3d70a3ep+0 } ,
    { 0x1.0b851eb851eb9p+1 } ,
    { 0x1.70a3d70a3d71p-4 } ,
    { -0x1.c28f5c28f5c28p-4 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { 0x1.851eb851eb85p-3 } ,
    { -0x1.cf5c28f5c28f6p+0 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.70a3d70a3d709p-4 } ,
    { 0x1.28f5c28f5c29p-2 } ,
    { -0x1.b5c28f5c28f5cp+0 } ,
    { -0x1.e8f5c28f5c29p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.47ae147ae143fp-7 } ,
    { -0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3d70a3d71p-4 } ,
    { 0x1.c28f5c28f5c28p-4 } ,
    { -0x1.e3d70a3d70a3ep+0 } ,
    { -0x1.0b851eb851eb9p+1 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.99999999999ap-4 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1 } ,
    { 0x1.199999999999ap+1 } ,
    { 0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+1 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.0cccccccccccdp-1021 } ,
    { 0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap-1022 } ,
    { -0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { -0x0.199999999999ap-1022 } ,
    { -0x1.0cccccccccccdp-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.999999999999fp+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.999999999999fp+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.47ae147ae143fp-7 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70a3d70a3ep+0 } ,
    { 0x1.0b851eb851eb9p+1 } ,
    { 0x1.70a3d70a3d71p-4 } ,
    { -0x1.c28f5c28f5c28p-4 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.1ae147ae147afp+1 } ,
    { 0x1.ae147ae147ae8p-3 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.0e147ae147ae2p+1 } ,
    { 0x1.27ae147ae147cp+1 } ,
    { 0x1.3d70a3d70a3dap-2 } ,
    { 0x1.c28f5c28f5c31p-4 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.47ae147ae143fp-7 } ,
    { -0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3d70a3d71p-4 } ,
    { 0x1.c28f5c28f5c28p-4 } ,
    { -0x1.e3d70a3d70a3ep+0 } ,
    { -0x1.0b851eb851eb9p+1 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.ae147ae147ae8p-3 } ,
    { -0x1.1ae147ae147afp+1 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.3d70a3d70a3dap-2 } ,
    { -0x1.c28f5c28f5c31p-4 } ,
    { -0x1.0e147ae147ae2p+1 } ,
    { -0x1.27ae147ae147cp+1 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.9999999999998p-4 } ,
    { -0x1.e666666666666p+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0 } ,
    { 0x1.999999999999cp-3 } ,
    { -0x1.ccccccccccccdp+0 } ,
    { -0x1p+1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.e666666666666p+0 } ,
    { -0x1.9999999999998p-4 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp+0 } ,
    { 0x1p+1 } ,
    { 0x0p+0 } ,
    { -0x1.999999999999cp-3 } ,
    { -0x0.e666666666666p-1022 } ,
    { -0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x0.199999999999ap-1022 } ,
    { -0x1.e666666666666p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x0.e666666666666p-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.e666666666666p-1022 } ,
    { -0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { 0x1.9999999999997p+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { -0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { INF } ,
    { -0x1.9999999999997p+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { 0x1.cccccccccccccp+1023 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { 0x1.851eb851eb85p-3 } ,
    { -0x1.cf5c28f5c28f6p+0 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { -0x1.9eb851eb851ecp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.70a3d70a3d709p-4 } ,
    { 0x1.28f5c28f5c29p-2 } ,
    { -0x1.b5c28f5c28f5cp+0 } ,
    { -0x1.e8f5c28f5c29p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.47ae147ae143fp-7 } ,
    { -0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3d70a3d71p-4 } ,
    { 0x1.c28f5c28f5c28p-4 } ,
    { -0x1.e3d70a3d70a3ep+0 } ,
    { -0x1.0b851eb851eb9p+1 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.cf5c28f5c28f6p+0 } ,
    { -0x1.851eb851eb85p-3 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.9eb851eb851ecp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.b5c28f5c28f5cp+0 } ,
    { 0x1.e8f5c28f5c29p+0 } ,
    { -0x1.70a3d70a3d709p-4 } ,
    { -0x1.28f5c28f5c29p-2 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.47ae147ae143fp-7 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70a3d70a3ep+0 } ,
    { 0x1.0b851eb851eb9p+1 } ,
    { 0x1.70a3d70a3d71p-4 } ,
    { -0x1.c28f5c28f5c28p-4 } ,
    { 0x0p+0 } ,
    { -0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x0p+0 } ,
    { 0x0p+0 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1p-1022 } ,
    { -0x1p-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.99999999999ap-4 } ,
    { -0x1.0cccccccccccdp+1 } ,
    { -0x1.199999999999ap+0 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1p+1 } ,
    { -0x1.199999999999ap+1 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.0cccccccccccdp+1 } ,
    { 0x1.99999999999ap-4 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1 } ,
    { 0x1.199999999999ap+1 } ,
    { 0x1.999999999999cp-3 } ,
    { 0x0p+0 } ,
    { -0x1.199999999999ap-1022 } ,
    { -0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { -0x0.199999999999ap-1022 } ,
    { -0x1.0cccccccccccdp-1021 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1.199999999999ap-1022 } ,
    { 0x1p+0 } ,
    { -0x1p+0 } ,
    { 0x1.0cccccccccccdp-1021 } ,
    { 0x0.199999999999ap-1022 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccccccccdp-1 } ,
    { 0x1.199999999999ap+0 } ,
    { -0x1.ccccccccccccdp-1 } ,
    { -0x1.199999999999ap+0 } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.999999999999fp+1020 } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.999999999999fp+1020 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.47ae147ae143fp-7 } ,
    { -0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.fae147ae147afp-1 } ,
    { -0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3d70a3d71p-4 } ,
    { 0x1.c28f5c28f5c28p-4 } ,
    { -0x1.e3d70a3d70a3ep+0 } ,
    { -0x1.0b851eb851eb9p+1 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.ae147ae147ae8p-3 } ,
    { -0x1.1ae147ae147afp+1 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { -0x1.35c28f5c28f5dp+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.3d70a3d70a3dap-2 } ,
    { -0x1.c28f5c28f5c31p-4 } ,
    { -0x1.0e147ae147ae2p+1 } ,
    { -0x1.27ae147ae147cp+1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fd70a3d70a3d8p+0 } ,
    { -0x1.47ae147ae143fp-7 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fae147ae147afp-1 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70a3d70a3ep+0 } ,
    { 0x1.0b851eb851eb9p+1 } ,
    { 0x1.70a3d70a3d71p-4 } ,
    { -0x1.c28f5c28f5c28p-4 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.1ae147ae147afp+1 } ,
    { 0x1.ae147ae147ae8p-3 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.35c28f5c28f5dp+0 } ,
    { 0x1.fffffffffffffp+1023 } ,
    { -0x1.fffffffffffffp+1023 } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.0e147ae147ae2p+1 } ,
    { 0x1.27ae147ae147cp+1 } ,
    { 0x1.3d70a3d70a3dap-2 } ,
    { 0x1.c28f5c28f5c31p-4 } ,

  };
  int i;
  int ok = 1;
  for (i = 0; i < sizeof(args) / sizeof(args[0]); ++i) {

    double result = fma(args[i].a

                                , args[i].a2

                                , args[i].a3


                                );

    if (!CMP(DOUBLE, result, expected[i].e)

        ) {
      fprintf(stderr,
              "fma(%.16e"

              ", %.16e"

              ", %.16e"


              ")\n",
              args[i].a

              , args[i].a2

              , args[i].a3

              );

      fprintf(stderr,
              "returned: %.16e, expected: %.16e\n",
              result,
              expected[i].e
              );

      ok = 0;
    }
  }
  TEST(ok);
}
