/*
   This file is automatically generated.
 */

/**
 * @file math/fmaf.test.c
 */
#include <math.h>
#include <float.h>
#include <stdio.h>
#include <stdint.h>
#include "common.h"

#ifndef NAN
#define NAN (0.0f/0.0f)
#endif

#ifndef INF
#define INF (1.0f/0.0f)
#endif

#define ABS_FLOAT       fabsf
#define ABS_DOUBLE      fabs
#define ABS_LONG_DOUBLE fabsl

#define TOLERANCE_FLOAT       0x1p-20
#define TOLERANCE_DOUBLE      0x1p-50
#define TOLERANCE_LONG_DOUBLE 0x1p-60

#define RET_MINIMUM_FLOAT       0x1p-126
#define RET_MINIMUM_DOUBLE      0x1p-1022
#define RET_MINIMUM_LONG_DOUBLE 0x1p-16382

#define max(a, b) ((a > b) ? a : b)

#define CMP_FLOAT(x, t) \
  static int compare_##x(t actual, t expected) { \
    if (isnan(actual) && isnan(expected)) \
      return 1; \
    else if (isinf(actual) && isinf(expected)) \
      return ((actual > (t)0.0) == (expected > (t)0.0)); \
    else \
      return (ABS_##x(actual - expected) < max(ABS_##x(expected * TOLERANCE_##x), RET_MINIMUM_##x)); \
  }

#define CMP_INT(x, t) \
  static int compare_##x(t actual, t expected) { return (actual == expected); }

CMP_FLOAT(FLOAT, float)
CMP_FLOAT(DOUBLE, double)
/* CMP_FLOAT(LONG_DOUBLE, long double) */

CMP_INT(INT, int)
CMP_INT(LONG, long)
CMP_INT(LONG_LONG, long long)

#define _CMP(t, a, e) compare_##t(a, e)
#define CMP(t, a, e) _CMP(t, a, e)

/**
 * @testname fmaf_2
 * @testfor fmaf
 */
TEST_CASE(fmaf_2)
{

  static struct {
    float a;

    float a2;

    float a3;

  } args[] = {
    { 0x0p+0F, 0x0p+0F, 0x0p+0F },
    { 0x0p+0F, 0x0p+0F, -0x0p+0F },
    { 0x0p+0F, 0x0p+0F, 0x1p+0F },
    { 0x0p+0F, 0x0p+0F, -0x1p+0F },
    { 0x0p+0F, 0x0p+0F, 0x1p-126F },
    { 0x0p+0F, 0x0p+0F, -0x1p-126F },
    { 0x0p+0F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x0p+0F, INF },
    { 0x0p+0F, 0x0p+0F, -INF },
    { 0x0p+0F, 0x0p+0F, NAN },
    { 0x0p+0F, 0x0p+0F, NAN },
    { 0x0p+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x0p+0F, 0x0p+0F },
    { 0x0p+0F, -0x0p+0F, -0x0p+0F },
    { 0x0p+0F, -0x0p+0F, 0x1p+0F },
    { 0x0p+0F, -0x0p+0F, -0x1p+0F },
    { 0x0p+0F, -0x0p+0F, 0x1p-126F },
    { 0x0p+0F, -0x0p+0F, -0x1p-126F },
    { 0x0p+0F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x0p+0F, INF },
    { 0x0p+0F, -0x0p+0F, -INF },
    { 0x0p+0F, -0x0p+0F, NAN },
    { 0x0p+0F, -0x0p+0F, NAN },
    { 0x0p+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x0p+0F, 0x1p+0F, 0x0p+0F },
    { 0x0p+0F, 0x1p+0F, -0x0p+0F },
    { 0x0p+0F, 0x1p+0F, 0x1p+0F },
    { 0x0p+0F, 0x1p+0F, -0x1p+0F },
    { 0x0p+0F, 0x1p+0F, 0x1p-126F },
    { 0x0p+0F, 0x1p+0F, -0x1p-126F },
    { 0x0p+0F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x1p+0F, INF },
    { 0x0p+0F, 0x1p+0F, -INF },
    { 0x0p+0F, 0x1p+0F, NAN },
    { 0x0p+0F, 0x1p+0F, NAN },
    { 0x0p+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x1p+0F, 0x0p+0F },
    { 0x0p+0F, -0x1p+0F, -0x0p+0F },
    { 0x0p+0F, -0x1p+0F, 0x1p+0F },
    { 0x0p+0F, -0x1p+0F, -0x1p+0F },
    { 0x0p+0F, -0x1p+0F, 0x1p-126F },
    { 0x0p+0F, -0x1p+0F, -0x1p-126F },
    { 0x0p+0F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x1p+0F, INF },
    { 0x0p+0F, -0x1p+0F, -INF },
    { 0x0p+0F, -0x1p+0F, NAN },
    { 0x0p+0F, -0x1p+0F, NAN },
    { 0x0p+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x0p+0F, 0x1p-126F, 0x0p+0F },
    { 0x0p+0F, 0x1p-126F, -0x0p+0F },
    { 0x0p+0F, 0x1p-126F, 0x1p+0F },
    { 0x0p+0F, 0x1p-126F, -0x1p+0F },
    { 0x0p+0F, 0x1p-126F, 0x1p-126F },
    { 0x0p+0F, 0x1p-126F, -0x1p-126F },
    { 0x0p+0F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x1p-126F, INF },
    { 0x0p+0F, 0x1p-126F, -INF },
    { 0x0p+0F, 0x1p-126F, NAN },
    { 0x0p+0F, 0x1p-126F, NAN },
    { 0x0p+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x1p-126F, 0x0p+0F },
    { 0x0p+0F, -0x1p-126F, -0x0p+0F },
    { 0x0p+0F, -0x1p-126F, 0x1p+0F },
    { 0x0p+0F, -0x1p-126F, -0x1p+0F },
    { 0x0p+0F, -0x1p-126F, 0x1p-126F },
    { 0x0p+0F, -0x1p-126F, -0x1p-126F },
    { 0x0p+0F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x1p-126F, INF },
    { 0x0p+0F, -0x1p-126F, -INF },
    { 0x0p+0F, -0x1p-126F, NAN },
    { 0x0p+0F, -0x1p-126F, NAN },
    { 0x0p+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x0p+0F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x0p+0F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x0p+0F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x0p+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x1.fffffep+127F, INF },
    { 0x0p+0F, 0x1.fffffep+127F, -INF },
    { 0x0p+0F, 0x1.fffffep+127F, NAN },
    { 0x0p+0F, 0x1.fffffep+127F, NAN },
    { 0x0p+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x0p+0F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x0p+0F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x0p+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x1.fffffep+127F, INF },
    { 0x0p+0F, -0x1.fffffep+127F, -INF },
    { 0x0p+0F, -0x1.fffffep+127F, NAN },
    { 0x0p+0F, -0x1.fffffep+127F, NAN },
    { 0x0p+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x0p+0F, INF, 0x0p+0F },
    { 0x0p+0F, INF, -0x0p+0F },
    { 0x0p+0F, INF, 0x1p+0F },
    { 0x0p+0F, INF, -0x1p+0F },
    { 0x0p+0F, INF, 0x1p-126F },
    { 0x0p+0F, INF, -0x1p-126F },
    { 0x0p+0F, INF, 0x1.fffffep+127F },
    { 0x0p+0F, INF, -0x1.fffffep+127F },
    { 0x0p+0F, INF, INF },
    { 0x0p+0F, INF, -INF },
    { 0x0p+0F, INF, NAN },
    { 0x0p+0F, INF, NAN },
    { 0x0p+0F, INF, 0x1.ccccccp-1F },
    { 0x0p+0F, INF, 0x1.19999ap+0F },
    { 0x0p+0F, INF, -0x1.ccccccp-1F },
    { 0x0p+0F, INF, -0x1.19999ap+0F },
    { 0x0p+0F, -INF, 0x0p+0F },
    { 0x0p+0F, -INF, -0x0p+0F },
    { 0x0p+0F, -INF, 0x1p+0F },
    { 0x0p+0F, -INF, -0x1p+0F },
    { 0x0p+0F, -INF, 0x1p-126F },
    { 0x0p+0F, -INF, -0x1p-126F },
    { 0x0p+0F, -INF, 0x1.fffffep+127F },
    { 0x0p+0F, -INF, -0x1.fffffep+127F },
    { 0x0p+0F, -INF, INF },
    { 0x0p+0F, -INF, -INF },
    { 0x0p+0F, -INF, NAN },
    { 0x0p+0F, -INF, NAN },
    { 0x0p+0F, -INF, 0x1.ccccccp-1F },
    { 0x0p+0F, -INF, 0x1.19999ap+0F },
    { 0x0p+0F, -INF, -0x1.ccccccp-1F },
    { 0x0p+0F, -INF, -0x1.19999ap+0F },
    { 0x0p+0F, NAN, 0x0p+0F },
    { 0x0p+0F, NAN, -0x0p+0F },
    { 0x0p+0F, NAN, 0x1p+0F },
    { 0x0p+0F, NAN, -0x1p+0F },
    { 0x0p+0F, NAN, 0x1p-126F },
    { 0x0p+0F, NAN, -0x1p-126F },
    { 0x0p+0F, NAN, 0x1.fffffep+127F },
    { 0x0p+0F, NAN, -0x1.fffffep+127F },
    { 0x0p+0F, NAN, INF },
    { 0x0p+0F, NAN, -INF },
    { 0x0p+0F, NAN, NAN },
    { 0x0p+0F, NAN, NAN },
    { 0x0p+0F, NAN, 0x1.ccccccp-1F },
    { 0x0p+0F, NAN, 0x1.19999ap+0F },
    { 0x0p+0F, NAN, -0x1.ccccccp-1F },
    { 0x0p+0F, NAN, -0x1.19999ap+0F },
    { 0x0p+0F, NAN, 0x0p+0F },
    { 0x0p+0F, NAN, -0x0p+0F },
    { 0x0p+0F, NAN, 0x1p+0F },
    { 0x0p+0F, NAN, -0x1p+0F },
    { 0x0p+0F, NAN, 0x1p-126F },
    { 0x0p+0F, NAN, -0x1p-126F },
    { 0x0p+0F, NAN, 0x1.fffffep+127F },
    { 0x0p+0F, NAN, -0x1.fffffep+127F },
    { 0x0p+0F, NAN, INF },
    { 0x0p+0F, NAN, -INF },
    { 0x0p+0F, NAN, NAN },
    { 0x0p+0F, NAN, NAN },
    { 0x0p+0F, NAN, 0x1.ccccccp-1F },
    { 0x0p+0F, NAN, 0x1.19999ap+0F },
    { 0x0p+0F, NAN, -0x1.ccccccp-1F },
    { 0x0p+0F, NAN, -0x1.19999ap+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x1.ccccccp-1F, INF },
    { 0x0p+0F, 0x1.ccccccp-1F, -INF },
    { 0x0p+0F, 0x1.ccccccp-1F, NAN },
    { 0x0p+0F, 0x1.ccccccp-1F, NAN },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x0p+0F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x0p+0F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x0p+0F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x0p+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x0p+0F, 0x1.19999ap+0F, INF },
    { 0x0p+0F, 0x1.19999ap+0F, -INF },
    { 0x0p+0F, 0x1.19999ap+0F, NAN },
    { 0x0p+0F, 0x1.19999ap+0F, NAN },
    { 0x0p+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x1.ccccccp-1F, INF },
    { 0x0p+0F, -0x1.ccccccp-1F, -INF },
    { 0x0p+0F, -0x1.ccccccp-1F, NAN },
    { 0x0p+0F, -0x1.ccccccp-1F, NAN },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x0p+0F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x0p+0F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x0p+0F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x0p+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x0p+0F, -0x1.19999ap+0F, INF },
    { 0x0p+0F, -0x1.19999ap+0F, -INF },
    { 0x0p+0F, -0x1.19999ap+0F, NAN },
    { 0x0p+0F, -0x1.19999ap+0F, NAN },
    { 0x0p+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x0p+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x0p+0F, 0x0p+0F },
    { -0x0p+0F, 0x0p+0F, -0x0p+0F },
    { -0x0p+0F, 0x0p+0F, 0x1p+0F },
    { -0x0p+0F, 0x0p+0F, -0x1p+0F },
    { -0x0p+0F, 0x0p+0F, 0x1p-126F },
    { -0x0p+0F, 0x0p+0F, -0x1p-126F },
    { -0x0p+0F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x0p+0F, INF },
    { -0x0p+0F, 0x0p+0F, -INF },
    { -0x0p+0F, 0x0p+0F, NAN },
    { -0x0p+0F, 0x0p+0F, NAN },
    { -0x0p+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x0p+0F, 0x0p+0F },
    { -0x0p+0F, -0x0p+0F, -0x0p+0F },
    { -0x0p+0F, -0x0p+0F, 0x1p+0F },
    { -0x0p+0F, -0x0p+0F, -0x1p+0F },
    { -0x0p+0F, -0x0p+0F, 0x1p-126F },
    { -0x0p+0F, -0x0p+0F, -0x1p-126F },
    { -0x0p+0F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x0p+0F, INF },
    { -0x0p+0F, -0x0p+0F, -INF },
    { -0x0p+0F, -0x0p+0F, NAN },
    { -0x0p+0F, -0x0p+0F, NAN },
    { -0x0p+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x1p+0F, 0x0p+0F },
    { -0x0p+0F, 0x1p+0F, -0x0p+0F },
    { -0x0p+0F, 0x1p+0F, 0x1p+0F },
    { -0x0p+0F, 0x1p+0F, -0x1p+0F },
    { -0x0p+0F, 0x1p+0F, 0x1p-126F },
    { -0x0p+0F, 0x1p+0F, -0x1p-126F },
    { -0x0p+0F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x1p+0F, INF },
    { -0x0p+0F, 0x1p+0F, -INF },
    { -0x0p+0F, 0x1p+0F, NAN },
    { -0x0p+0F, 0x1p+0F, NAN },
    { -0x0p+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x1p+0F, 0x0p+0F },
    { -0x0p+0F, -0x1p+0F, -0x0p+0F },
    { -0x0p+0F, -0x1p+0F, 0x1p+0F },
    { -0x0p+0F, -0x1p+0F, -0x1p+0F },
    { -0x0p+0F, -0x1p+0F, 0x1p-126F },
    { -0x0p+0F, -0x1p+0F, -0x1p-126F },
    { -0x0p+0F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x1p+0F, INF },
    { -0x0p+0F, -0x1p+0F, -INF },
    { -0x0p+0F, -0x1p+0F, NAN },
    { -0x0p+0F, -0x1p+0F, NAN },
    { -0x0p+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x1p-126F, 0x0p+0F },
    { -0x0p+0F, 0x1p-126F, -0x0p+0F },
    { -0x0p+0F, 0x1p-126F, 0x1p+0F },
    { -0x0p+0F, 0x1p-126F, -0x1p+0F },
    { -0x0p+0F, 0x1p-126F, 0x1p-126F },
    { -0x0p+0F, 0x1p-126F, -0x1p-126F },
    { -0x0p+0F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x1p-126F, INF },
    { -0x0p+0F, 0x1p-126F, -INF },
    { -0x0p+0F, 0x1p-126F, NAN },
    { -0x0p+0F, 0x1p-126F, NAN },
    { -0x0p+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x1p-126F, 0x0p+0F },
    { -0x0p+0F, -0x1p-126F, -0x0p+0F },
    { -0x0p+0F, -0x1p-126F, 0x1p+0F },
    { -0x0p+0F, -0x1p-126F, -0x1p+0F },
    { -0x0p+0F, -0x1p-126F, 0x1p-126F },
    { -0x0p+0F, -0x1p-126F, -0x1p-126F },
    { -0x0p+0F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x1p-126F, INF },
    { -0x0p+0F, -0x1p-126F, -INF },
    { -0x0p+0F, -0x1p-126F, NAN },
    { -0x0p+0F, -0x1p-126F, NAN },
    { -0x0p+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x0p+0F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x0p+0F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x0p+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x1.fffffep+127F, INF },
    { -0x0p+0F, 0x1.fffffep+127F, -INF },
    { -0x0p+0F, 0x1.fffffep+127F, NAN },
    { -0x0p+0F, 0x1.fffffep+127F, NAN },
    { -0x0p+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x0p+0F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x0p+0F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x0p+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x1.fffffep+127F, INF },
    { -0x0p+0F, -0x1.fffffep+127F, -INF },
    { -0x0p+0F, -0x1.fffffep+127F, NAN },
    { -0x0p+0F, -0x1.fffffep+127F, NAN },
    { -0x0p+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x0p+0F, INF, 0x0p+0F },
    { -0x0p+0F, INF, -0x0p+0F },
    { -0x0p+0F, INF, 0x1p+0F },
    { -0x0p+0F, INF, -0x1p+0F },
    { -0x0p+0F, INF, 0x1p-126F },
    { -0x0p+0F, INF, -0x1p-126F },
    { -0x0p+0F, INF, 0x1.fffffep+127F },
    { -0x0p+0F, INF, -0x1.fffffep+127F },
    { -0x0p+0F, INF, INF },
    { -0x0p+0F, INF, -INF },
    { -0x0p+0F, INF, NAN },
    { -0x0p+0F, INF, NAN },
    { -0x0p+0F, INF, 0x1.ccccccp-1F },
    { -0x0p+0F, INF, 0x1.19999ap+0F },
    { -0x0p+0F, INF, -0x1.ccccccp-1F },
    { -0x0p+0F, INF, -0x1.19999ap+0F },
    { -0x0p+0F, -INF, 0x0p+0F },
    { -0x0p+0F, -INF, -0x0p+0F },
    { -0x0p+0F, -INF, 0x1p+0F },
    { -0x0p+0F, -INF, -0x1p+0F },
    { -0x0p+0F, -INF, 0x1p-126F },
    { -0x0p+0F, -INF, -0x1p-126F },
    { -0x0p+0F, -INF, 0x1.fffffep+127F },
    { -0x0p+0F, -INF, -0x1.fffffep+127F },
    { -0x0p+0F, -INF, INF },
    { -0x0p+0F, -INF, -INF },
    { -0x0p+0F, -INF, NAN },
    { -0x0p+0F, -INF, NAN },
    { -0x0p+0F, -INF, 0x1.ccccccp-1F },
    { -0x0p+0F, -INF, 0x1.19999ap+0F },
    { -0x0p+0F, -INF, -0x1.ccccccp-1F },
    { -0x0p+0F, -INF, -0x1.19999ap+0F },
    { -0x0p+0F, NAN, 0x0p+0F },
    { -0x0p+0F, NAN, -0x0p+0F },
    { -0x0p+0F, NAN, 0x1p+0F },
    { -0x0p+0F, NAN, -0x1p+0F },
    { -0x0p+0F, NAN, 0x1p-126F },
    { -0x0p+0F, NAN, -0x1p-126F },
    { -0x0p+0F, NAN, 0x1.fffffep+127F },
    { -0x0p+0F, NAN, -0x1.fffffep+127F },
    { -0x0p+0F, NAN, INF },
    { -0x0p+0F, NAN, -INF },
    { -0x0p+0F, NAN, NAN },
    { -0x0p+0F, NAN, NAN },
    { -0x0p+0F, NAN, 0x1.ccccccp-1F },
    { -0x0p+0F, NAN, 0x1.19999ap+0F },
    { -0x0p+0F, NAN, -0x1.ccccccp-1F },
    { -0x0p+0F, NAN, -0x1.19999ap+0F },
    { -0x0p+0F, NAN, 0x0p+0F },
    { -0x0p+0F, NAN, -0x0p+0F },
    { -0x0p+0F, NAN, 0x1p+0F },
    { -0x0p+0F, NAN, -0x1p+0F },
    { -0x0p+0F, NAN, 0x1p-126F },
    { -0x0p+0F, NAN, -0x1p-126F },
    { -0x0p+0F, NAN, 0x1.fffffep+127F },
    { -0x0p+0F, NAN, -0x1.fffffep+127F },
    { -0x0p+0F, NAN, INF },
    { -0x0p+0F, NAN, -INF },
    { -0x0p+0F, NAN, NAN },
    { -0x0p+0F, NAN, NAN },
    { -0x0p+0F, NAN, 0x1.ccccccp-1F },
    { -0x0p+0F, NAN, 0x1.19999ap+0F },
    { -0x0p+0F, NAN, -0x1.ccccccp-1F },
    { -0x0p+0F, NAN, -0x1.19999ap+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x1.ccccccp-1F, INF },
    { -0x0p+0F, 0x1.ccccccp-1F, -INF },
    { -0x0p+0F, 0x1.ccccccp-1F, NAN },
    { -0x0p+0F, 0x1.ccccccp-1F, NAN },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x0p+0F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x0p+0F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x0p+0F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x0p+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x0p+0F, 0x1.19999ap+0F, INF },
    { -0x0p+0F, 0x1.19999ap+0F, -INF },
    { -0x0p+0F, 0x1.19999ap+0F, NAN },
    { -0x0p+0F, 0x1.19999ap+0F, NAN },
    { -0x0p+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x1.ccccccp-1F, INF },
    { -0x0p+0F, -0x1.ccccccp-1F, -INF },
    { -0x0p+0F, -0x1.ccccccp-1F, NAN },
    { -0x0p+0F, -0x1.ccccccp-1F, NAN },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x0p+0F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x0p+0F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x0p+0F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x0p+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x0p+0F, -0x1.19999ap+0F, INF },
    { -0x0p+0F, -0x1.19999ap+0F, -INF },
    { -0x0p+0F, -0x1.19999ap+0F, NAN },
    { -0x0p+0F, -0x1.19999ap+0F, NAN },
    { -0x0p+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x0p+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x0p+0F, 0x0p+0F },
    { 0x1p+0F, 0x0p+0F, -0x0p+0F },
    { 0x1p+0F, 0x0p+0F, 0x1p+0F },
    { 0x1p+0F, 0x0p+0F, -0x1p+0F },
    { 0x1p+0F, 0x0p+0F, 0x1p-126F },
    { 0x1p+0F, 0x0p+0F, -0x1p-126F },
    { 0x1p+0F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x0p+0F, INF },
    { 0x1p+0F, 0x0p+0F, -INF },
    { 0x1p+0F, 0x0p+0F, NAN },
    { 0x1p+0F, 0x0p+0F, NAN },
    { 0x1p+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x0p+0F, 0x0p+0F },
    { 0x1p+0F, -0x0p+0F, -0x0p+0F },
    { 0x1p+0F, -0x0p+0F, 0x1p+0F },
    { 0x1p+0F, -0x0p+0F, -0x1p+0F },
    { 0x1p+0F, -0x0p+0F, 0x1p-126F },
    { 0x1p+0F, -0x0p+0F, -0x1p-126F },
    { 0x1p+0F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x0p+0F, INF },
    { 0x1p+0F, -0x0p+0F, -INF },
    { 0x1p+0F, -0x0p+0F, NAN },
    { 0x1p+0F, -0x0p+0F, NAN },
    { 0x1p+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x1p+0F, 0x0p+0F },
    { 0x1p+0F, 0x1p+0F, -0x0p+0F },
    { 0x1p+0F, 0x1p+0F, 0x1p+0F },
    { 0x1p+0F, 0x1p+0F, -0x1p+0F },
    { 0x1p+0F, 0x1p+0F, 0x1p-126F },
    { 0x1p+0F, 0x1p+0F, -0x1p-126F },
    { 0x1p+0F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x1p+0F, INF },
    { 0x1p+0F, 0x1p+0F, -INF },
    { 0x1p+0F, 0x1p+0F, NAN },
    { 0x1p+0F, 0x1p+0F, NAN },
    { 0x1p+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x1p+0F, 0x0p+0F },
    { 0x1p+0F, -0x1p+0F, -0x0p+0F },
    { 0x1p+0F, -0x1p+0F, 0x1p+0F },
    { 0x1p+0F, -0x1p+0F, -0x1p+0F },
    { 0x1p+0F, -0x1p+0F, 0x1p-126F },
    { 0x1p+0F, -0x1p+0F, -0x1p-126F },
    { 0x1p+0F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x1p+0F, INF },
    { 0x1p+0F, -0x1p+0F, -INF },
    { 0x1p+0F, -0x1p+0F, NAN },
    { 0x1p+0F, -0x1p+0F, NAN },
    { 0x1p+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x1p-126F, 0x0p+0F },
    { 0x1p+0F, 0x1p-126F, -0x0p+0F },
    { 0x1p+0F, 0x1p-126F, 0x1p+0F },
    { 0x1p+0F, 0x1p-126F, -0x1p+0F },
    { 0x1p+0F, 0x1p-126F, 0x1p-126F },
    { 0x1p+0F, 0x1p-126F, -0x1p-126F },
    { 0x1p+0F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x1p-126F, INF },
    { 0x1p+0F, 0x1p-126F, -INF },
    { 0x1p+0F, 0x1p-126F, NAN },
    { 0x1p+0F, 0x1p-126F, NAN },
    { 0x1p+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x1p-126F, 0x0p+0F },
    { 0x1p+0F, -0x1p-126F, -0x0p+0F },
    { 0x1p+0F, -0x1p-126F, 0x1p+0F },
    { 0x1p+0F, -0x1p-126F, -0x1p+0F },
    { 0x1p+0F, -0x1p-126F, 0x1p-126F },
    { 0x1p+0F, -0x1p-126F, -0x1p-126F },
    { 0x1p+0F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x1p-126F, INF },
    { 0x1p+0F, -0x1p-126F, -INF },
    { 0x1p+0F, -0x1p-126F, NAN },
    { 0x1p+0F, -0x1p-126F, NAN },
    { 0x1p+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x1p+0F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x1p+0F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x1p+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x1.fffffep+127F, INF },
    { 0x1p+0F, 0x1.fffffep+127F, -INF },
    { 0x1p+0F, 0x1.fffffep+127F, NAN },
    { 0x1p+0F, 0x1.fffffep+127F, NAN },
    { 0x1p+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x1p+0F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x1p+0F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x1p+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x1.fffffep+127F, INF },
    { 0x1p+0F, -0x1.fffffep+127F, -INF },
    { 0x1p+0F, -0x1.fffffep+127F, NAN },
    { 0x1p+0F, -0x1.fffffep+127F, NAN },
    { 0x1p+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1p+0F, INF, 0x0p+0F },
    { 0x1p+0F, INF, -0x0p+0F },
    { 0x1p+0F, INF, 0x1p+0F },
    { 0x1p+0F, INF, -0x1p+0F },
    { 0x1p+0F, INF, 0x1p-126F },
    { 0x1p+0F, INF, -0x1p-126F },
    { 0x1p+0F, INF, 0x1.fffffep+127F },
    { 0x1p+0F, INF, -0x1.fffffep+127F },
    { 0x1p+0F, INF, INF },
    { 0x1p+0F, INF, -INF },
    { 0x1p+0F, INF, NAN },
    { 0x1p+0F, INF, NAN },
    { 0x1p+0F, INF, 0x1.ccccccp-1F },
    { 0x1p+0F, INF, 0x1.19999ap+0F },
    { 0x1p+0F, INF, -0x1.ccccccp-1F },
    { 0x1p+0F, INF, -0x1.19999ap+0F },
    { 0x1p+0F, -INF, 0x0p+0F },
    { 0x1p+0F, -INF, -0x0p+0F },
    { 0x1p+0F, -INF, 0x1p+0F },
    { 0x1p+0F, -INF, -0x1p+0F },
    { 0x1p+0F, -INF, 0x1p-126F },
    { 0x1p+0F, -INF, -0x1p-126F },
    { 0x1p+0F, -INF, 0x1.fffffep+127F },
    { 0x1p+0F, -INF, -0x1.fffffep+127F },
    { 0x1p+0F, -INF, INF },
    { 0x1p+0F, -INF, -INF },
    { 0x1p+0F, -INF, NAN },
    { 0x1p+0F, -INF, NAN },
    { 0x1p+0F, -INF, 0x1.ccccccp-1F },
    { 0x1p+0F, -INF, 0x1.19999ap+0F },
    { 0x1p+0F, -INF, -0x1.ccccccp-1F },
    { 0x1p+0F, -INF, -0x1.19999ap+0F },
    { 0x1p+0F, NAN, 0x0p+0F },
    { 0x1p+0F, NAN, -0x0p+0F },
    { 0x1p+0F, NAN, 0x1p+0F },
    { 0x1p+0F, NAN, -0x1p+0F },
    { 0x1p+0F, NAN, 0x1p-126F },
    { 0x1p+0F, NAN, -0x1p-126F },
    { 0x1p+0F, NAN, 0x1.fffffep+127F },
    { 0x1p+0F, NAN, -0x1.fffffep+127F },
    { 0x1p+0F, NAN, INF },
    { 0x1p+0F, NAN, -INF },
    { 0x1p+0F, NAN, NAN },
    { 0x1p+0F, NAN, NAN },
    { 0x1p+0F, NAN, 0x1.ccccccp-1F },
    { 0x1p+0F, NAN, 0x1.19999ap+0F },
    { 0x1p+0F, NAN, -0x1.ccccccp-1F },
    { 0x1p+0F, NAN, -0x1.19999ap+0F },
    { 0x1p+0F, NAN, 0x0p+0F },
    { 0x1p+0F, NAN, -0x0p+0F },
    { 0x1p+0F, NAN, 0x1p+0F },
    { 0x1p+0F, NAN, -0x1p+0F },
    { 0x1p+0F, NAN, 0x1p-126F },
    { 0x1p+0F, NAN, -0x1p-126F },
    { 0x1p+0F, NAN, 0x1.fffffep+127F },
    { 0x1p+0F, NAN, -0x1.fffffep+127F },
    { 0x1p+0F, NAN, INF },
    { 0x1p+0F, NAN, -INF },
    { 0x1p+0F, NAN, NAN },
    { 0x1p+0F, NAN, NAN },
    { 0x1p+0F, NAN, 0x1.ccccccp-1F },
    { 0x1p+0F, NAN, 0x1.19999ap+0F },
    { 0x1p+0F, NAN, -0x1.ccccccp-1F },
    { 0x1p+0F, NAN, -0x1.19999ap+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x1.ccccccp-1F, INF },
    { 0x1p+0F, 0x1.ccccccp-1F, -INF },
    { 0x1p+0F, 0x1.ccccccp-1F, NAN },
    { 0x1p+0F, 0x1.ccccccp-1F, NAN },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1p+0F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x1p+0F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x1p+0F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x1p+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1p+0F, 0x1.19999ap+0F, INF },
    { 0x1p+0F, 0x1.19999ap+0F, -INF },
    { 0x1p+0F, 0x1.19999ap+0F, NAN },
    { 0x1p+0F, 0x1.19999ap+0F, NAN },
    { 0x1p+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x1.ccccccp-1F, INF },
    { 0x1p+0F, -0x1.ccccccp-1F, -INF },
    { 0x1p+0F, -0x1.ccccccp-1F, NAN },
    { 0x1p+0F, -0x1.ccccccp-1F, NAN },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1p+0F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x1p+0F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x1p+0F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x1p+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1p+0F, -0x1.19999ap+0F, INF },
    { 0x1p+0F, -0x1.19999ap+0F, -INF },
    { 0x1p+0F, -0x1.19999ap+0F, NAN },
    { 0x1p+0F, -0x1.19999ap+0F, NAN },
    { 0x1p+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1p+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x0p+0F, 0x0p+0F },
    { -0x1p+0F, 0x0p+0F, -0x0p+0F },
    { -0x1p+0F, 0x0p+0F, 0x1p+0F },
    { -0x1p+0F, 0x0p+0F, -0x1p+0F },
    { -0x1p+0F, 0x0p+0F, 0x1p-126F },
    { -0x1p+0F, 0x0p+0F, -0x1p-126F },
    { -0x1p+0F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x0p+0F, INF },
    { -0x1p+0F, 0x0p+0F, -INF },
    { -0x1p+0F, 0x0p+0F, NAN },
    { -0x1p+0F, 0x0p+0F, NAN },
    { -0x1p+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x0p+0F, 0x0p+0F },
    { -0x1p+0F, -0x0p+0F, -0x0p+0F },
    { -0x1p+0F, -0x0p+0F, 0x1p+0F },
    { -0x1p+0F, -0x0p+0F, -0x1p+0F },
    { -0x1p+0F, -0x0p+0F, 0x1p-126F },
    { -0x1p+0F, -0x0p+0F, -0x1p-126F },
    { -0x1p+0F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x0p+0F, INF },
    { -0x1p+0F, -0x0p+0F, -INF },
    { -0x1p+0F, -0x0p+0F, NAN },
    { -0x1p+0F, -0x0p+0F, NAN },
    { -0x1p+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x1p+0F, 0x0p+0F },
    { -0x1p+0F, 0x1p+0F, -0x0p+0F },
    { -0x1p+0F, 0x1p+0F, 0x1p+0F },
    { -0x1p+0F, 0x1p+0F, -0x1p+0F },
    { -0x1p+0F, 0x1p+0F, 0x1p-126F },
    { -0x1p+0F, 0x1p+0F, -0x1p-126F },
    { -0x1p+0F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x1p+0F, INF },
    { -0x1p+0F, 0x1p+0F, -INF },
    { -0x1p+0F, 0x1p+0F, NAN },
    { -0x1p+0F, 0x1p+0F, NAN },
    { -0x1p+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x1p+0F, 0x0p+0F },
    { -0x1p+0F, -0x1p+0F, -0x0p+0F },
    { -0x1p+0F, -0x1p+0F, 0x1p+0F },
    { -0x1p+0F, -0x1p+0F, -0x1p+0F },
    { -0x1p+0F, -0x1p+0F, 0x1p-126F },
    { -0x1p+0F, -0x1p+0F, -0x1p-126F },
    { -0x1p+0F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x1p+0F, INF },
    { -0x1p+0F, -0x1p+0F, -INF },
    { -0x1p+0F, -0x1p+0F, NAN },
    { -0x1p+0F, -0x1p+0F, NAN },
    { -0x1p+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x1p-126F, 0x0p+0F },
    { -0x1p+0F, 0x1p-126F, -0x0p+0F },
    { -0x1p+0F, 0x1p-126F, 0x1p+0F },
    { -0x1p+0F, 0x1p-126F, -0x1p+0F },
    { -0x1p+0F, 0x1p-126F, 0x1p-126F },
    { -0x1p+0F, 0x1p-126F, -0x1p-126F },
    { -0x1p+0F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x1p-126F, INF },
    { -0x1p+0F, 0x1p-126F, -INF },
    { -0x1p+0F, 0x1p-126F, NAN },
    { -0x1p+0F, 0x1p-126F, NAN },
    { -0x1p+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x1p-126F, 0x0p+0F },
    { -0x1p+0F, -0x1p-126F, -0x0p+0F },
    { -0x1p+0F, -0x1p-126F, 0x1p+0F },
    { -0x1p+0F, -0x1p-126F, -0x1p+0F },
    { -0x1p+0F, -0x1p-126F, 0x1p-126F },
    { -0x1p+0F, -0x1p-126F, -0x1p-126F },
    { -0x1p+0F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x1p-126F, INF },
    { -0x1p+0F, -0x1p-126F, -INF },
    { -0x1p+0F, -0x1p-126F, NAN },
    { -0x1p+0F, -0x1p-126F, NAN },
    { -0x1p+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x1p+0F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x1p+0F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x1p+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x1.fffffep+127F, INF },
    { -0x1p+0F, 0x1.fffffep+127F, -INF },
    { -0x1p+0F, 0x1.fffffep+127F, NAN },
    { -0x1p+0F, 0x1.fffffep+127F, NAN },
    { -0x1p+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x1p+0F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x1p+0F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x1p+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x1.fffffep+127F, INF },
    { -0x1p+0F, -0x1.fffffep+127F, -INF },
    { -0x1p+0F, -0x1.fffffep+127F, NAN },
    { -0x1p+0F, -0x1.fffffep+127F, NAN },
    { -0x1p+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1p+0F, INF, 0x0p+0F },
    { -0x1p+0F, INF, -0x0p+0F },
    { -0x1p+0F, INF, 0x1p+0F },
    { -0x1p+0F, INF, -0x1p+0F },
    { -0x1p+0F, INF, 0x1p-126F },
    { -0x1p+0F, INF, -0x1p-126F },
    { -0x1p+0F, INF, 0x1.fffffep+127F },
    { -0x1p+0F, INF, -0x1.fffffep+127F },
    { -0x1p+0F, INF, INF },
    { -0x1p+0F, INF, -INF },
    { -0x1p+0F, INF, NAN },
    { -0x1p+0F, INF, NAN },
    { -0x1p+0F, INF, 0x1.ccccccp-1F },
    { -0x1p+0F, INF, 0x1.19999ap+0F },
    { -0x1p+0F, INF, -0x1.ccccccp-1F },
    { -0x1p+0F, INF, -0x1.19999ap+0F },
    { -0x1p+0F, -INF, 0x0p+0F },
    { -0x1p+0F, -INF, -0x0p+0F },
    { -0x1p+0F, -INF, 0x1p+0F },
    { -0x1p+0F, -INF, -0x1p+0F },
    { -0x1p+0F, -INF, 0x1p-126F },
    { -0x1p+0F, -INF, -0x1p-126F },
    { -0x1p+0F, -INF, 0x1.fffffep+127F },
    { -0x1p+0F, -INF, -0x1.fffffep+127F },
    { -0x1p+0F, -INF, INF },
    { -0x1p+0F, -INF, -INF },
    { -0x1p+0F, -INF, NAN },
    { -0x1p+0F, -INF, NAN },
    { -0x1p+0F, -INF, 0x1.ccccccp-1F },
    { -0x1p+0F, -INF, 0x1.19999ap+0F },
    { -0x1p+0F, -INF, -0x1.ccccccp-1F },
    { -0x1p+0F, -INF, -0x1.19999ap+0F },
    { -0x1p+0F, NAN, 0x0p+0F },
    { -0x1p+0F, NAN, -0x0p+0F },
    { -0x1p+0F, NAN, 0x1p+0F },
    { -0x1p+0F, NAN, -0x1p+0F },
    { -0x1p+0F, NAN, 0x1p-126F },
    { -0x1p+0F, NAN, -0x1p-126F },
    { -0x1p+0F, NAN, 0x1.fffffep+127F },
    { -0x1p+0F, NAN, -0x1.fffffep+127F },
    { -0x1p+0F, NAN, INF },
    { -0x1p+0F, NAN, -INF },
    { -0x1p+0F, NAN, NAN },
    { -0x1p+0F, NAN, NAN },
    { -0x1p+0F, NAN, 0x1.ccccccp-1F },
    { -0x1p+0F, NAN, 0x1.19999ap+0F },
    { -0x1p+0F, NAN, -0x1.ccccccp-1F },
    { -0x1p+0F, NAN, -0x1.19999ap+0F },
    { -0x1p+0F, NAN, 0x0p+0F },
    { -0x1p+0F, NAN, -0x0p+0F },
    { -0x1p+0F, NAN, 0x1p+0F },
    { -0x1p+0F, NAN, -0x1p+0F },
    { -0x1p+0F, NAN, 0x1p-126F },
    { -0x1p+0F, NAN, -0x1p-126F },
    { -0x1p+0F, NAN, 0x1.fffffep+127F },
    { -0x1p+0F, NAN, -0x1.fffffep+127F },
    { -0x1p+0F, NAN, INF },
    { -0x1p+0F, NAN, -INF },
    { -0x1p+0F, NAN, NAN },
    { -0x1p+0F, NAN, NAN },
    { -0x1p+0F, NAN, 0x1.ccccccp-1F },
    { -0x1p+0F, NAN, 0x1.19999ap+0F },
    { -0x1p+0F, NAN, -0x1.ccccccp-1F },
    { -0x1p+0F, NAN, -0x1.19999ap+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x1.ccccccp-1F, INF },
    { -0x1p+0F, 0x1.ccccccp-1F, -INF },
    { -0x1p+0F, 0x1.ccccccp-1F, NAN },
    { -0x1p+0F, 0x1.ccccccp-1F, NAN },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1p+0F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x1p+0F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x1p+0F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x1p+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1p+0F, 0x1.19999ap+0F, INF },
    { -0x1p+0F, 0x1.19999ap+0F, -INF },
    { -0x1p+0F, 0x1.19999ap+0F, NAN },
    { -0x1p+0F, 0x1.19999ap+0F, NAN },
    { -0x1p+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x1.ccccccp-1F, INF },
    { -0x1p+0F, -0x1.ccccccp-1F, -INF },
    { -0x1p+0F, -0x1.ccccccp-1F, NAN },
    { -0x1p+0F, -0x1.ccccccp-1F, NAN },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1p+0F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x1p+0F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x1p+0F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x1p+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1p+0F, -0x1.19999ap+0F, INF },
    { -0x1p+0F, -0x1.19999ap+0F, -INF },
    { -0x1p+0F, -0x1.19999ap+0F, NAN },
    { -0x1p+0F, -0x1.19999ap+0F, NAN },
    { -0x1p+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1p+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x0p+0F, 0x0p+0F },
    { 0x1p-126F, 0x0p+0F, -0x0p+0F },
    { 0x1p-126F, 0x0p+0F, 0x1p+0F },
    { 0x1p-126F, 0x0p+0F, -0x1p+0F },
    { 0x1p-126F, 0x0p+0F, 0x1p-126F },
    { 0x1p-126F, 0x0p+0F, -0x1p-126F },
    { 0x1p-126F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x0p+0F, INF },
    { 0x1p-126F, 0x0p+0F, -INF },
    { 0x1p-126F, 0x0p+0F, NAN },
    { 0x1p-126F, 0x0p+0F, NAN },
    { 0x1p-126F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x0p+0F, 0x0p+0F },
    { 0x1p-126F, -0x0p+0F, -0x0p+0F },
    { 0x1p-126F, -0x0p+0F, 0x1p+0F },
    { 0x1p-126F, -0x0p+0F, -0x1p+0F },
    { 0x1p-126F, -0x0p+0F, 0x1p-126F },
    { 0x1p-126F, -0x0p+0F, -0x1p-126F },
    { 0x1p-126F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x0p+0F, INF },
    { 0x1p-126F, -0x0p+0F, -INF },
    { 0x1p-126F, -0x0p+0F, NAN },
    { 0x1p-126F, -0x0p+0F, NAN },
    { 0x1p-126F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x1p+0F, 0x0p+0F },
    { 0x1p-126F, 0x1p+0F, -0x0p+0F },
    { 0x1p-126F, 0x1p+0F, 0x1p+0F },
    { 0x1p-126F, 0x1p+0F, -0x1p+0F },
    { 0x1p-126F, 0x1p+0F, 0x1p-126F },
    { 0x1p-126F, 0x1p+0F, -0x1p-126F },
    { 0x1p-126F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x1p+0F, INF },
    { 0x1p-126F, 0x1p+0F, -INF },
    { 0x1p-126F, 0x1p+0F, NAN },
    { 0x1p-126F, 0x1p+0F, NAN },
    { 0x1p-126F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x1p+0F, 0x0p+0F },
    { 0x1p-126F, -0x1p+0F, -0x0p+0F },
    { 0x1p-126F, -0x1p+0F, 0x1p+0F },
    { 0x1p-126F, -0x1p+0F, -0x1p+0F },
    { 0x1p-126F, -0x1p+0F, 0x1p-126F },
    { 0x1p-126F, -0x1p+0F, -0x1p-126F },
    { 0x1p-126F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x1p+0F, INF },
    { 0x1p-126F, -0x1p+0F, -INF },
    { 0x1p-126F, -0x1p+0F, NAN },
    { 0x1p-126F, -0x1p+0F, NAN },
    { 0x1p-126F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x1p-126F, 0x0p+0F },
    { 0x1p-126F, 0x1p-126F, -0x0p+0F },
    { 0x1p-126F, 0x1p-126F, 0x1p+0F },
    { 0x1p-126F, 0x1p-126F, -0x1p+0F },
    { 0x1p-126F, 0x1p-126F, 0x1p-126F },
    { 0x1p-126F, 0x1p-126F, -0x1p-126F },
    { 0x1p-126F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x1p-126F, INF },
    { 0x1p-126F, 0x1p-126F, -INF },
    { 0x1p-126F, 0x1p-126F, NAN },
    { 0x1p-126F, 0x1p-126F, NAN },
    { 0x1p-126F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x1p-126F, 0x0p+0F },
    { 0x1p-126F, -0x1p-126F, -0x0p+0F },
    { 0x1p-126F, -0x1p-126F, 0x1p+0F },
    { 0x1p-126F, -0x1p-126F, -0x1p+0F },
    { 0x1p-126F, -0x1p-126F, 0x1p-126F },
    { 0x1p-126F, -0x1p-126F, -0x1p-126F },
    { 0x1p-126F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x1p-126F, INF },
    { 0x1p-126F, -0x1p-126F, -INF },
    { 0x1p-126F, -0x1p-126F, NAN },
    { 0x1p-126F, -0x1p-126F, NAN },
    { 0x1p-126F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x1p-126F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x1p-126F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x1p-126F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x1.fffffep+127F, INF },
    { 0x1p-126F, 0x1.fffffep+127F, -INF },
    { 0x1p-126F, 0x1.fffffep+127F, NAN },
    { 0x1p-126F, 0x1.fffffep+127F, NAN },
    { 0x1p-126F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x1p-126F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x1p-126F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x1p-126F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x1.fffffep+127F, INF },
    { 0x1p-126F, -0x1.fffffep+127F, -INF },
    { 0x1p-126F, -0x1.fffffep+127F, NAN },
    { 0x1p-126F, -0x1.fffffep+127F, NAN },
    { 0x1p-126F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1p-126F, INF, 0x0p+0F },
    { 0x1p-126F, INF, -0x0p+0F },
    { 0x1p-126F, INF, 0x1p+0F },
    { 0x1p-126F, INF, -0x1p+0F },
    { 0x1p-126F, INF, 0x1p-126F },
    { 0x1p-126F, INF, -0x1p-126F },
    { 0x1p-126F, INF, 0x1.fffffep+127F },
    { 0x1p-126F, INF, -0x1.fffffep+127F },
    { 0x1p-126F, INF, INF },
    { 0x1p-126F, INF, -INF },
    { 0x1p-126F, INF, NAN },
    { 0x1p-126F, INF, NAN },
    { 0x1p-126F, INF, 0x1.ccccccp-1F },
    { 0x1p-126F, INF, 0x1.19999ap+0F },
    { 0x1p-126F, INF, -0x1.ccccccp-1F },
    { 0x1p-126F, INF, -0x1.19999ap+0F },
    { 0x1p-126F, -INF, 0x0p+0F },
    { 0x1p-126F, -INF, -0x0p+0F },
    { 0x1p-126F, -INF, 0x1p+0F },
    { 0x1p-126F, -INF, -0x1p+0F },
    { 0x1p-126F, -INF, 0x1p-126F },
    { 0x1p-126F, -INF, -0x1p-126F },
    { 0x1p-126F, -INF, 0x1.fffffep+127F },
    { 0x1p-126F, -INF, -0x1.fffffep+127F },
    { 0x1p-126F, -INF, INF },
    { 0x1p-126F, -INF, -INF },
    { 0x1p-126F, -INF, NAN },
    { 0x1p-126F, -INF, NAN },
    { 0x1p-126F, -INF, 0x1.ccccccp-1F },
    { 0x1p-126F, -INF, 0x1.19999ap+0F },
    { 0x1p-126F, -INF, -0x1.ccccccp-1F },
    { 0x1p-126F, -INF, -0x1.19999ap+0F },
    { 0x1p-126F, NAN, 0x0p+0F },
    { 0x1p-126F, NAN, -0x0p+0F },
    { 0x1p-126F, NAN, 0x1p+0F },
    { 0x1p-126F, NAN, -0x1p+0F },
    { 0x1p-126F, NAN, 0x1p-126F },
    { 0x1p-126F, NAN, -0x1p-126F },
    { 0x1p-126F, NAN, 0x1.fffffep+127F },
    { 0x1p-126F, NAN, -0x1.fffffep+127F },
    { 0x1p-126F, NAN, INF },
    { 0x1p-126F, NAN, -INF },
    { 0x1p-126F, NAN, NAN },
    { 0x1p-126F, NAN, NAN },
    { 0x1p-126F, NAN, 0x1.ccccccp-1F },
    { 0x1p-126F, NAN, 0x1.19999ap+0F },
    { 0x1p-126F, NAN, -0x1.ccccccp-1F },
    { 0x1p-126F, NAN, -0x1.19999ap+0F },
    { 0x1p-126F, NAN, 0x0p+0F },
    { 0x1p-126F, NAN, -0x0p+0F },
    { 0x1p-126F, NAN, 0x1p+0F },
    { 0x1p-126F, NAN, -0x1p+0F },
    { 0x1p-126F, NAN, 0x1p-126F },
    { 0x1p-126F, NAN, -0x1p-126F },
    { 0x1p-126F, NAN, 0x1.fffffep+127F },
    { 0x1p-126F, NAN, -0x1.fffffep+127F },
    { 0x1p-126F, NAN, INF },
    { 0x1p-126F, NAN, -INF },
    { 0x1p-126F, NAN, NAN },
    { 0x1p-126F, NAN, NAN },
    { 0x1p-126F, NAN, 0x1.ccccccp-1F },
    { 0x1p-126F, NAN, 0x1.19999ap+0F },
    { 0x1p-126F, NAN, -0x1.ccccccp-1F },
    { 0x1p-126F, NAN, -0x1.19999ap+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x1.ccccccp-1F, INF },
    { 0x1p-126F, 0x1.ccccccp-1F, -INF },
    { 0x1p-126F, 0x1.ccccccp-1F, NAN },
    { 0x1p-126F, 0x1.ccccccp-1F, NAN },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1p-126F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x1p-126F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x1p-126F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x1p-126F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1p-126F, 0x1.19999ap+0F, INF },
    { 0x1p-126F, 0x1.19999ap+0F, -INF },
    { 0x1p-126F, 0x1.19999ap+0F, NAN },
    { 0x1p-126F, 0x1.19999ap+0F, NAN },
    { 0x1p-126F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x1.ccccccp-1F, INF },
    { 0x1p-126F, -0x1.ccccccp-1F, -INF },
    { 0x1p-126F, -0x1.ccccccp-1F, NAN },
    { 0x1p-126F, -0x1.ccccccp-1F, NAN },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1p-126F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x1p-126F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x1p-126F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x1p-126F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1p-126F, -0x1.19999ap+0F, INF },
    { 0x1p-126F, -0x1.19999ap+0F, -INF },
    { 0x1p-126F, -0x1.19999ap+0F, NAN },
    { 0x1p-126F, -0x1.19999ap+0F, NAN },
    { 0x1p-126F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1p-126F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x0p+0F, 0x0p+0F },
    { -0x1p-126F, 0x0p+0F, -0x0p+0F },
    { -0x1p-126F, 0x0p+0F, 0x1p+0F },
    { -0x1p-126F, 0x0p+0F, -0x1p+0F },
    { -0x1p-126F, 0x0p+0F, 0x1p-126F },
    { -0x1p-126F, 0x0p+0F, -0x1p-126F },
    { -0x1p-126F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x0p+0F, INF },
    { -0x1p-126F, 0x0p+0F, -INF },
    { -0x1p-126F, 0x0p+0F, NAN },
    { -0x1p-126F, 0x0p+0F, NAN },
    { -0x1p-126F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x0p+0F, 0x0p+0F },
    { -0x1p-126F, -0x0p+0F, -0x0p+0F },
    { -0x1p-126F, -0x0p+0F, 0x1p+0F },
    { -0x1p-126F, -0x0p+0F, -0x1p+0F },
    { -0x1p-126F, -0x0p+0F, 0x1p-126F },
    { -0x1p-126F, -0x0p+0F, -0x1p-126F },
    { -0x1p-126F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x0p+0F, INF },
    { -0x1p-126F, -0x0p+0F, -INF },
    { -0x1p-126F, -0x0p+0F, NAN },
    { -0x1p-126F, -0x0p+0F, NAN },
    { -0x1p-126F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x1p+0F, 0x0p+0F },
    { -0x1p-126F, 0x1p+0F, -0x0p+0F },
    { -0x1p-126F, 0x1p+0F, 0x1p+0F },
    { -0x1p-126F, 0x1p+0F, -0x1p+0F },
    { -0x1p-126F, 0x1p+0F, 0x1p-126F },
    { -0x1p-126F, 0x1p+0F, -0x1p-126F },
    { -0x1p-126F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x1p+0F, INF },
    { -0x1p-126F, 0x1p+0F, -INF },
    { -0x1p-126F, 0x1p+0F, NAN },
    { -0x1p-126F, 0x1p+0F, NAN },
    { -0x1p-126F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x1p+0F, 0x0p+0F },
    { -0x1p-126F, -0x1p+0F, -0x0p+0F },
    { -0x1p-126F, -0x1p+0F, 0x1p+0F },
    { -0x1p-126F, -0x1p+0F, -0x1p+0F },
    { -0x1p-126F, -0x1p+0F, 0x1p-126F },
    { -0x1p-126F, -0x1p+0F, -0x1p-126F },
    { -0x1p-126F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x1p+0F, INF },
    { -0x1p-126F, -0x1p+0F, -INF },
    { -0x1p-126F, -0x1p+0F, NAN },
    { -0x1p-126F, -0x1p+0F, NAN },
    { -0x1p-126F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x1p-126F, 0x0p+0F },
    { -0x1p-126F, 0x1p-126F, -0x0p+0F },
    { -0x1p-126F, 0x1p-126F, 0x1p+0F },
    { -0x1p-126F, 0x1p-126F, -0x1p+0F },
    { -0x1p-126F, 0x1p-126F, 0x1p-126F },
    { -0x1p-126F, 0x1p-126F, -0x1p-126F },
    { -0x1p-126F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x1p-126F, INF },
    { -0x1p-126F, 0x1p-126F, -INF },
    { -0x1p-126F, 0x1p-126F, NAN },
    { -0x1p-126F, 0x1p-126F, NAN },
    { -0x1p-126F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x1p-126F, 0x0p+0F },
    { -0x1p-126F, -0x1p-126F, -0x0p+0F },
    { -0x1p-126F, -0x1p-126F, 0x1p+0F },
    { -0x1p-126F, -0x1p-126F, -0x1p+0F },
    { -0x1p-126F, -0x1p-126F, 0x1p-126F },
    { -0x1p-126F, -0x1p-126F, -0x1p-126F },
    { -0x1p-126F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x1p-126F, INF },
    { -0x1p-126F, -0x1p-126F, -INF },
    { -0x1p-126F, -0x1p-126F, NAN },
    { -0x1p-126F, -0x1p-126F, NAN },
    { -0x1p-126F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x1p-126F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x1p-126F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x1p-126F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x1.fffffep+127F, INF },
    { -0x1p-126F, 0x1.fffffep+127F, -INF },
    { -0x1p-126F, 0x1.fffffep+127F, NAN },
    { -0x1p-126F, 0x1.fffffep+127F, NAN },
    { -0x1p-126F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x1p-126F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x1p-126F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x1p-126F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x1.fffffep+127F, INF },
    { -0x1p-126F, -0x1.fffffep+127F, -INF },
    { -0x1p-126F, -0x1.fffffep+127F, NAN },
    { -0x1p-126F, -0x1.fffffep+127F, NAN },
    { -0x1p-126F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1p-126F, INF, 0x0p+0F },
    { -0x1p-126F, INF, -0x0p+0F },
    { -0x1p-126F, INF, 0x1p+0F },
    { -0x1p-126F, INF, -0x1p+0F },
    { -0x1p-126F, INF, 0x1p-126F },
    { -0x1p-126F, INF, -0x1p-126F },
    { -0x1p-126F, INF, 0x1.fffffep+127F },
    { -0x1p-126F, INF, -0x1.fffffep+127F },
    { -0x1p-126F, INF, INF },
    { -0x1p-126F, INF, -INF },
    { -0x1p-126F, INF, NAN },
    { -0x1p-126F, INF, NAN },
    { -0x1p-126F, INF, 0x1.ccccccp-1F },
    { -0x1p-126F, INF, 0x1.19999ap+0F },
    { -0x1p-126F, INF, -0x1.ccccccp-1F },
    { -0x1p-126F, INF, -0x1.19999ap+0F },
    { -0x1p-126F, -INF, 0x0p+0F },
    { -0x1p-126F, -INF, -0x0p+0F },
    { -0x1p-126F, -INF, 0x1p+0F },
    { -0x1p-126F, -INF, -0x1p+0F },
    { -0x1p-126F, -INF, 0x1p-126F },
    { -0x1p-126F, -INF, -0x1p-126F },
    { -0x1p-126F, -INF, 0x1.fffffep+127F },
    { -0x1p-126F, -INF, -0x1.fffffep+127F },
    { -0x1p-126F, -INF, INF },
    { -0x1p-126F, -INF, -INF },
    { -0x1p-126F, -INF, NAN },
    { -0x1p-126F, -INF, NAN },
    { -0x1p-126F, -INF, 0x1.ccccccp-1F },
    { -0x1p-126F, -INF, 0x1.19999ap+0F },
    { -0x1p-126F, -INF, -0x1.ccccccp-1F },
    { -0x1p-126F, -INF, -0x1.19999ap+0F },
    { -0x1p-126F, NAN, 0x0p+0F },
    { -0x1p-126F, NAN, -0x0p+0F },
    { -0x1p-126F, NAN, 0x1p+0F },
    { -0x1p-126F, NAN, -0x1p+0F },
    { -0x1p-126F, NAN, 0x1p-126F },
    { -0x1p-126F, NAN, -0x1p-126F },
    { -0x1p-126F, NAN, 0x1.fffffep+127F },
    { -0x1p-126F, NAN, -0x1.fffffep+127F },
    { -0x1p-126F, NAN, INF },
    { -0x1p-126F, NAN, -INF },
    { -0x1p-126F, NAN, NAN },
    { -0x1p-126F, NAN, NAN },
    { -0x1p-126F, NAN, 0x1.ccccccp-1F },
    { -0x1p-126F, NAN, 0x1.19999ap+0F },
    { -0x1p-126F, NAN, -0x1.ccccccp-1F },
    { -0x1p-126F, NAN, -0x1.19999ap+0F },
    { -0x1p-126F, NAN, 0x0p+0F },
    { -0x1p-126F, NAN, -0x0p+0F },
    { -0x1p-126F, NAN, 0x1p+0F },
    { -0x1p-126F, NAN, -0x1p+0F },
    { -0x1p-126F, NAN, 0x1p-126F },
    { -0x1p-126F, NAN, -0x1p-126F },
    { -0x1p-126F, NAN, 0x1.fffffep+127F },
    { -0x1p-126F, NAN, -0x1.fffffep+127F },
    { -0x1p-126F, NAN, INF },
    { -0x1p-126F, NAN, -INF },
    { -0x1p-126F, NAN, NAN },
    { -0x1p-126F, NAN, NAN },
    { -0x1p-126F, NAN, 0x1.ccccccp-1F },
    { -0x1p-126F, NAN, 0x1.19999ap+0F },
    { -0x1p-126F, NAN, -0x1.ccccccp-1F },
    { -0x1p-126F, NAN, -0x1.19999ap+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x1.ccccccp-1F, INF },
    { -0x1p-126F, 0x1.ccccccp-1F, -INF },
    { -0x1p-126F, 0x1.ccccccp-1F, NAN },
    { -0x1p-126F, 0x1.ccccccp-1F, NAN },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1p-126F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x1p-126F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x1p-126F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x1p-126F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1p-126F, 0x1.19999ap+0F, INF },
    { -0x1p-126F, 0x1.19999ap+0F, -INF },
    { -0x1p-126F, 0x1.19999ap+0F, NAN },
    { -0x1p-126F, 0x1.19999ap+0F, NAN },
    { -0x1p-126F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x1.ccccccp-1F, INF },
    { -0x1p-126F, -0x1.ccccccp-1F, -INF },
    { -0x1p-126F, -0x1.ccccccp-1F, NAN },
    { -0x1p-126F, -0x1.ccccccp-1F, NAN },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1p-126F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x1p-126F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x1p-126F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x1p-126F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1p-126F, -0x1.19999ap+0F, INF },
    { -0x1p-126F, -0x1.19999ap+0F, -INF },
    { -0x1p-126F, -0x1.19999ap+0F, NAN },
    { -0x1p-126F, -0x1.19999ap+0F, NAN },
    { -0x1p-126F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1p-126F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x0p+0F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x0p+0F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x0p+0F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x0p+0F, INF },
    { 0x1.fffffep+127F, 0x0p+0F, -INF },
    { 0x1.fffffep+127F, 0x0p+0F, NAN },
    { 0x1.fffffep+127F, 0x0p+0F, NAN },
    { 0x1.fffffep+127F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x0p+0F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x0p+0F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x0p+0F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x0p+0F, INF },
    { 0x1.fffffep+127F, -0x0p+0F, -INF },
    { 0x1.fffffep+127F, -0x0p+0F, NAN },
    { 0x1.fffffep+127F, -0x0p+0F, NAN },
    { 0x1.fffffep+127F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1p+0F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1p+0F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1p+0F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1p+0F, INF },
    { 0x1.fffffep+127F, 0x1p+0F, -INF },
    { 0x1.fffffep+127F, 0x1p+0F, NAN },
    { 0x1.fffffep+127F, 0x1p+0F, NAN },
    { 0x1.fffffep+127F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1p+0F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x1p+0F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x1p+0F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1p+0F, INF },
    { 0x1.fffffep+127F, -0x1p+0F, -INF },
    { 0x1.fffffep+127F, -0x1p+0F, NAN },
    { 0x1.fffffep+127F, -0x1p+0F, NAN },
    { 0x1.fffffep+127F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1p-126F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1p-126F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1p-126F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1p-126F, INF },
    { 0x1.fffffep+127F, 0x1p-126F, -INF },
    { 0x1.fffffep+127F, 0x1p-126F, NAN },
    { 0x1.fffffep+127F, 0x1p-126F, NAN },
    { 0x1.fffffep+127F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1p-126F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x1p-126F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x1p-126F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1p-126F, INF },
    { 0x1.fffffep+127F, -0x1p-126F, -INF },
    { 0x1.fffffep+127F, -0x1p-126F, NAN },
    { 0x1.fffffep+127F, -0x1p-126F, NAN },
    { 0x1.fffffep+127F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, INF },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -INF },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, INF },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -INF },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, NAN },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, INF, 0x0p+0F },
    { 0x1.fffffep+127F, INF, -0x0p+0F },
    { 0x1.fffffep+127F, INF, 0x1p+0F },
    { 0x1.fffffep+127F, INF, -0x1p+0F },
    { 0x1.fffffep+127F, INF, 0x1p-126F },
    { 0x1.fffffep+127F, INF, -0x1p-126F },
    { 0x1.fffffep+127F, INF, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, INF, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, INF, INF },
    { 0x1.fffffep+127F, INF, -INF },
    { 0x1.fffffep+127F, INF, NAN },
    { 0x1.fffffep+127F, INF, NAN },
    { 0x1.fffffep+127F, INF, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, INF, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, INF, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, INF, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -INF, 0x0p+0F },
    { 0x1.fffffep+127F, -INF, -0x0p+0F },
    { 0x1.fffffep+127F, -INF, 0x1p+0F },
    { 0x1.fffffep+127F, -INF, -0x1p+0F },
    { 0x1.fffffep+127F, -INF, 0x1p-126F },
    { 0x1.fffffep+127F, -INF, -0x1p-126F },
    { 0x1.fffffep+127F, -INF, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -INF, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -INF, INF },
    { 0x1.fffffep+127F, -INF, -INF },
    { 0x1.fffffep+127F, -INF, NAN },
    { 0x1.fffffep+127F, -INF, NAN },
    { 0x1.fffffep+127F, -INF, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -INF, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -INF, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -INF, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, NAN, 0x0p+0F },
    { 0x1.fffffep+127F, NAN, -0x0p+0F },
    { 0x1.fffffep+127F, NAN, 0x1p+0F },
    { 0x1.fffffep+127F, NAN, -0x1p+0F },
    { 0x1.fffffep+127F, NAN, 0x1p-126F },
    { 0x1.fffffep+127F, NAN, -0x1p-126F },
    { 0x1.fffffep+127F, NAN, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, NAN, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, NAN, INF },
    { 0x1.fffffep+127F, NAN, -INF },
    { 0x1.fffffep+127F, NAN, NAN },
    { 0x1.fffffep+127F, NAN, NAN },
    { 0x1.fffffep+127F, NAN, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, NAN, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, NAN, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, NAN, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, NAN, 0x0p+0F },
    { 0x1.fffffep+127F, NAN, -0x0p+0F },
    { 0x1.fffffep+127F, NAN, 0x1p+0F },
    { 0x1.fffffep+127F, NAN, -0x1p+0F },
    { 0x1.fffffep+127F, NAN, 0x1p-126F },
    { 0x1.fffffep+127F, NAN, -0x1p-126F },
    { 0x1.fffffep+127F, NAN, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, NAN, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, NAN, INF },
    { 0x1.fffffep+127F, NAN, -INF },
    { 0x1.fffffep+127F, NAN, NAN },
    { 0x1.fffffep+127F, NAN, NAN },
    { 0x1.fffffep+127F, NAN, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, NAN, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, NAN, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, NAN, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, INF },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -INF },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, NAN },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, NAN },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, INF },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -INF },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, NAN },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, NAN },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, INF },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -INF },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, NAN },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, NAN },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, INF },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -INF },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, NAN },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, NAN },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x0p+0F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x0p+0F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x0p+0F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x0p+0F, INF },
    { -0x1.fffffep+127F, 0x0p+0F, -INF },
    { -0x1.fffffep+127F, 0x0p+0F, NAN },
    { -0x1.fffffep+127F, 0x0p+0F, NAN },
    { -0x1.fffffep+127F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x0p+0F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x0p+0F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x0p+0F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x0p+0F, INF },
    { -0x1.fffffep+127F, -0x0p+0F, -INF },
    { -0x1.fffffep+127F, -0x0p+0F, NAN },
    { -0x1.fffffep+127F, -0x0p+0F, NAN },
    { -0x1.fffffep+127F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1p+0F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1p+0F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1p+0F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1p+0F, INF },
    { -0x1.fffffep+127F, 0x1p+0F, -INF },
    { -0x1.fffffep+127F, 0x1p+0F, NAN },
    { -0x1.fffffep+127F, 0x1p+0F, NAN },
    { -0x1.fffffep+127F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1p+0F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x1p+0F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x1p+0F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1p+0F, INF },
    { -0x1.fffffep+127F, -0x1p+0F, -INF },
    { -0x1.fffffep+127F, -0x1p+0F, NAN },
    { -0x1.fffffep+127F, -0x1p+0F, NAN },
    { -0x1.fffffep+127F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1p-126F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1p-126F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1p-126F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1p-126F, INF },
    { -0x1.fffffep+127F, 0x1p-126F, -INF },
    { -0x1.fffffep+127F, 0x1p-126F, NAN },
    { -0x1.fffffep+127F, 0x1p-126F, NAN },
    { -0x1.fffffep+127F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1p-126F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x1p-126F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x1p-126F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1p-126F, INF },
    { -0x1.fffffep+127F, -0x1p-126F, -INF },
    { -0x1.fffffep+127F, -0x1p-126F, NAN },
    { -0x1.fffffep+127F, -0x1p-126F, NAN },
    { -0x1.fffffep+127F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, INF },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -INF },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, INF },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -INF },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, NAN },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, INF, 0x0p+0F },
    { -0x1.fffffep+127F, INF, -0x0p+0F },
    { -0x1.fffffep+127F, INF, 0x1p+0F },
    { -0x1.fffffep+127F, INF, -0x1p+0F },
    { -0x1.fffffep+127F, INF, 0x1p-126F },
    { -0x1.fffffep+127F, INF, -0x1p-126F },
    { -0x1.fffffep+127F, INF, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, INF, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, INF, INF },
    { -0x1.fffffep+127F, INF, -INF },
    { -0x1.fffffep+127F, INF, NAN },
    { -0x1.fffffep+127F, INF, NAN },
    { -0x1.fffffep+127F, INF, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, INF, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, INF, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, INF, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -INF, 0x0p+0F },
    { -0x1.fffffep+127F, -INF, -0x0p+0F },
    { -0x1.fffffep+127F, -INF, 0x1p+0F },
    { -0x1.fffffep+127F, -INF, -0x1p+0F },
    { -0x1.fffffep+127F, -INF, 0x1p-126F },
    { -0x1.fffffep+127F, -INF, -0x1p-126F },
    { -0x1.fffffep+127F, -INF, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -INF, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -INF, INF },
    { -0x1.fffffep+127F, -INF, -INF },
    { -0x1.fffffep+127F, -INF, NAN },
    { -0x1.fffffep+127F, -INF, NAN },
    { -0x1.fffffep+127F, -INF, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -INF, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -INF, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -INF, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, NAN, 0x0p+0F },
    { -0x1.fffffep+127F, NAN, -0x0p+0F },
    { -0x1.fffffep+127F, NAN, 0x1p+0F },
    { -0x1.fffffep+127F, NAN, -0x1p+0F },
    { -0x1.fffffep+127F, NAN, 0x1p-126F },
    { -0x1.fffffep+127F, NAN, -0x1p-126F },
    { -0x1.fffffep+127F, NAN, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, NAN, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, NAN, INF },
    { -0x1.fffffep+127F, NAN, -INF },
    { -0x1.fffffep+127F, NAN, NAN },
    { -0x1.fffffep+127F, NAN, NAN },
    { -0x1.fffffep+127F, NAN, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, NAN, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, NAN, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, NAN, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, NAN, 0x0p+0F },
    { -0x1.fffffep+127F, NAN, -0x0p+0F },
    { -0x1.fffffep+127F, NAN, 0x1p+0F },
    { -0x1.fffffep+127F, NAN, -0x1p+0F },
    { -0x1.fffffep+127F, NAN, 0x1p-126F },
    { -0x1.fffffep+127F, NAN, -0x1p-126F },
    { -0x1.fffffep+127F, NAN, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, NAN, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, NAN, INF },
    { -0x1.fffffep+127F, NAN, -INF },
    { -0x1.fffffep+127F, NAN, NAN },
    { -0x1.fffffep+127F, NAN, NAN },
    { -0x1.fffffep+127F, NAN, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, NAN, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, NAN, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, NAN, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, INF },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -INF },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, NAN },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, NAN },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, INF },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -INF },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, NAN },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, NAN },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, INF },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -INF },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, NAN },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, NAN },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, INF },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -INF },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, NAN },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, NAN },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.fffffep+127F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { INF, 0x0p+0F, 0x0p+0F },
    { INF, 0x0p+0F, -0x0p+0F },
    { INF, 0x0p+0F, 0x1p+0F },
    { INF, 0x0p+0F, -0x1p+0F },
    { INF, 0x0p+0F, 0x1p-126F },
    { INF, 0x0p+0F, -0x1p-126F },
    { INF, 0x0p+0F, 0x1.fffffep+127F },
    { INF, 0x0p+0F, -0x1.fffffep+127F },
    { INF, 0x0p+0F, INF },
    { INF, 0x0p+0F, -INF },
    { INF, 0x0p+0F, NAN },
    { INF, 0x0p+0F, NAN },
    { INF, 0x0p+0F, 0x1.ccccccp-1F },
    { INF, 0x0p+0F, 0x1.19999ap+0F },
    { INF, 0x0p+0F, -0x1.ccccccp-1F },
    { INF, 0x0p+0F, -0x1.19999ap+0F },
    { INF, -0x0p+0F, 0x0p+0F },
    { INF, -0x0p+0F, -0x0p+0F },
    { INF, -0x0p+0F, 0x1p+0F },
    { INF, -0x0p+0F, -0x1p+0F },
    { INF, -0x0p+0F, 0x1p-126F },
    { INF, -0x0p+0F, -0x1p-126F },
    { INF, -0x0p+0F, 0x1.fffffep+127F },
    { INF, -0x0p+0F, -0x1.fffffep+127F },
    { INF, -0x0p+0F, INF },
    { INF, -0x0p+0F, -INF },
    { INF, -0x0p+0F, NAN },
    { INF, -0x0p+0F, NAN },
    { INF, -0x0p+0F, 0x1.ccccccp-1F },
    { INF, -0x0p+0F, 0x1.19999ap+0F },
    { INF, -0x0p+0F, -0x1.ccccccp-1F },
    { INF, -0x0p+0F, -0x1.19999ap+0F },
    { INF, 0x1p+0F, 0x0p+0F },
    { INF, 0x1p+0F, -0x0p+0F },
    { INF, 0x1p+0F, 0x1p+0F },
    { INF, 0x1p+0F, -0x1p+0F },
    { INF, 0x1p+0F, 0x1p-126F },
    { INF, 0x1p+0F, -0x1p-126F },
    { INF, 0x1p+0F, 0x1.fffffep+127F },
    { INF, 0x1p+0F, -0x1.fffffep+127F },
    { INF, 0x1p+0F, INF },
    { INF, 0x1p+0F, -INF },
    { INF, 0x1p+0F, NAN },
    { INF, 0x1p+0F, NAN },
    { INF, 0x1p+0F, 0x1.ccccccp-1F },
    { INF, 0x1p+0F, 0x1.19999ap+0F },
    { INF, 0x1p+0F, -0x1.ccccccp-1F },
    { INF, 0x1p+0F, -0x1.19999ap+0F },
    { INF, -0x1p+0F, 0x0p+0F },
    { INF, -0x1p+0F, -0x0p+0F },
    { INF, -0x1p+0F, 0x1p+0F },
    { INF, -0x1p+0F, -0x1p+0F },
    { INF, -0x1p+0F, 0x1p-126F },
    { INF, -0x1p+0F, -0x1p-126F },
    { INF, -0x1p+0F, 0x1.fffffep+127F },
    { INF, -0x1p+0F, -0x1.fffffep+127F },
    { INF, -0x1p+0F, INF },
    { INF, -0x1p+0F, -INF },
    { INF, -0x1p+0F, NAN },
    { INF, -0x1p+0F, NAN },
    { INF, -0x1p+0F, 0x1.ccccccp-1F },
    { INF, -0x1p+0F, 0x1.19999ap+0F },
    { INF, -0x1p+0F, -0x1.ccccccp-1F },
    { INF, -0x1p+0F, -0x1.19999ap+0F },
    { INF, 0x1p-126F, 0x0p+0F },
    { INF, 0x1p-126F, -0x0p+0F },
    { INF, 0x1p-126F, 0x1p+0F },
    { INF, 0x1p-126F, -0x1p+0F },
    { INF, 0x1p-126F, 0x1p-126F },
    { INF, 0x1p-126F, -0x1p-126F },
    { INF, 0x1p-126F, 0x1.fffffep+127F },
    { INF, 0x1p-126F, -0x1.fffffep+127F },
    { INF, 0x1p-126F, INF },
    { INF, 0x1p-126F, -INF },
    { INF, 0x1p-126F, NAN },
    { INF, 0x1p-126F, NAN },
    { INF, 0x1p-126F, 0x1.ccccccp-1F },
    { INF, 0x1p-126F, 0x1.19999ap+0F },
    { INF, 0x1p-126F, -0x1.ccccccp-1F },
    { INF, 0x1p-126F, -0x1.19999ap+0F },
    { INF, -0x1p-126F, 0x0p+0F },
    { INF, -0x1p-126F, -0x0p+0F },
    { INF, -0x1p-126F, 0x1p+0F },
    { INF, -0x1p-126F, -0x1p+0F },
    { INF, -0x1p-126F, 0x1p-126F },
    { INF, -0x1p-126F, -0x1p-126F },
    { INF, -0x1p-126F, 0x1.fffffep+127F },
    { INF, -0x1p-126F, -0x1.fffffep+127F },
    { INF, -0x1p-126F, INF },
    { INF, -0x1p-126F, -INF },
    { INF, -0x1p-126F, NAN },
    { INF, -0x1p-126F, NAN },
    { INF, -0x1p-126F, 0x1.ccccccp-1F },
    { INF, -0x1p-126F, 0x1.19999ap+0F },
    { INF, -0x1p-126F, -0x1.ccccccp-1F },
    { INF, -0x1p-126F, -0x1.19999ap+0F },
    { INF, 0x1.fffffep+127F, 0x0p+0F },
    { INF, 0x1.fffffep+127F, -0x0p+0F },
    { INF, 0x1.fffffep+127F, 0x1p+0F },
    { INF, 0x1.fffffep+127F, -0x1p+0F },
    { INF, 0x1.fffffep+127F, 0x1p-126F },
    { INF, 0x1.fffffep+127F, -0x1p-126F },
    { INF, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { INF, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { INF, 0x1.fffffep+127F, INF },
    { INF, 0x1.fffffep+127F, -INF },
    { INF, 0x1.fffffep+127F, NAN },
    { INF, 0x1.fffffep+127F, NAN },
    { INF, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { INF, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { INF, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { INF, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { INF, -0x1.fffffep+127F, 0x0p+0F },
    { INF, -0x1.fffffep+127F, -0x0p+0F },
    { INF, -0x1.fffffep+127F, 0x1p+0F },
    { INF, -0x1.fffffep+127F, -0x1p+0F },
    { INF, -0x1.fffffep+127F, 0x1p-126F },
    { INF, -0x1.fffffep+127F, -0x1p-126F },
    { INF, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { INF, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { INF, -0x1.fffffep+127F, INF },
    { INF, -0x1.fffffep+127F, -INF },
    { INF, -0x1.fffffep+127F, NAN },
    { INF, -0x1.fffffep+127F, NAN },
    { INF, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { INF, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { INF, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { INF, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { INF, INF, 0x0p+0F },
    { INF, INF, -0x0p+0F },
    { INF, INF, 0x1p+0F },
    { INF, INF, -0x1p+0F },
    { INF, INF, 0x1p-126F },
    { INF, INF, -0x1p-126F },
    { INF, INF, 0x1.fffffep+127F },
    { INF, INF, -0x1.fffffep+127F },
    { INF, INF, INF },
    { INF, INF, -INF },
    { INF, INF, NAN },
    { INF, INF, NAN },
    { INF, INF, 0x1.ccccccp-1F },
    { INF, INF, 0x1.19999ap+0F },
    { INF, INF, -0x1.ccccccp-1F },
    { INF, INF, -0x1.19999ap+0F },
    { INF, -INF, 0x0p+0F },
    { INF, -INF, -0x0p+0F },
    { INF, -INF, 0x1p+0F },
    { INF, -INF, -0x1p+0F },
    { INF, -INF, 0x1p-126F },
    { INF, -INF, -0x1p-126F },
    { INF, -INF, 0x1.fffffep+127F },
    { INF, -INF, -0x1.fffffep+127F },
    { INF, -INF, INF },
    { INF, -INF, -INF },
    { INF, -INF, NAN },
    { INF, -INF, NAN },
    { INF, -INF, 0x1.ccccccp-1F },
    { INF, -INF, 0x1.19999ap+0F },
    { INF, -INF, -0x1.ccccccp-1F },
    { INF, -INF, -0x1.19999ap+0F },
    { INF, NAN, 0x0p+0F },
    { INF, NAN, -0x0p+0F },
    { INF, NAN, 0x1p+0F },
    { INF, NAN, -0x1p+0F },
    { INF, NAN, 0x1p-126F },
    { INF, NAN, -0x1p-126F },
    { INF, NAN, 0x1.fffffep+127F },
    { INF, NAN, -0x1.fffffep+127F },
    { INF, NAN, INF },
    { INF, NAN, -INF },
    { INF, NAN, NAN },
    { INF, NAN, NAN },
    { INF, NAN, 0x1.ccccccp-1F },
    { INF, NAN, 0x1.19999ap+0F },
    { INF, NAN, -0x1.ccccccp-1F },
    { INF, NAN, -0x1.19999ap+0F },
    { INF, NAN, 0x0p+0F },
    { INF, NAN, -0x0p+0F },
    { INF, NAN, 0x1p+0F },
    { INF, NAN, -0x1p+0F },
    { INF, NAN, 0x1p-126F },
    { INF, NAN, -0x1p-126F },
    { INF, NAN, 0x1.fffffep+127F },
    { INF, NAN, -0x1.fffffep+127F },
    { INF, NAN, INF },
    { INF, NAN, -INF },
    { INF, NAN, NAN },
    { INF, NAN, NAN },
    { INF, NAN, 0x1.ccccccp-1F },
    { INF, NAN, 0x1.19999ap+0F },
    { INF, NAN, -0x1.ccccccp-1F },
    { INF, NAN, -0x1.19999ap+0F },
    { INF, 0x1.ccccccp-1F, 0x0p+0F },
    { INF, 0x1.ccccccp-1F, -0x0p+0F },
    { INF, 0x1.ccccccp-1F, 0x1p+0F },
    { INF, 0x1.ccccccp-1F, -0x1p+0F },
    { INF, 0x1.ccccccp-1F, 0x1p-126F },
    { INF, 0x1.ccccccp-1F, -0x1p-126F },
    { INF, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { INF, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { INF, 0x1.ccccccp-1F, INF },
    { INF, 0x1.ccccccp-1F, -INF },
    { INF, 0x1.ccccccp-1F, NAN },
    { INF, 0x1.ccccccp-1F, NAN },
    { INF, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { INF, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { INF, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { INF, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { INF, 0x1.19999ap+0F, 0x0p+0F },
    { INF, 0x1.19999ap+0F, -0x0p+0F },
    { INF, 0x1.19999ap+0F, 0x1p+0F },
    { INF, 0x1.19999ap+0F, -0x1p+0F },
    { INF, 0x1.19999ap+0F, 0x1p-126F },
    { INF, 0x1.19999ap+0F, -0x1p-126F },
    { INF, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { INF, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { INF, 0x1.19999ap+0F, INF },
    { INF, 0x1.19999ap+0F, -INF },
    { INF, 0x1.19999ap+0F, NAN },
    { INF, 0x1.19999ap+0F, NAN },
    { INF, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { INF, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { INF, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { INF, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { INF, -0x1.ccccccp-1F, 0x0p+0F },
    { INF, -0x1.ccccccp-1F, -0x0p+0F },
    { INF, -0x1.ccccccp-1F, 0x1p+0F },
    { INF, -0x1.ccccccp-1F, -0x1p+0F },
    { INF, -0x1.ccccccp-1F, 0x1p-126F },
    { INF, -0x1.ccccccp-1F, -0x1p-126F },
    { INF, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { INF, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { INF, -0x1.ccccccp-1F, INF },
    { INF, -0x1.ccccccp-1F, -INF },
    { INF, -0x1.ccccccp-1F, NAN },
    { INF, -0x1.ccccccp-1F, NAN },
    { INF, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { INF, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { INF, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { INF, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { INF, -0x1.19999ap+0F, 0x0p+0F },
    { INF, -0x1.19999ap+0F, -0x0p+0F },
    { INF, -0x1.19999ap+0F, 0x1p+0F },
    { INF, -0x1.19999ap+0F, -0x1p+0F },
    { INF, -0x1.19999ap+0F, 0x1p-126F },
    { INF, -0x1.19999ap+0F, -0x1p-126F },
    { INF, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { INF, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { INF, -0x1.19999ap+0F, INF },
    { INF, -0x1.19999ap+0F, -INF },
    { INF, -0x1.19999ap+0F, NAN },
    { INF, -0x1.19999ap+0F, NAN },
    { INF, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { INF, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { INF, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { INF, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -INF, 0x0p+0F, 0x0p+0F },
    { -INF, 0x0p+0F, -0x0p+0F },
    { -INF, 0x0p+0F, 0x1p+0F },
    { -INF, 0x0p+0F, -0x1p+0F },
    { -INF, 0x0p+0F, 0x1p-126F },
    { -INF, 0x0p+0F, -0x1p-126F },
    { -INF, 0x0p+0F, 0x1.fffffep+127F },
    { -INF, 0x0p+0F, -0x1.fffffep+127F },
    { -INF, 0x0p+0F, INF },
    { -INF, 0x0p+0F, -INF },
    { -INF, 0x0p+0F, NAN },
    { -INF, 0x0p+0F, NAN },
    { -INF, 0x0p+0F, 0x1.ccccccp-1F },
    { -INF, 0x0p+0F, 0x1.19999ap+0F },
    { -INF, 0x0p+0F, -0x1.ccccccp-1F },
    { -INF, 0x0p+0F, -0x1.19999ap+0F },
    { -INF, -0x0p+0F, 0x0p+0F },
    { -INF, -0x0p+0F, -0x0p+0F },
    { -INF, -0x0p+0F, 0x1p+0F },
    { -INF, -0x0p+0F, -0x1p+0F },
    { -INF, -0x0p+0F, 0x1p-126F },
    { -INF, -0x0p+0F, -0x1p-126F },
    { -INF, -0x0p+0F, 0x1.fffffep+127F },
    { -INF, -0x0p+0F, -0x1.fffffep+127F },
    { -INF, -0x0p+0F, INF },
    { -INF, -0x0p+0F, -INF },
    { -INF, -0x0p+0F, NAN },
    { -INF, -0x0p+0F, NAN },
    { -INF, -0x0p+0F, 0x1.ccccccp-1F },
    { -INF, -0x0p+0F, 0x1.19999ap+0F },
    { -INF, -0x0p+0F, -0x1.ccccccp-1F },
    { -INF, -0x0p+0F, -0x1.19999ap+0F },
    { -INF, 0x1p+0F, 0x0p+0F },
    { -INF, 0x1p+0F, -0x0p+0F },
    { -INF, 0x1p+0F, 0x1p+0F },
    { -INF, 0x1p+0F, -0x1p+0F },
    { -INF, 0x1p+0F, 0x1p-126F },
    { -INF, 0x1p+0F, -0x1p-126F },
    { -INF, 0x1p+0F, 0x1.fffffep+127F },
    { -INF, 0x1p+0F, -0x1.fffffep+127F },
    { -INF, 0x1p+0F, INF },
    { -INF, 0x1p+0F, -INF },
    { -INF, 0x1p+0F, NAN },
    { -INF, 0x1p+0F, NAN },
    { -INF, 0x1p+0F, 0x1.ccccccp-1F },
    { -INF, 0x1p+0F, 0x1.19999ap+0F },
    { -INF, 0x1p+0F, -0x1.ccccccp-1F },
    { -INF, 0x1p+0F, -0x1.19999ap+0F },
    { -INF, -0x1p+0F, 0x0p+0F },
    { -INF, -0x1p+0F, -0x0p+0F },
    { -INF, -0x1p+0F, 0x1p+0F },
    { -INF, -0x1p+0F, -0x1p+0F },
    { -INF, -0x1p+0F, 0x1p-126F },
    { -INF, -0x1p+0F, -0x1p-126F },
    { -INF, -0x1p+0F, 0x1.fffffep+127F },
    { -INF, -0x1p+0F, -0x1.fffffep+127F },
    { -INF, -0x1p+0F, INF },
    { -INF, -0x1p+0F, -INF },
    { -INF, -0x1p+0F, NAN },
    { -INF, -0x1p+0F, NAN },
    { -INF, -0x1p+0F, 0x1.ccccccp-1F },
    { -INF, -0x1p+0F, 0x1.19999ap+0F },
    { -INF, -0x1p+0F, -0x1.ccccccp-1F },
    { -INF, -0x1p+0F, -0x1.19999ap+0F },
    { -INF, 0x1p-126F, 0x0p+0F },
    { -INF, 0x1p-126F, -0x0p+0F },
    { -INF, 0x1p-126F, 0x1p+0F },
    { -INF, 0x1p-126F, -0x1p+0F },
    { -INF, 0x1p-126F, 0x1p-126F },
    { -INF, 0x1p-126F, -0x1p-126F },
    { -INF, 0x1p-126F, 0x1.fffffep+127F },
    { -INF, 0x1p-126F, -0x1.fffffep+127F },
    { -INF, 0x1p-126F, INF },
    { -INF, 0x1p-126F, -INF },
    { -INF, 0x1p-126F, NAN },
    { -INF, 0x1p-126F, NAN },
    { -INF, 0x1p-126F, 0x1.ccccccp-1F },
    { -INF, 0x1p-126F, 0x1.19999ap+0F },
    { -INF, 0x1p-126F, -0x1.ccccccp-1F },
    { -INF, 0x1p-126F, -0x1.19999ap+0F },
    { -INF, -0x1p-126F, 0x0p+0F },
    { -INF, -0x1p-126F, -0x0p+0F },
    { -INF, -0x1p-126F, 0x1p+0F },
    { -INF, -0x1p-126F, -0x1p+0F },
    { -INF, -0x1p-126F, 0x1p-126F },
    { -INF, -0x1p-126F, -0x1p-126F },
    { -INF, -0x1p-126F, 0x1.fffffep+127F },
    { -INF, -0x1p-126F, -0x1.fffffep+127F },
    { -INF, -0x1p-126F, INF },
    { -INF, -0x1p-126F, -INF },
    { -INF, -0x1p-126F, NAN },
    { -INF, -0x1p-126F, NAN },
    { -INF, -0x1p-126F, 0x1.ccccccp-1F },
    { -INF, -0x1p-126F, 0x1.19999ap+0F },
    { -INF, -0x1p-126F, -0x1.ccccccp-1F },
    { -INF, -0x1p-126F, -0x1.19999ap+0F },
    { -INF, 0x1.fffffep+127F, 0x0p+0F },
    { -INF, 0x1.fffffep+127F, -0x0p+0F },
    { -INF, 0x1.fffffep+127F, 0x1p+0F },
    { -INF, 0x1.fffffep+127F, -0x1p+0F },
    { -INF, 0x1.fffffep+127F, 0x1p-126F },
    { -INF, 0x1.fffffep+127F, -0x1p-126F },
    { -INF, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -INF, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -INF, 0x1.fffffep+127F, INF },
    { -INF, 0x1.fffffep+127F, -INF },
    { -INF, 0x1.fffffep+127F, NAN },
    { -INF, 0x1.fffffep+127F, NAN },
    { -INF, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -INF, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -INF, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -INF, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -INF, -0x1.fffffep+127F, 0x0p+0F },
    { -INF, -0x1.fffffep+127F, -0x0p+0F },
    { -INF, -0x1.fffffep+127F, 0x1p+0F },
    { -INF, -0x1.fffffep+127F, -0x1p+0F },
    { -INF, -0x1.fffffep+127F, 0x1p-126F },
    { -INF, -0x1.fffffep+127F, -0x1p-126F },
    { -INF, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -INF, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -INF, -0x1.fffffep+127F, INF },
    { -INF, -0x1.fffffep+127F, -INF },
    { -INF, -0x1.fffffep+127F, NAN },
    { -INF, -0x1.fffffep+127F, NAN },
    { -INF, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -INF, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -INF, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -INF, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -INF, INF, 0x0p+0F },
    { -INF, INF, -0x0p+0F },
    { -INF, INF, 0x1p+0F },
    { -INF, INF, -0x1p+0F },
    { -INF, INF, 0x1p-126F },
    { -INF, INF, -0x1p-126F },
    { -INF, INF, 0x1.fffffep+127F },
    { -INF, INF, -0x1.fffffep+127F },
    { -INF, INF, INF },
    { -INF, INF, -INF },
    { -INF, INF, NAN },
    { -INF, INF, NAN },
    { -INF, INF, 0x1.ccccccp-1F },
    { -INF, INF, 0x1.19999ap+0F },
    { -INF, INF, -0x1.ccccccp-1F },
    { -INF, INF, -0x1.19999ap+0F },
    { -INF, -INF, 0x0p+0F },
    { -INF, -INF, -0x0p+0F },
    { -INF, -INF, 0x1p+0F },
    { -INF, -INF, -0x1p+0F },
    { -INF, -INF, 0x1p-126F },
    { -INF, -INF, -0x1p-126F },
    { -INF, -INF, 0x1.fffffep+127F },
    { -INF, -INF, -0x1.fffffep+127F },
    { -INF, -INF, INF },
    { -INF, -INF, -INF },
    { -INF, -INF, NAN },
    { -INF, -INF, NAN },
    { -INF, -INF, 0x1.ccccccp-1F },
    { -INF, -INF, 0x1.19999ap+0F },
    { -INF, -INF, -0x1.ccccccp-1F },
    { -INF, -INF, -0x1.19999ap+0F },
    { -INF, NAN, 0x0p+0F },
    { -INF, NAN, -0x0p+0F },
    { -INF, NAN, 0x1p+0F },
    { -INF, NAN, -0x1p+0F },
    { -INF, NAN, 0x1p-126F },
    { -INF, NAN, -0x1p-126F },
    { -INF, NAN, 0x1.fffffep+127F },
    { -INF, NAN, -0x1.fffffep+127F },
    { -INF, NAN, INF },
    { -INF, NAN, -INF },
    { -INF, NAN, NAN },
    { -INF, NAN, NAN },
    { -INF, NAN, 0x1.ccccccp-1F },
    { -INF, NAN, 0x1.19999ap+0F },
    { -INF, NAN, -0x1.ccccccp-1F },
    { -INF, NAN, -0x1.19999ap+0F },
    { -INF, NAN, 0x0p+0F },
    { -INF, NAN, -0x0p+0F },
    { -INF, NAN, 0x1p+0F },
    { -INF, NAN, -0x1p+0F },
    { -INF, NAN, 0x1p-126F },
    { -INF, NAN, -0x1p-126F },
    { -INF, NAN, 0x1.fffffep+127F },
    { -INF, NAN, -0x1.fffffep+127F },
    { -INF, NAN, INF },
    { -INF, NAN, -INF },
    { -INF, NAN, NAN },
    { -INF, NAN, NAN },
    { -INF, NAN, 0x1.ccccccp-1F },
    { -INF, NAN, 0x1.19999ap+0F },
    { -INF, NAN, -0x1.ccccccp-1F },
    { -INF, NAN, -0x1.19999ap+0F },
    { -INF, 0x1.ccccccp-1F, 0x0p+0F },
    { -INF, 0x1.ccccccp-1F, -0x0p+0F },
    { -INF, 0x1.ccccccp-1F, 0x1p+0F },
    { -INF, 0x1.ccccccp-1F, -0x1p+0F },
    { -INF, 0x1.ccccccp-1F, 0x1p-126F },
    { -INF, 0x1.ccccccp-1F, -0x1p-126F },
    { -INF, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -INF, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -INF, 0x1.ccccccp-1F, INF },
    { -INF, 0x1.ccccccp-1F, -INF },
    { -INF, 0x1.ccccccp-1F, NAN },
    { -INF, 0x1.ccccccp-1F, NAN },
    { -INF, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -INF, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -INF, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -INF, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -INF, 0x1.19999ap+0F, 0x0p+0F },
    { -INF, 0x1.19999ap+0F, -0x0p+0F },
    { -INF, 0x1.19999ap+0F, 0x1p+0F },
    { -INF, 0x1.19999ap+0F, -0x1p+0F },
    { -INF, 0x1.19999ap+0F, 0x1p-126F },
    { -INF, 0x1.19999ap+0F, -0x1p-126F },
    { -INF, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -INF, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -INF, 0x1.19999ap+0F, INF },
    { -INF, 0x1.19999ap+0F, -INF },
    { -INF, 0x1.19999ap+0F, NAN },
    { -INF, 0x1.19999ap+0F, NAN },
    { -INF, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -INF, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -INF, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -INF, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -INF, -0x1.ccccccp-1F, 0x0p+0F },
    { -INF, -0x1.ccccccp-1F, -0x0p+0F },
    { -INF, -0x1.ccccccp-1F, 0x1p+0F },
    { -INF, -0x1.ccccccp-1F, -0x1p+0F },
    { -INF, -0x1.ccccccp-1F, 0x1p-126F },
    { -INF, -0x1.ccccccp-1F, -0x1p-126F },
    { -INF, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -INF, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -INF, -0x1.ccccccp-1F, INF },
    { -INF, -0x1.ccccccp-1F, -INF },
    { -INF, -0x1.ccccccp-1F, NAN },
    { -INF, -0x1.ccccccp-1F, NAN },
    { -INF, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -INF, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -INF, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -INF, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -INF, -0x1.19999ap+0F, 0x0p+0F },
    { -INF, -0x1.19999ap+0F, -0x0p+0F },
    { -INF, -0x1.19999ap+0F, 0x1p+0F },
    { -INF, -0x1.19999ap+0F, -0x1p+0F },
    { -INF, -0x1.19999ap+0F, 0x1p-126F },
    { -INF, -0x1.19999ap+0F, -0x1p-126F },
    { -INF, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -INF, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -INF, -0x1.19999ap+0F, INF },
    { -INF, -0x1.19999ap+0F, -INF },
    { -INF, -0x1.19999ap+0F, NAN },
    { -INF, -0x1.19999ap+0F, NAN },
    { -INF, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -INF, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -INF, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -INF, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { NAN, 0x0p+0F, 0x0p+0F },
    { NAN, 0x0p+0F, -0x0p+0F },
    { NAN, 0x0p+0F, 0x1p+0F },
    { NAN, 0x0p+0F, -0x1p+0F },
    { NAN, 0x0p+0F, 0x1p-126F },
    { NAN, 0x0p+0F, -0x1p-126F },
    { NAN, 0x0p+0F, 0x1.fffffep+127F },
    { NAN, 0x0p+0F, -0x1.fffffep+127F },
    { NAN, 0x0p+0F, INF },
    { NAN, 0x0p+0F, -INF },
    { NAN, 0x0p+0F, NAN },
    { NAN, 0x0p+0F, NAN },
    { NAN, 0x0p+0F, 0x1.ccccccp-1F },
    { NAN, 0x0p+0F, 0x1.19999ap+0F },
    { NAN, 0x0p+0F, -0x1.ccccccp-1F },
    { NAN, 0x0p+0F, -0x1.19999ap+0F },
    { NAN, -0x0p+0F, 0x0p+0F },
    { NAN, -0x0p+0F, -0x0p+0F },
    { NAN, -0x0p+0F, 0x1p+0F },
    { NAN, -0x0p+0F, -0x1p+0F },
    { NAN, -0x0p+0F, 0x1p-126F },
    { NAN, -0x0p+0F, -0x1p-126F },
    { NAN, -0x0p+0F, 0x1.fffffep+127F },
    { NAN, -0x0p+0F, -0x1.fffffep+127F },
    { NAN, -0x0p+0F, INF },
    { NAN, -0x0p+0F, -INF },
    { NAN, -0x0p+0F, NAN },
    { NAN, -0x0p+0F, NAN },
    { NAN, -0x0p+0F, 0x1.ccccccp-1F },
    { NAN, -0x0p+0F, 0x1.19999ap+0F },
    { NAN, -0x0p+0F, -0x1.ccccccp-1F },
    { NAN, -0x0p+0F, -0x1.19999ap+0F },
    { NAN, 0x1p+0F, 0x0p+0F },
    { NAN, 0x1p+0F, -0x0p+0F },
    { NAN, 0x1p+0F, 0x1p+0F },
    { NAN, 0x1p+0F, -0x1p+0F },
    { NAN, 0x1p+0F, 0x1p-126F },
    { NAN, 0x1p+0F, -0x1p-126F },
    { NAN, 0x1p+0F, 0x1.fffffep+127F },
    { NAN, 0x1p+0F, -0x1.fffffep+127F },
    { NAN, 0x1p+0F, INF },
    { NAN, 0x1p+0F, -INF },
    { NAN, 0x1p+0F, NAN },
    { NAN, 0x1p+0F, NAN },
    { NAN, 0x1p+0F, 0x1.ccccccp-1F },
    { NAN, 0x1p+0F, 0x1.19999ap+0F },
    { NAN, 0x1p+0F, -0x1.ccccccp-1F },
    { NAN, 0x1p+0F, -0x1.19999ap+0F },
    { NAN, -0x1p+0F, 0x0p+0F },
    { NAN, -0x1p+0F, -0x0p+0F },
    { NAN, -0x1p+0F, 0x1p+0F },
    { NAN, -0x1p+0F, -0x1p+0F },
    { NAN, -0x1p+0F, 0x1p-126F },
    { NAN, -0x1p+0F, -0x1p-126F },
    { NAN, -0x1p+0F, 0x1.fffffep+127F },
    { NAN, -0x1p+0F, -0x1.fffffep+127F },
    { NAN, -0x1p+0F, INF },
    { NAN, -0x1p+0F, -INF },
    { NAN, -0x1p+0F, NAN },
    { NAN, -0x1p+0F, NAN },
    { NAN, -0x1p+0F, 0x1.ccccccp-1F },
    { NAN, -0x1p+0F, 0x1.19999ap+0F },
    { NAN, -0x1p+0F, -0x1.ccccccp-1F },
    { NAN, -0x1p+0F, -0x1.19999ap+0F },
    { NAN, 0x1p-126F, 0x0p+0F },
    { NAN, 0x1p-126F, -0x0p+0F },
    { NAN, 0x1p-126F, 0x1p+0F },
    { NAN, 0x1p-126F, -0x1p+0F },
    { NAN, 0x1p-126F, 0x1p-126F },
    { NAN, 0x1p-126F, -0x1p-126F },
    { NAN, 0x1p-126F, 0x1.fffffep+127F },
    { NAN, 0x1p-126F, -0x1.fffffep+127F },
    { NAN, 0x1p-126F, INF },
    { NAN, 0x1p-126F, -INF },
    { NAN, 0x1p-126F, NAN },
    { NAN, 0x1p-126F, NAN },
    { NAN, 0x1p-126F, 0x1.ccccccp-1F },
    { NAN, 0x1p-126F, 0x1.19999ap+0F },
    { NAN, 0x1p-126F, -0x1.ccccccp-1F },
    { NAN, 0x1p-126F, -0x1.19999ap+0F },
    { NAN, -0x1p-126F, 0x0p+0F },
    { NAN, -0x1p-126F, -0x0p+0F },
    { NAN, -0x1p-126F, 0x1p+0F },
    { NAN, -0x1p-126F, -0x1p+0F },
    { NAN, -0x1p-126F, 0x1p-126F },
    { NAN, -0x1p-126F, -0x1p-126F },
    { NAN, -0x1p-126F, 0x1.fffffep+127F },
    { NAN, -0x1p-126F, -0x1.fffffep+127F },
    { NAN, -0x1p-126F, INF },
    { NAN, -0x1p-126F, -INF },
    { NAN, -0x1p-126F, NAN },
    { NAN, -0x1p-126F, NAN },
    { NAN, -0x1p-126F, 0x1.ccccccp-1F },
    { NAN, -0x1p-126F, 0x1.19999ap+0F },
    { NAN, -0x1p-126F, -0x1.ccccccp-1F },
    { NAN, -0x1p-126F, -0x1.19999ap+0F },
    { NAN, 0x1.fffffep+127F, 0x0p+0F },
    { NAN, 0x1.fffffep+127F, -0x0p+0F },
    { NAN, 0x1.fffffep+127F, 0x1p+0F },
    { NAN, 0x1.fffffep+127F, -0x1p+0F },
    { NAN, 0x1.fffffep+127F, 0x1p-126F },
    { NAN, 0x1.fffffep+127F, -0x1p-126F },
    { NAN, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { NAN, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { NAN, 0x1.fffffep+127F, INF },
    { NAN, 0x1.fffffep+127F, -INF },
    { NAN, 0x1.fffffep+127F, NAN },
    { NAN, 0x1.fffffep+127F, NAN },
    { NAN, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { NAN, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { NAN, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { NAN, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { NAN, -0x1.fffffep+127F, 0x0p+0F },
    { NAN, -0x1.fffffep+127F, -0x0p+0F },
    { NAN, -0x1.fffffep+127F, 0x1p+0F },
    { NAN, -0x1.fffffep+127F, -0x1p+0F },
    { NAN, -0x1.fffffep+127F, 0x1p-126F },
    { NAN, -0x1.fffffep+127F, -0x1p-126F },
    { NAN, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F, INF },
    { NAN, -0x1.fffffep+127F, -INF },
    { NAN, -0x1.fffffep+127F, NAN },
    { NAN, -0x1.fffffep+127F, NAN },
    { NAN, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { NAN, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { NAN, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { NAN, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { NAN, INF, 0x0p+0F },
    { NAN, INF, -0x0p+0F },
    { NAN, INF, 0x1p+0F },
    { NAN, INF, -0x1p+0F },
    { NAN, INF, 0x1p-126F },
    { NAN, INF, -0x1p-126F },
    { NAN, INF, 0x1.fffffep+127F },
    { NAN, INF, -0x1.fffffep+127F },
    { NAN, INF, INF },
    { NAN, INF, -INF },
    { NAN, INF, NAN },
    { NAN, INF, NAN },
    { NAN, INF, 0x1.ccccccp-1F },
    { NAN, INF, 0x1.19999ap+0F },
    { NAN, INF, -0x1.ccccccp-1F },
    { NAN, INF, -0x1.19999ap+0F },
    { NAN, -INF, 0x0p+0F },
    { NAN, -INF, -0x0p+0F },
    { NAN, -INF, 0x1p+0F },
    { NAN, -INF, -0x1p+0F },
    { NAN, -INF, 0x1p-126F },
    { NAN, -INF, -0x1p-126F },
    { NAN, -INF, 0x1.fffffep+127F },
    { NAN, -INF, -0x1.fffffep+127F },
    { NAN, -INF, INF },
    { NAN, -INF, -INF },
    { NAN, -INF, NAN },
    { NAN, -INF, NAN },
    { NAN, -INF, 0x1.ccccccp-1F },
    { NAN, -INF, 0x1.19999ap+0F },
    { NAN, -INF, -0x1.ccccccp-1F },
    { NAN, -INF, -0x1.19999ap+0F },
    { NAN, NAN, 0x0p+0F },
    { NAN, NAN, -0x0p+0F },
    { NAN, NAN, 0x1p+0F },
    { NAN, NAN, -0x1p+0F },
    { NAN, NAN, 0x1p-126F },
    { NAN, NAN, -0x1p-126F },
    { NAN, NAN, 0x1.fffffep+127F },
    { NAN, NAN, -0x1.fffffep+127F },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccp-1F },
    { NAN, NAN, 0x1.19999ap+0F },
    { NAN, NAN, -0x1.ccccccp-1F },
    { NAN, NAN, -0x1.19999ap+0F },
    { NAN, NAN, 0x0p+0F },
    { NAN, NAN, -0x0p+0F },
    { NAN, NAN, 0x1p+0F },
    { NAN, NAN, -0x1p+0F },
    { NAN, NAN, 0x1p-126F },
    { NAN, NAN, -0x1p-126F },
    { NAN, NAN, 0x1.fffffep+127F },
    { NAN, NAN, -0x1.fffffep+127F },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccp-1F },
    { NAN, NAN, 0x1.19999ap+0F },
    { NAN, NAN, -0x1.ccccccp-1F },
    { NAN, NAN, -0x1.19999ap+0F },
    { NAN, 0x1.ccccccp-1F, 0x0p+0F },
    { NAN, 0x1.ccccccp-1F, -0x0p+0F },
    { NAN, 0x1.ccccccp-1F, 0x1p+0F },
    { NAN, 0x1.ccccccp-1F, -0x1p+0F },
    { NAN, 0x1.ccccccp-1F, 0x1p-126F },
    { NAN, 0x1.ccccccp-1F, -0x1p-126F },
    { NAN, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { NAN, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { NAN, 0x1.ccccccp-1F, INF },
    { NAN, 0x1.ccccccp-1F, -INF },
    { NAN, 0x1.ccccccp-1F, NAN },
    { NAN, 0x1.ccccccp-1F, NAN },
    { NAN, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { NAN, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { NAN, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { NAN, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { NAN, 0x1.19999ap+0F, 0x0p+0F },
    { NAN, 0x1.19999ap+0F, -0x0p+0F },
    { NAN, 0x1.19999ap+0F, 0x1p+0F },
    { NAN, 0x1.19999ap+0F, -0x1p+0F },
    { NAN, 0x1.19999ap+0F, 0x1p-126F },
    { NAN, 0x1.19999ap+0F, -0x1p-126F },
    { NAN, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { NAN, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { NAN, 0x1.19999ap+0F, INF },
    { NAN, 0x1.19999ap+0F, -INF },
    { NAN, 0x1.19999ap+0F, NAN },
    { NAN, 0x1.19999ap+0F, NAN },
    { NAN, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { NAN, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F, 0x0p+0F },
    { NAN, -0x1.ccccccp-1F, -0x0p+0F },
    { NAN, -0x1.ccccccp-1F, 0x1p+0F },
    { NAN, -0x1.ccccccp-1F, -0x1p+0F },
    { NAN, -0x1.ccccccp-1F, 0x1p-126F },
    { NAN, -0x1.ccccccp-1F, -0x1p-126F },
    { NAN, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { NAN, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { NAN, -0x1.ccccccp-1F, INF },
    { NAN, -0x1.ccccccp-1F, -INF },
    { NAN, -0x1.ccccccp-1F, NAN },
    { NAN, -0x1.ccccccp-1F, NAN },
    { NAN, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { NAN, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { NAN, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { NAN, -0x1.19999ap+0F, 0x0p+0F },
    { NAN, -0x1.19999ap+0F, -0x0p+0F },
    { NAN, -0x1.19999ap+0F, 0x1p+0F },
    { NAN, -0x1.19999ap+0F, -0x1p+0F },
    { NAN, -0x1.19999ap+0F, 0x1p-126F },
    { NAN, -0x1.19999ap+0F, -0x1p-126F },
    { NAN, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { NAN, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { NAN, -0x1.19999ap+0F, INF },
    { NAN, -0x1.19999ap+0F, -INF },
    { NAN, -0x1.19999ap+0F, NAN },
    { NAN, -0x1.19999ap+0F, NAN },
    { NAN, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { NAN, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { NAN, 0x0p+0F, 0x0p+0F },
    { NAN, 0x0p+0F, -0x0p+0F },
    { NAN, 0x0p+0F, 0x1p+0F },
    { NAN, 0x0p+0F, -0x1p+0F },
    { NAN, 0x0p+0F, 0x1p-126F },
    { NAN, 0x0p+0F, -0x1p-126F },
    { NAN, 0x0p+0F, 0x1.fffffep+127F },
    { NAN, 0x0p+0F, -0x1.fffffep+127F },
    { NAN, 0x0p+0F, INF },
    { NAN, 0x0p+0F, -INF },
    { NAN, 0x0p+0F, NAN },
    { NAN, 0x0p+0F, NAN },
    { NAN, 0x0p+0F, 0x1.ccccccp-1F },
    { NAN, 0x0p+0F, 0x1.19999ap+0F },
    { NAN, 0x0p+0F, -0x1.ccccccp-1F },
    { NAN, 0x0p+0F, -0x1.19999ap+0F },
    { NAN, -0x0p+0F, 0x0p+0F },
    { NAN, -0x0p+0F, -0x0p+0F },
    { NAN, -0x0p+0F, 0x1p+0F },
    { NAN, -0x0p+0F, -0x1p+0F },
    { NAN, -0x0p+0F, 0x1p-126F },
    { NAN, -0x0p+0F, -0x1p-126F },
    { NAN, -0x0p+0F, 0x1.fffffep+127F },
    { NAN, -0x0p+0F, -0x1.fffffep+127F },
    { NAN, -0x0p+0F, INF },
    { NAN, -0x0p+0F, -INF },
    { NAN, -0x0p+0F, NAN },
    { NAN, -0x0p+0F, NAN },
    { NAN, -0x0p+0F, 0x1.ccccccp-1F },
    { NAN, -0x0p+0F, 0x1.19999ap+0F },
    { NAN, -0x0p+0F, -0x1.ccccccp-1F },
    { NAN, -0x0p+0F, -0x1.19999ap+0F },
    { NAN, 0x1p+0F, 0x0p+0F },
    { NAN, 0x1p+0F, -0x0p+0F },
    { NAN, 0x1p+0F, 0x1p+0F },
    { NAN, 0x1p+0F, -0x1p+0F },
    { NAN, 0x1p+0F, 0x1p-126F },
    { NAN, 0x1p+0F, -0x1p-126F },
    { NAN, 0x1p+0F, 0x1.fffffep+127F },
    { NAN, 0x1p+0F, -0x1.fffffep+127F },
    { NAN, 0x1p+0F, INF },
    { NAN, 0x1p+0F, -INF },
    { NAN, 0x1p+0F, NAN },
    { NAN, 0x1p+0F, NAN },
    { NAN, 0x1p+0F, 0x1.ccccccp-1F },
    { NAN, 0x1p+0F, 0x1.19999ap+0F },
    { NAN, 0x1p+0F, -0x1.ccccccp-1F },
    { NAN, 0x1p+0F, -0x1.19999ap+0F },
    { NAN, -0x1p+0F, 0x0p+0F },
    { NAN, -0x1p+0F, -0x0p+0F },
    { NAN, -0x1p+0F, 0x1p+0F },
    { NAN, -0x1p+0F, -0x1p+0F },
    { NAN, -0x1p+0F, 0x1p-126F },
    { NAN, -0x1p+0F, -0x1p-126F },
    { NAN, -0x1p+0F, 0x1.fffffep+127F },
    { NAN, -0x1p+0F, -0x1.fffffep+127F },
    { NAN, -0x1p+0F, INF },
    { NAN, -0x1p+0F, -INF },
    { NAN, -0x1p+0F, NAN },
    { NAN, -0x1p+0F, NAN },
    { NAN, -0x1p+0F, 0x1.ccccccp-1F },
    { NAN, -0x1p+0F, 0x1.19999ap+0F },
    { NAN, -0x1p+0F, -0x1.ccccccp-1F },
    { NAN, -0x1p+0F, -0x1.19999ap+0F },
    { NAN, 0x1p-126F, 0x0p+0F },
    { NAN, 0x1p-126F, -0x0p+0F },
    { NAN, 0x1p-126F, 0x1p+0F },
    { NAN, 0x1p-126F, -0x1p+0F },
    { NAN, 0x1p-126F, 0x1p-126F },
    { NAN, 0x1p-126F, -0x1p-126F },
    { NAN, 0x1p-126F, 0x1.fffffep+127F },
    { NAN, 0x1p-126F, -0x1.fffffep+127F },
    { NAN, 0x1p-126F, INF },
    { NAN, 0x1p-126F, -INF },
    { NAN, 0x1p-126F, NAN },
    { NAN, 0x1p-126F, NAN },
    { NAN, 0x1p-126F, 0x1.ccccccp-1F },
    { NAN, 0x1p-126F, 0x1.19999ap+0F },
    { NAN, 0x1p-126F, -0x1.ccccccp-1F },
    { NAN, 0x1p-126F, -0x1.19999ap+0F },
    { NAN, -0x1p-126F, 0x0p+0F },
    { NAN, -0x1p-126F, -0x0p+0F },
    { NAN, -0x1p-126F, 0x1p+0F },
    { NAN, -0x1p-126F, -0x1p+0F },
    { NAN, -0x1p-126F, 0x1p-126F },
    { NAN, -0x1p-126F, -0x1p-126F },
    { NAN, -0x1p-126F, 0x1.fffffep+127F },
    { NAN, -0x1p-126F, -0x1.fffffep+127F },
    { NAN, -0x1p-126F, INF },
    { NAN, -0x1p-126F, -INF },
    { NAN, -0x1p-126F, NAN },
    { NAN, -0x1p-126F, NAN },
    { NAN, -0x1p-126F, 0x1.ccccccp-1F },
    { NAN, -0x1p-126F, 0x1.19999ap+0F },
    { NAN, -0x1p-126F, -0x1.ccccccp-1F },
    { NAN, -0x1p-126F, -0x1.19999ap+0F },
    { NAN, 0x1.fffffep+127F, 0x0p+0F },
    { NAN, 0x1.fffffep+127F, -0x0p+0F },
    { NAN, 0x1.fffffep+127F, 0x1p+0F },
    { NAN, 0x1.fffffep+127F, -0x1p+0F },
    { NAN, 0x1.fffffep+127F, 0x1p-126F },
    { NAN, 0x1.fffffep+127F, -0x1p-126F },
    { NAN, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { NAN, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { NAN, 0x1.fffffep+127F, INF },
    { NAN, 0x1.fffffep+127F, -INF },
    { NAN, 0x1.fffffep+127F, NAN },
    { NAN, 0x1.fffffep+127F, NAN },
    { NAN, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { NAN, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { NAN, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { NAN, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { NAN, -0x1.fffffep+127F, 0x0p+0F },
    { NAN, -0x1.fffffep+127F, -0x0p+0F },
    { NAN, -0x1.fffffep+127F, 0x1p+0F },
    { NAN, -0x1.fffffep+127F, -0x1p+0F },
    { NAN, -0x1.fffffep+127F, 0x1p-126F },
    { NAN, -0x1.fffffep+127F, -0x1p-126F },
    { NAN, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { NAN, -0x1.fffffep+127F, INF },
    { NAN, -0x1.fffffep+127F, -INF },
    { NAN, -0x1.fffffep+127F, NAN },
    { NAN, -0x1.fffffep+127F, NAN },
    { NAN, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { NAN, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { NAN, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { NAN, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { NAN, INF, 0x0p+0F },
    { NAN, INF, -0x0p+0F },
    { NAN, INF, 0x1p+0F },
    { NAN, INF, -0x1p+0F },
    { NAN, INF, 0x1p-126F },
    { NAN, INF, -0x1p-126F },
    { NAN, INF, 0x1.fffffep+127F },
    { NAN, INF, -0x1.fffffep+127F },
    { NAN, INF, INF },
    { NAN, INF, -INF },
    { NAN, INF, NAN },
    { NAN, INF, NAN },
    { NAN, INF, 0x1.ccccccp-1F },
    { NAN, INF, 0x1.19999ap+0F },
    { NAN, INF, -0x1.ccccccp-1F },
    { NAN, INF, -0x1.19999ap+0F },
    { NAN, -INF, 0x0p+0F },
    { NAN, -INF, -0x0p+0F },
    { NAN, -INF, 0x1p+0F },
    { NAN, -INF, -0x1p+0F },
    { NAN, -INF, 0x1p-126F },
    { NAN, -INF, -0x1p-126F },
    { NAN, -INF, 0x1.fffffep+127F },
    { NAN, -INF, -0x1.fffffep+127F },
    { NAN, -INF, INF },
    { NAN, -INF, -INF },
    { NAN, -INF, NAN },
    { NAN, -INF, NAN },
    { NAN, -INF, 0x1.ccccccp-1F },
    { NAN, -INF, 0x1.19999ap+0F },
    { NAN, -INF, -0x1.ccccccp-1F },
    { NAN, -INF, -0x1.19999ap+0F },
    { NAN, NAN, 0x0p+0F },
    { NAN, NAN, -0x0p+0F },
    { NAN, NAN, 0x1p+0F },
    { NAN, NAN, -0x1p+0F },
    { NAN, NAN, 0x1p-126F },
    { NAN, NAN, -0x1p-126F },
    { NAN, NAN, 0x1.fffffep+127F },
    { NAN, NAN, -0x1.fffffep+127F },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccp-1F },
    { NAN, NAN, 0x1.19999ap+0F },
    { NAN, NAN, -0x1.ccccccp-1F },
    { NAN, NAN, -0x1.19999ap+0F },
    { NAN, NAN, 0x0p+0F },
    { NAN, NAN, -0x0p+0F },
    { NAN, NAN, 0x1p+0F },
    { NAN, NAN, -0x1p+0F },
    { NAN, NAN, 0x1p-126F },
    { NAN, NAN, -0x1p-126F },
    { NAN, NAN, 0x1.fffffep+127F },
    { NAN, NAN, -0x1.fffffep+127F },
    { NAN, NAN, INF },
    { NAN, NAN, -INF },
    { NAN, NAN, NAN },
    { NAN, NAN, NAN },
    { NAN, NAN, 0x1.ccccccp-1F },
    { NAN, NAN, 0x1.19999ap+0F },
    { NAN, NAN, -0x1.ccccccp-1F },
    { NAN, NAN, -0x1.19999ap+0F },
    { NAN, 0x1.ccccccp-1F, 0x0p+0F },
    { NAN, 0x1.ccccccp-1F, -0x0p+0F },
    { NAN, 0x1.ccccccp-1F, 0x1p+0F },
    { NAN, 0x1.ccccccp-1F, -0x1p+0F },
    { NAN, 0x1.ccccccp-1F, 0x1p-126F },
    { NAN, 0x1.ccccccp-1F, -0x1p-126F },
    { NAN, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { NAN, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { NAN, 0x1.ccccccp-1F, INF },
    { NAN, 0x1.ccccccp-1F, -INF },
    { NAN, 0x1.ccccccp-1F, NAN },
    { NAN, 0x1.ccccccp-1F, NAN },
    { NAN, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { NAN, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { NAN, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { NAN, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { NAN, 0x1.19999ap+0F, 0x0p+0F },
    { NAN, 0x1.19999ap+0F, -0x0p+0F },
    { NAN, 0x1.19999ap+0F, 0x1p+0F },
    { NAN, 0x1.19999ap+0F, -0x1p+0F },
    { NAN, 0x1.19999ap+0F, 0x1p-126F },
    { NAN, 0x1.19999ap+0F, -0x1p-126F },
    { NAN, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { NAN, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { NAN, 0x1.19999ap+0F, INF },
    { NAN, 0x1.19999ap+0F, -INF },
    { NAN, 0x1.19999ap+0F, NAN },
    { NAN, 0x1.19999ap+0F, NAN },
    { NAN, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { NAN, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { NAN, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F, 0x0p+0F },
    { NAN, -0x1.ccccccp-1F, -0x0p+0F },
    { NAN, -0x1.ccccccp-1F, 0x1p+0F },
    { NAN, -0x1.ccccccp-1F, -0x1p+0F },
    { NAN, -0x1.ccccccp-1F, 0x1p-126F },
    { NAN, -0x1.ccccccp-1F, -0x1p-126F },
    { NAN, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { NAN, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { NAN, -0x1.ccccccp-1F, INF },
    { NAN, -0x1.ccccccp-1F, -INF },
    { NAN, -0x1.ccccccp-1F, NAN },
    { NAN, -0x1.ccccccp-1F, NAN },
    { NAN, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { NAN, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { NAN, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { NAN, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { NAN, -0x1.19999ap+0F, 0x0p+0F },
    { NAN, -0x1.19999ap+0F, -0x0p+0F },
    { NAN, -0x1.19999ap+0F, 0x1p+0F },
    { NAN, -0x1.19999ap+0F, -0x1p+0F },
    { NAN, -0x1.19999ap+0F, 0x1p-126F },
    { NAN, -0x1.19999ap+0F, -0x1p-126F },
    { NAN, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { NAN, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { NAN, -0x1.19999ap+0F, INF },
    { NAN, -0x1.19999ap+0F, -INF },
    { NAN, -0x1.19999ap+0F, NAN },
    { NAN, -0x1.19999ap+0F, NAN },
    { NAN, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { NAN, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { NAN, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x0p+0F, INF },
    { 0x1.ccccccp-1F, 0x0p+0F, -INF },
    { 0x1.ccccccp-1F, 0x0p+0F, NAN },
    { 0x1.ccccccp-1F, 0x0p+0F, NAN },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x0p+0F, INF },
    { 0x1.ccccccp-1F, -0x0p+0F, -INF },
    { 0x1.ccccccp-1F, -0x0p+0F, NAN },
    { 0x1.ccccccp-1F, -0x0p+0F, NAN },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1p+0F, INF },
    { 0x1.ccccccp-1F, 0x1p+0F, -INF },
    { 0x1.ccccccp-1F, 0x1p+0F, NAN },
    { 0x1.ccccccp-1F, 0x1p+0F, NAN },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1p+0F, INF },
    { 0x1.ccccccp-1F, -0x1p+0F, -INF },
    { 0x1.ccccccp-1F, -0x1p+0F, NAN },
    { 0x1.ccccccp-1F, -0x1p+0F, NAN },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1p-126F, INF },
    { 0x1.ccccccp-1F, 0x1p-126F, -INF },
    { 0x1.ccccccp-1F, 0x1p-126F, NAN },
    { 0x1.ccccccp-1F, 0x1p-126F, NAN },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1p-126F, INF },
    { 0x1.ccccccp-1F, -0x1p-126F, -INF },
    { 0x1.ccccccp-1F, -0x1p-126F, NAN },
    { 0x1.ccccccp-1F, -0x1p-126F, NAN },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, INF },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -INF },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, NAN },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, NAN },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, INF },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -INF },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, NAN },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, NAN },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, INF, 0x0p+0F },
    { 0x1.ccccccp-1F, INF, -0x0p+0F },
    { 0x1.ccccccp-1F, INF, 0x1p+0F },
    { 0x1.ccccccp-1F, INF, -0x1p+0F },
    { 0x1.ccccccp-1F, INF, 0x1p-126F },
    { 0x1.ccccccp-1F, INF, -0x1p-126F },
    { 0x1.ccccccp-1F, INF, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, INF, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, INF, INF },
    { 0x1.ccccccp-1F, INF, -INF },
    { 0x1.ccccccp-1F, INF, NAN },
    { 0x1.ccccccp-1F, INF, NAN },
    { 0x1.ccccccp-1F, INF, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, INF, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, INF, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, INF, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -INF, 0x0p+0F },
    { 0x1.ccccccp-1F, -INF, -0x0p+0F },
    { 0x1.ccccccp-1F, -INF, 0x1p+0F },
    { 0x1.ccccccp-1F, -INF, -0x1p+0F },
    { 0x1.ccccccp-1F, -INF, 0x1p-126F },
    { 0x1.ccccccp-1F, -INF, -0x1p-126F },
    { 0x1.ccccccp-1F, -INF, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -INF, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -INF, INF },
    { 0x1.ccccccp-1F, -INF, -INF },
    { 0x1.ccccccp-1F, -INF, NAN },
    { 0x1.ccccccp-1F, -INF, NAN },
    { 0x1.ccccccp-1F, -INF, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -INF, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -INF, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -INF, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, NAN, 0x0p+0F },
    { 0x1.ccccccp-1F, NAN, -0x0p+0F },
    { 0x1.ccccccp-1F, NAN, 0x1p+0F },
    { 0x1.ccccccp-1F, NAN, -0x1p+0F },
    { 0x1.ccccccp-1F, NAN, 0x1p-126F },
    { 0x1.ccccccp-1F, NAN, -0x1p-126F },
    { 0x1.ccccccp-1F, NAN, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, NAN, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, NAN, INF },
    { 0x1.ccccccp-1F, NAN, -INF },
    { 0x1.ccccccp-1F, NAN, NAN },
    { 0x1.ccccccp-1F, NAN, NAN },
    { 0x1.ccccccp-1F, NAN, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, NAN, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, NAN, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, NAN, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, NAN, 0x0p+0F },
    { 0x1.ccccccp-1F, NAN, -0x0p+0F },
    { 0x1.ccccccp-1F, NAN, 0x1p+0F },
    { 0x1.ccccccp-1F, NAN, -0x1p+0F },
    { 0x1.ccccccp-1F, NAN, 0x1p-126F },
    { 0x1.ccccccp-1F, NAN, -0x1p-126F },
    { 0x1.ccccccp-1F, NAN, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, NAN, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, NAN, INF },
    { 0x1.ccccccp-1F, NAN, -INF },
    { 0x1.ccccccp-1F, NAN, NAN },
    { 0x1.ccccccp-1F, NAN, NAN },
    { 0x1.ccccccp-1F, NAN, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, NAN, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, NAN, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, NAN, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, INF },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -INF },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, INF },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -INF },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, NAN },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, NAN },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, INF },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -INF },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, NAN },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, INF },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -INF },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, NAN },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, NAN },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x0p+0F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x0p+0F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x0p+0F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x0p+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x0p+0F, INF },
    { 0x1.19999ap+0F, 0x0p+0F, -INF },
    { 0x1.19999ap+0F, 0x0p+0F, NAN },
    { 0x1.19999ap+0F, 0x0p+0F, NAN },
    { 0x1.19999ap+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x0p+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x0p+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x0p+0F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x0p+0F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x0p+0F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x0p+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x0p+0F, INF },
    { 0x1.19999ap+0F, -0x0p+0F, -INF },
    { 0x1.19999ap+0F, -0x0p+0F, NAN },
    { 0x1.19999ap+0F, -0x0p+0F, NAN },
    { 0x1.19999ap+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x0p+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x0p+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1p+0F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1p+0F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1p+0F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1p+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1p+0F, INF },
    { 0x1.19999ap+0F, 0x1p+0F, -INF },
    { 0x1.19999ap+0F, 0x1p+0F, NAN },
    { 0x1.19999ap+0F, 0x1p+0F, NAN },
    { 0x1.19999ap+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1p+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1p+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1p+0F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x1p+0F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x1p+0F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x1p+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1p+0F, INF },
    { 0x1.19999ap+0F, -0x1p+0F, -INF },
    { 0x1.19999ap+0F, -0x1p+0F, NAN },
    { 0x1.19999ap+0F, -0x1p+0F, NAN },
    { 0x1.19999ap+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1p+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1p+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1p-126F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1p-126F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1p-126F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1p-126F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1p-126F, INF },
    { 0x1.19999ap+0F, 0x1p-126F, -INF },
    { 0x1.19999ap+0F, 0x1p-126F, NAN },
    { 0x1.19999ap+0F, 0x1p-126F, NAN },
    { 0x1.19999ap+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1p-126F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1p-126F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1p-126F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x1p-126F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x1p-126F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x1p-126F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1p-126F, INF },
    { 0x1.19999ap+0F, -0x1p-126F, -INF },
    { 0x1.19999ap+0F, -0x1p-126F, NAN },
    { 0x1.19999ap+0F, -0x1p-126F, NAN },
    { 0x1.19999ap+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1p-126F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1p-126F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, INF },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -INF },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, NAN },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, NAN },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, INF },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -INF },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, NAN },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, NAN },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, INF, 0x0p+0F },
    { 0x1.19999ap+0F, INF, -0x0p+0F },
    { 0x1.19999ap+0F, INF, 0x1p+0F },
    { 0x1.19999ap+0F, INF, -0x1p+0F },
    { 0x1.19999ap+0F, INF, 0x1p-126F },
    { 0x1.19999ap+0F, INF, -0x1p-126F },
    { 0x1.19999ap+0F, INF, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, INF, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, INF, INF },
    { 0x1.19999ap+0F, INF, -INF },
    { 0x1.19999ap+0F, INF, NAN },
    { 0x1.19999ap+0F, INF, NAN },
    { 0x1.19999ap+0F, INF, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, INF, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, INF, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, INF, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -INF, 0x0p+0F },
    { 0x1.19999ap+0F, -INF, -0x0p+0F },
    { 0x1.19999ap+0F, -INF, 0x1p+0F },
    { 0x1.19999ap+0F, -INF, -0x1p+0F },
    { 0x1.19999ap+0F, -INF, 0x1p-126F },
    { 0x1.19999ap+0F, -INF, -0x1p-126F },
    { 0x1.19999ap+0F, -INF, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -INF, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -INF, INF },
    { 0x1.19999ap+0F, -INF, -INF },
    { 0x1.19999ap+0F, -INF, NAN },
    { 0x1.19999ap+0F, -INF, NAN },
    { 0x1.19999ap+0F, -INF, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -INF, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -INF, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -INF, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, NAN, 0x0p+0F },
    { 0x1.19999ap+0F, NAN, -0x0p+0F },
    { 0x1.19999ap+0F, NAN, 0x1p+0F },
    { 0x1.19999ap+0F, NAN, -0x1p+0F },
    { 0x1.19999ap+0F, NAN, 0x1p-126F },
    { 0x1.19999ap+0F, NAN, -0x1p-126F },
    { 0x1.19999ap+0F, NAN, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, NAN, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, NAN, INF },
    { 0x1.19999ap+0F, NAN, -INF },
    { 0x1.19999ap+0F, NAN, NAN },
    { 0x1.19999ap+0F, NAN, NAN },
    { 0x1.19999ap+0F, NAN, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, NAN, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, NAN, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, NAN, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, NAN, 0x0p+0F },
    { 0x1.19999ap+0F, NAN, -0x0p+0F },
    { 0x1.19999ap+0F, NAN, 0x1p+0F },
    { 0x1.19999ap+0F, NAN, -0x1p+0F },
    { 0x1.19999ap+0F, NAN, 0x1p-126F },
    { 0x1.19999ap+0F, NAN, -0x1p-126F },
    { 0x1.19999ap+0F, NAN, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, NAN, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, NAN, INF },
    { 0x1.19999ap+0F, NAN, -INF },
    { 0x1.19999ap+0F, NAN, NAN },
    { 0x1.19999ap+0F, NAN, NAN },
    { 0x1.19999ap+0F, NAN, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, NAN, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, NAN, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, NAN, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, INF },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -INF },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, NAN },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, NAN },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x0p+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x0p+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x1p+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x1p+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x1p-126F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x1p-126F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, INF },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -INF },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, INF },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -INF },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, NAN },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, NAN },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x0p+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x0p+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x1p+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x1p+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x1p-126F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x1p-126F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, INF },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -INF },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, NAN },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { 0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x0p+0F, INF },
    { -0x1.ccccccp-1F, 0x0p+0F, -INF },
    { -0x1.ccccccp-1F, 0x0p+0F, NAN },
    { -0x1.ccccccp-1F, 0x0p+0F, NAN },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x0p+0F, INF },
    { -0x1.ccccccp-1F, -0x0p+0F, -INF },
    { -0x1.ccccccp-1F, -0x0p+0F, NAN },
    { -0x1.ccccccp-1F, -0x0p+0F, NAN },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1p+0F, INF },
    { -0x1.ccccccp-1F, 0x1p+0F, -INF },
    { -0x1.ccccccp-1F, 0x1p+0F, NAN },
    { -0x1.ccccccp-1F, 0x1p+0F, NAN },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1p+0F, INF },
    { -0x1.ccccccp-1F, -0x1p+0F, -INF },
    { -0x1.ccccccp-1F, -0x1p+0F, NAN },
    { -0x1.ccccccp-1F, -0x1p+0F, NAN },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1p-126F, INF },
    { -0x1.ccccccp-1F, 0x1p-126F, -INF },
    { -0x1.ccccccp-1F, 0x1p-126F, NAN },
    { -0x1.ccccccp-1F, 0x1p-126F, NAN },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1p-126F, INF },
    { -0x1.ccccccp-1F, -0x1p-126F, -INF },
    { -0x1.ccccccp-1F, -0x1p-126F, NAN },
    { -0x1.ccccccp-1F, -0x1p-126F, NAN },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, INF },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -INF },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, NAN },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, NAN },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, INF },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -INF },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, NAN },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, NAN },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, INF, 0x0p+0F },
    { -0x1.ccccccp-1F, INF, -0x0p+0F },
    { -0x1.ccccccp-1F, INF, 0x1p+0F },
    { -0x1.ccccccp-1F, INF, -0x1p+0F },
    { -0x1.ccccccp-1F, INF, 0x1p-126F },
    { -0x1.ccccccp-1F, INF, -0x1p-126F },
    { -0x1.ccccccp-1F, INF, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, INF, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, INF, INF },
    { -0x1.ccccccp-1F, INF, -INF },
    { -0x1.ccccccp-1F, INF, NAN },
    { -0x1.ccccccp-1F, INF, NAN },
    { -0x1.ccccccp-1F, INF, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, INF, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, INF, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, INF, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -INF, 0x0p+0F },
    { -0x1.ccccccp-1F, -INF, -0x0p+0F },
    { -0x1.ccccccp-1F, -INF, 0x1p+0F },
    { -0x1.ccccccp-1F, -INF, -0x1p+0F },
    { -0x1.ccccccp-1F, -INF, 0x1p-126F },
    { -0x1.ccccccp-1F, -INF, -0x1p-126F },
    { -0x1.ccccccp-1F, -INF, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -INF, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -INF, INF },
    { -0x1.ccccccp-1F, -INF, -INF },
    { -0x1.ccccccp-1F, -INF, NAN },
    { -0x1.ccccccp-1F, -INF, NAN },
    { -0x1.ccccccp-1F, -INF, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -INF, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -INF, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -INF, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, NAN, 0x0p+0F },
    { -0x1.ccccccp-1F, NAN, -0x0p+0F },
    { -0x1.ccccccp-1F, NAN, 0x1p+0F },
    { -0x1.ccccccp-1F, NAN, -0x1p+0F },
    { -0x1.ccccccp-1F, NAN, 0x1p-126F },
    { -0x1.ccccccp-1F, NAN, -0x1p-126F },
    { -0x1.ccccccp-1F, NAN, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, NAN, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, NAN, INF },
    { -0x1.ccccccp-1F, NAN, -INF },
    { -0x1.ccccccp-1F, NAN, NAN },
    { -0x1.ccccccp-1F, NAN, NAN },
    { -0x1.ccccccp-1F, NAN, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, NAN, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, NAN, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, NAN, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, NAN, 0x0p+0F },
    { -0x1.ccccccp-1F, NAN, -0x0p+0F },
    { -0x1.ccccccp-1F, NAN, 0x1p+0F },
    { -0x1.ccccccp-1F, NAN, -0x1p+0F },
    { -0x1.ccccccp-1F, NAN, 0x1p-126F },
    { -0x1.ccccccp-1F, NAN, -0x1p-126F },
    { -0x1.ccccccp-1F, NAN, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, NAN, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, NAN, INF },
    { -0x1.ccccccp-1F, NAN, -INF },
    { -0x1.ccccccp-1F, NAN, NAN },
    { -0x1.ccccccp-1F, NAN, NAN },
    { -0x1.ccccccp-1F, NAN, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, NAN, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, NAN, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, NAN, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, INF },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -INF },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, INF },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -INF },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, NAN },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, NAN },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, INF },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -INF },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, NAN },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, INF },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -INF },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, NAN },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, NAN },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.ccccccp-1F, -0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x0p+0F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x0p+0F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x0p+0F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x0p+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x0p+0F, INF },
    { -0x1.19999ap+0F, 0x0p+0F, -INF },
    { -0x1.19999ap+0F, 0x0p+0F, NAN },
    { -0x1.19999ap+0F, 0x0p+0F, NAN },
    { -0x1.19999ap+0F, 0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x0p+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x0p+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x0p+0F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x0p+0F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x0p+0F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x0p+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x0p+0F, INF },
    { -0x1.19999ap+0F, -0x0p+0F, -INF },
    { -0x1.19999ap+0F, -0x0p+0F, NAN },
    { -0x1.19999ap+0F, -0x0p+0F, NAN },
    { -0x1.19999ap+0F, -0x0p+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x0p+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x0p+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1p+0F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1p+0F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1p+0F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1p+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1p+0F, INF },
    { -0x1.19999ap+0F, 0x1p+0F, -INF },
    { -0x1.19999ap+0F, 0x1p+0F, NAN },
    { -0x1.19999ap+0F, 0x1p+0F, NAN },
    { -0x1.19999ap+0F, 0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1p+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1p+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1p+0F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x1p+0F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x1p+0F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x1p+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1p+0F, INF },
    { -0x1.19999ap+0F, -0x1p+0F, -INF },
    { -0x1.19999ap+0F, -0x1p+0F, NAN },
    { -0x1.19999ap+0F, -0x1p+0F, NAN },
    { -0x1.19999ap+0F, -0x1p+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1p+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1p+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1p-126F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1p-126F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1p-126F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1p-126F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1p-126F, INF },
    { -0x1.19999ap+0F, 0x1p-126F, -INF },
    { -0x1.19999ap+0F, 0x1p-126F, NAN },
    { -0x1.19999ap+0F, 0x1p-126F, NAN },
    { -0x1.19999ap+0F, 0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1p-126F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1p-126F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1p-126F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x1p-126F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x1p-126F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x1p-126F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1p-126F, INF },
    { -0x1.19999ap+0F, -0x1p-126F, -INF },
    { -0x1.19999ap+0F, -0x1p-126F, NAN },
    { -0x1.19999ap+0F, -0x1p-126F, NAN },
    { -0x1.19999ap+0F, -0x1p-126F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1p-126F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1p-126F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, INF },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -INF },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, NAN },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, NAN },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, INF },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -INF },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, NAN },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, NAN },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.fffffep+127F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, INF, 0x0p+0F },
    { -0x1.19999ap+0F, INF, -0x0p+0F },
    { -0x1.19999ap+0F, INF, 0x1p+0F },
    { -0x1.19999ap+0F, INF, -0x1p+0F },
    { -0x1.19999ap+0F, INF, 0x1p-126F },
    { -0x1.19999ap+0F, INF, -0x1p-126F },
    { -0x1.19999ap+0F, INF, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, INF, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, INF, INF },
    { -0x1.19999ap+0F, INF, -INF },
    { -0x1.19999ap+0F, INF, NAN },
    { -0x1.19999ap+0F, INF, NAN },
    { -0x1.19999ap+0F, INF, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, INF, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, INF, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, INF, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -INF, 0x0p+0F },
    { -0x1.19999ap+0F, -INF, -0x0p+0F },
    { -0x1.19999ap+0F, -INF, 0x1p+0F },
    { -0x1.19999ap+0F, -INF, -0x1p+0F },
    { -0x1.19999ap+0F, -INF, 0x1p-126F },
    { -0x1.19999ap+0F, -INF, -0x1p-126F },
    { -0x1.19999ap+0F, -INF, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -INF, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -INF, INF },
    { -0x1.19999ap+0F, -INF, -INF },
    { -0x1.19999ap+0F, -INF, NAN },
    { -0x1.19999ap+0F, -INF, NAN },
    { -0x1.19999ap+0F, -INF, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -INF, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -INF, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -INF, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, NAN, 0x0p+0F },
    { -0x1.19999ap+0F, NAN, -0x0p+0F },
    { -0x1.19999ap+0F, NAN, 0x1p+0F },
    { -0x1.19999ap+0F, NAN, -0x1p+0F },
    { -0x1.19999ap+0F, NAN, 0x1p-126F },
    { -0x1.19999ap+0F, NAN, -0x1p-126F },
    { -0x1.19999ap+0F, NAN, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, NAN, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, NAN, INF },
    { -0x1.19999ap+0F, NAN, -INF },
    { -0x1.19999ap+0F, NAN, NAN },
    { -0x1.19999ap+0F, NAN, NAN },
    { -0x1.19999ap+0F, NAN, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, NAN, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, NAN, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, NAN, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, NAN, 0x0p+0F },
    { -0x1.19999ap+0F, NAN, -0x0p+0F },
    { -0x1.19999ap+0F, NAN, 0x1p+0F },
    { -0x1.19999ap+0F, NAN, -0x1p+0F },
    { -0x1.19999ap+0F, NAN, 0x1p-126F },
    { -0x1.19999ap+0F, NAN, -0x1p-126F },
    { -0x1.19999ap+0F, NAN, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, NAN, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, NAN, INF },
    { -0x1.19999ap+0F, NAN, -INF },
    { -0x1.19999ap+0F, NAN, NAN },
    { -0x1.19999ap+0F, NAN, NAN },
    { -0x1.19999ap+0F, NAN, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, NAN, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, NAN, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, NAN, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, INF },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -INF },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, NAN },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, NAN },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x0p+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x0p+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x1p+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x1p+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x1p-126F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x1p-126F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, INF },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -INF },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, 0x1.19999ap+0F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, INF },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -INF },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, NAN },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, NAN },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.ccccccp-1F, -0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x0p+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x0p+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x1p+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x1p+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x1p-126F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x1p-126F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.fffffep+127F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, INF },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -INF },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, NAN },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, 0x1.19999ap+0F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.ccccccp-1F },
    { -0x1.19999ap+0F, -0x1.19999ap+0F, -0x1.19999ap+0F },

  };
  static struct {
    float e;

  } expected[] = {
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1p+0F } ,
    { 0x1p+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { 0x1p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e66666p+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.9999ap-4F } ,
    { -0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-4F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.0cccccp+1F } ,
    { 0x1p-126F } ,
    { 0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-125F } ,
    { 0x0p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { INF } ,
    { 0x0p+0F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x0p+0F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.e66666p+0F } ,
    { -0x1.9999ap-4F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { -0x1.9999ap-3F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1F } ,
    { 0x1.19999ap+1F } ,
    { 0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x1.9999ap-3F } ,
    { -0x1.ccccccp+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.9999ap-4F } ,
    { -0x1.0cccccp+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+1F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-4F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.0cccccp+1F } ,
    { 0x1p+0F } ,
    { 0x1p+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { 0x1p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e66666p+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.9999ap-4F } ,
    { -0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1p-126F } ,
    { 0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-125F } ,
    { 0x0p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x0p+0F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { INF } ,
    { 0x0p+0F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x1.9999ap-3F } ,
    { -0x1.ccccccp+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.9999ap-4F } ,
    { -0x1.0cccccp+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.e66666p+0F } ,
    { -0x1.9999ap-4F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { -0x1.9999ap-3F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1F } ,
    { 0x1.19999ap+1F } ,
    { 0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1p-126F } ,
    { 0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-125F } ,
    { 0x0p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.4p+2F } ,
    { 0x1.7ffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.399998p+2F } ,
    { 0x1.466666p+2F } ,
    { 0x1.8cccccp+1F } ,
    { 0x1.73333p+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.7ffffep+1F } ,
    { -0x1.4p+2F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8cccccp+1F } ,
    { -0x1.73333p+1F } ,
    { -0x1.399998p+2F } ,
    { -0x1.466666p+2F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-127F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.e66666p-126F } ,
    { -0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.19999ap-126F } ,
    { 0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.0cccccp-125F } ,
    { 0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.ccccccp-127F } ,
    { -0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.9999ap-130F } ,
    { -0x1.e66666p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap-126F } ,
    { -0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { -0x1.9999ap-130F } ,
    { -0x1.0cccccp-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x0p+0F } ,
    { -0x1p-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1p-126F } ,
    { 0x1p-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-125F } ,
    { 0x0p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.7ffffep+1F } ,
    { -0x1.4p+2F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8cccccp+1F } ,
    { -0x1.73333p+1F } ,
    { -0x1.399998p+2F } ,
    { -0x1.466666p+2F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.4p+2F } ,
    { 0x1.7ffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.399998p+2F } ,
    { 0x1.466666p+2F } ,
    { 0x1.8cccccp+1F } ,
    { 0x1.73333p+1F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.ccccccp-127F } ,
    { -0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.9999ap-130F } ,
    { -0x1.e66666p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap-126F } ,
    { -0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { -0x1.9999ap-130F } ,
    { -0x1.0cccccp-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.e66666p-126F } ,
    { -0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.19999ap-126F } ,
    { 0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.0cccccp-125F } ,
    { 0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { INF } ,
    { 0x0p+0F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x0p+0F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.4p+2F } ,
    { 0x1.7ffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.399998p+2F } ,
    { 0x1.466666p+2F } ,
    { 0x1.8cccccp+1F } ,
    { 0x1.73333p+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.7ffffep+1F } ,
    { -0x1.4p+2F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8cccccp+1F } ,
    { -0x1.73333p+1F } ,
    { -0x1.399998p+2F } ,
    { -0x1.466666p+2F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { -0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { 0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x0p+0F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { INF } ,
    { 0x0p+0F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.7ffffep+1F } ,
    { -0x1.4p+2F } ,
    { -0x1.fffffep+1F } ,
    { -0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.8cccccp+1F } ,
    { -0x1.73333p+1F } ,
    { -0x1.399998p+2F } ,
    { -0x1.466666p+2F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.4p+2F } ,
    { 0x1.7ffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.399998p+2F } ,
    { 0x1.466666p+2F } ,
    { 0x1.8cccccp+1F } ,
    { 0x1.73333p+1F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { 0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { -0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.e66666p+0F } ,
    { -0x1.9999ap-4F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { -0x1.9999ap-3F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x1.9999ap-3F } ,
    { -0x1.ccccccp+0F } ,
    { -0x1p+1F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.e66666p-126F } ,
    { -0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.ccccccp-127F } ,
    { -0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.9999ap-130F } ,
    { -0x1.e66666p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { -0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { 0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.cf5c28p+0F } ,
    { -0x1.851ebep-3F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.b5c28ep+0F } ,
    { 0x1.e8f5c2p+0F } ,
    { -0x1.70a3dcp-4F } ,
    { -0x1.28f5c8p-2F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fd70a4p+0F } ,
    { -0x1.47ae1ep-7F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70ap+0F } ,
    { 0x1.0b851ep+1F } ,
    { 0x1.70a3dcp-4F } ,
    { -0x1.c28f64p-4F } ,
    { -0x1.9eb85p-1F } ,
    { -0x1.9eb85p-1F } ,
    { 0x1.851ebep-3F } ,
    { -0x1.cf5c28p+0F } ,
    { -0x1.9eb85p-1F } ,
    { -0x1.9eb85p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.70a3dcp-4F } ,
    { 0x1.28f5c8p-2F } ,
    { -0x1.b5c28ep+0F } ,
    { -0x1.e8f5c2p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.47ae1ep-7F } ,
    { -0x1.fd70a4p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3dcp-4F } ,
    { 0x1.c28f64p-4F } ,
    { -0x1.e3d70ap+0F } ,
    { -0x1.0b851ep+1F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1F } ,
    { 0x1.19999ap+1F } ,
    { 0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.9999ap-4F } ,
    { -0x1.0cccccp+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+1F } ,
    { 0x1.19999ap-126F } ,
    { 0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.0cccccp-125F } ,
    { 0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap-126F } ,
    { -0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { -0x1.9999ap-130F } ,
    { -0x1.0cccccp-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fd70a4p+0F } ,
    { -0x1.47ae1ep-7F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70ap+0F } ,
    { 0x1.0b851ep+1F } ,
    { 0x1.70a3dcp-4F } ,
    { -0x1.c28f64p-4F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.1ae148p+1F } ,
    { 0x1.ae1482p-3F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.0e147cp+1F } ,
    { 0x1.27ae16p+1F } ,
    { 0x1.3d70a8p-2F } ,
    { 0x1.c28f64p-4F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.47ae1ep-7F } ,
    { -0x1.fd70a4p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3dcp-4F } ,
    { 0x1.c28f64p-4F } ,
    { -0x1.e3d70ap+0F } ,
    { -0x1.0b851ep+1F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.ae1482p-3F } ,
    { -0x1.1ae148p+1F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.35c29p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.3d70a8p-2F } ,
    { -0x1.c28f64p-4F } ,
    { -0x1.0e147cp+1F } ,
    { -0x1.27ae16p+1F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.9999ap-4F } ,
    { -0x1.e66666p+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x0p+0F } ,
    { 0x1.9999ap-3F } ,
    { -0x1.ccccccp+0F } ,
    { -0x1p+1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.e66666p+0F } ,
    { -0x1.9999ap-4F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp+0F } ,
    { 0x1p+1F } ,
    { 0x0p+0F } ,
    { -0x1.9999ap-3F } ,
    { -0x1.ccccccp-127F } ,
    { -0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.9999ap-130F } ,
    { -0x1.e66666p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1.ccccccp-127F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.e66666p-126F } ,
    { -0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { 0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { -0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { INF } ,
    { -0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { 0x1.cccccap+127F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9eb85p-1F } ,
    { -0x1.9eb85p-1F } ,
    { 0x1.851ebep-3F } ,
    { -0x1.cf5c28p+0F } ,
    { -0x1.9eb85p-1F } ,
    { -0x1.9eb85p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.70a3dcp-4F } ,
    { 0x1.28f5c8p-2F } ,
    { -0x1.b5c28ep+0F } ,
    { -0x1.e8f5c2p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.47ae1ep-7F } ,
    { -0x1.fd70a4p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3dcp-4F } ,
    { 0x1.c28f64p-4F } ,
    { -0x1.e3d70ap+0F } ,
    { -0x1.0b851ep+1F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.cf5c28p+0F } ,
    { -0x1.851ebep-3F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.9eb85p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.b5c28ep+0F } ,
    { 0x1.e8f5c2p+0F } ,
    { -0x1.70a3dcp-4F } ,
    { -0x1.28f5c8p-2F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fd70a4p+0F } ,
    { -0x1.47ae1ep-7F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70ap+0F } ,
    { 0x1.0b851ep+1F } ,
    { 0x1.70a3dcp-4F } ,
    { -0x1.c28f64p-4F } ,
    { 0x0p+0F } ,
    { -0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x0p+0F } ,
    { 0x0p+0F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1p-126F } ,
    { -0x1p-126F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.9999ap-4F } ,
    { -0x1.0cccccp+1F } ,
    { -0x1.19999ap+0F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1p+1F } ,
    { -0x1.19999ap+1F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.0cccccp+1F } ,
    { 0x1.9999ap-4F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.19999ap+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1p+1F } ,
    { 0x1.19999ap+1F } ,
    { 0x1.9999ap-3F } ,
    { 0x0p+0F } ,
    { -0x1.19999ap-126F } ,
    { -0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { -0x1.9999ap-130F } ,
    { -0x1.0cccccp-125F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { 0x1.19999ap-126F } ,
    { 0x1.19999ap-126F } ,
    { 0x1p+0F } ,
    { -0x1p+0F } ,
    { 0x1.0cccccp-125F } ,
    { 0x1.9999ap-130F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.ccccccp-1F } ,
    { 0x1.19999ap+0F } ,
    { -0x1.ccccccp-1F } ,
    { -0x1.19999ap+0F } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -0x1.99999ep+124F } ,
    { -INF } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { 0x1.99999ep+124F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { NAN } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { -INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { INF } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { NAN } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.47ae1ep-7F } ,
    { -0x1.fd70a4p+0F } ,
    { -0x1.fae148p-1F } ,
    { -0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.70a3dcp-4F } ,
    { 0x1.c28f64p-4F } ,
    { -0x1.e3d70ap+0F } ,
    { -0x1.0b851ep+1F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.ae1482p-3F } ,
    { -0x1.1ae148p+1F } ,
    { -0x1.35c29p+0F } ,
    { -0x1.35c29p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { -0x1.3d70a8p-2F } ,
    { -0x1.c28f64p-4F } ,
    { -0x1.0e147cp+1F } ,
    { -0x1.27ae16p+1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fd70a4p+0F } ,
    { -0x1.47ae1ep-7F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fae148p-1F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.e3d70ap+0F } ,
    { 0x1.0b851ep+1F } ,
    { 0x1.70a3dcp-4F } ,
    { -0x1.c28f64p-4F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.1ae148p+1F } ,
    { 0x1.ae1482p-3F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.35c29p+0F } ,
    { 0x1.fffffep+127F } ,
    { -0x1.fffffep+127F } ,
    { INF } ,
    { -INF } ,
    { NAN } ,
    { NAN } ,
    { 0x1.0e147cp+1F } ,
    { 0x1.27ae16p+1F } ,
    { 0x1.3d70a8p-2F } ,
    { 0x1.c28f64p-4F } ,

  };
  int i;
  int ok = 1;
  for (i = 0; i < sizeof(args) / sizeof(args[0]); ++i) {

    float result = fmaf(args[i].a

                                , args[i].a2

                                , args[i].a3


                                );

    if (!CMP(FLOAT, result, expected[i].e)

        ) {
      fprintf(stderr,
              "fmaf(%.8eF"

              ", %.8eF"

              ", %.8eF"


              ")\n",
              args[i].a

              , args[i].a2

              , args[i].a3

              );

      fprintf(stderr,
              "returned: %.8eF, expected: %.8eF\n",
              result,
              expected[i].e
              );

      ok = 0;
    }
  }
  TEST(ok);
}
